@@ -501,6 +501,9 @@ prolog_message(autoload(read_index(Dir))) -->
 		 *	TOPLEVEL MESSAGES	*
 		 *******************************/
 
+prolog_message(version) -->
+	{ current_prolog_flag(version_git, Version) }, !,
+	[ '~w'-[Version] ].
 prolog_message(version) -->
 	{ current_prolog_flag(version, Version),
 	  Major is Version // 10000,
@@ -562,7 +565,7 @@ prolog_message(query(QueryResult)) -->
 	query_result(QueryResult).
 
 query_result(no) -->		% failure
-	[ 'fail.' ],
+	[ 'false.' ],
 	extra_line.
 query_result(yes([])) --> !,	% prompt_alternatives_on: groundness
 	[ 'true.' ],
