@@ -49,6 +49,7 @@
 	    write_ln/1,
 	    proper_list/1,
 	    free_variables/2,		% +Term, -Variables
+	    hash_term/2,		% +Term, -Hash
 	    checklist/2,		% :Goal, +List
 	    sublist/3,			% :Goal, +List, -Sublist
 	    convert_time/2,		% +Stamp, -String
@@ -56,7 +57,9 @@
 	    'C'/3,			% +List, -Head, -Tail
 	    current_thread/2,		% ?Thread, ?Status
 	    current_mutex/3,		% ?Mutex, ?Owner, ?Count
-	    message_queue_size/2	% +Queue, -TermsWaiting
+	    message_queue_size/2,	% +Queue, -TermsWaiting
+	    lock_predicate/2,		% +Name, +Arity
+	    unlock_predicate/2		% +Name, +Arity
 	  ]).
 
 /** <module> Backward compatibility
@@ -181,6 +184,16 @@ proper_list(List) :-
 free_variables(Term, Variables) :-
 	term_variables(Term, Variables).
 
+%%	hash_term(+Term, -Hash) is det.
+%
+%	If Term is ground, Hash is unified to an integer representing
+%	a hash for Term.  Otherwise Hash is left unbound.
+%	
+%	@deprecated Use term_hash/2.
+
+hash_term(Term, Hash) :-
+	term_hash(Term, Hash).
+
 %%	checklist(:Goal, +List)
 %	
 %	@deprecated Use maplist/2
@@ -307,3 +320,19 @@ map_mutex_status(locked(Owner, Count), Owner, Count).
 
 message_queue_size(Queue, Size) :-
 	message_queue_property(Queue, size(Size)).
+
+%%	lock_predicate(+Name, +Arity) is det.
+%%	unlock_predicate(+Name, +Arity) is det.
+%
+%	@deprecated see lock_predicate/1 and unlock_predicate/1.
+
+:- module_transparent
+	lock_predicate/2,
+	unlock_predicate/2.
+
+lock_predicate(Name, Arity) :-
+	lock_predicate(Name/Arity).
+
+unlock_predicate(Name, Arity) :-
+	unlock_predicate(Name/Arity).
+
