@@ -36,7 +36,7 @@
 	  portray_clause/2		% +Stream, +Clause
 	]).
 :- use_module(library(lists), [member/2]).
-:- set_prolog_flag(generate_debug_info, false).
+%:- set_prolog_flag(generate_debug_info, false).
 
 :- module_transparent
 	listing/0, 
@@ -100,10 +100,10 @@ list_predicates(Preds, X) :-
 	context_module(Context),
 	member(Pred, Preds),
 	unify_args(Pred, X),
-	nl, 
 	'$define_predicate'(Pred),
 	strip_module(Pred, Module, Head), 
-        list_predicate(Module:Head, Context), 
+        list_predicate(Module:Head, Context),
+	nl,
         fail.
 list_predicates(_, _).
 
@@ -124,7 +124,7 @@ list_predicate(Pred, Context) :-
 	decl_term(Pred, Context, Decl),
 	format('%   Foreign: ~q~n', [Decl]).
 list_predicate(Pred, Context) :-
-	notify_changed(Pred),
+	notify_changed(Pred, Context),
 	list_declarations(Pred, Context), 
 	list_clauses(Pred, Context).
 
@@ -151,7 +151,7 @@ declaration(Pred, Source, Decl) :-
 	    
 list_declarations(Pred, Source) :-
 	findall(Decl, declaration(Pred, Source, Decl), Decls),
-	(   Decl == []
+	(   Decls == []
 	->  true
 	;   write_declarations(Decls, Source),
 	    format('~n', [])
@@ -165,27 +165,26 @@ write_declarations([H|T], Module) :-
 
 list_clauses(Pred, Source) :-
 	strip_module(Pred, Module, Head), 
-	clause(Pred, Body), 
+	(   clause(Pred, Body), 
 	    write_module(Module, Source), 
-	    portray_clause((Head:-Body)), 
-	fail.
+	    portray_clause((Head:-Body)),
+	    fail
+	;   true
+	).
 
 write_module(system, _) :- !.
 write_module(Module, Module) :- !.
 write_module(Module, _) :-
 	format('~q:', [Module]).
 
-notify_changed(Pred) :-
+notify_changed(Pred, Context) :-
 	strip_module(Pred, user, Head),
-	'$c_current_predicate'(_, system:Head),
-	\+ ( predicate_property(user:Head, imported_from(System)),
-	     (System == system ; import_module(System, system))
-	   ),
-	\+ predicate_property(system:Head, (dynamic)), !,
-	functor(Head, Name, Arity),
-	format('%   NOTE: system definition has been overruled for ~w/~w~n~n',
-	       [Name, Arity]).
-notify_changed(_).
+	predicate_property(Head, built_in),
+	\+ predicate_property(Head, (dynamic)), !,
+	decl_term(Pred, Context, Decl),
+	format('%   NOTE: system definition has been overruled for ~q~n',
+	       [Decl]).
+notify_changed(_, _).
 
 %%	portray_clause(+Clause) is det.
 %%	portray_clause(+Out:stream, +Clause) is det.
@@ -193,7 +192,8 @@ notify_changed(_).
 %	Portray `Clause' on the current  output   stream.  Layout of the
 %	clause is to our best standards.   As  the actual variable names
 %	are not available we use A, B, ... Deals with ';', '|', '->' and
-%	various calls via meta-call predicates.
+%	various calls via  meta-call  predicates.   If  Clause  contains
+%	attributed variables, these are treated as normal variables.
 
 %	The prolog_list_goal/1 hook is  a  dubious   as  it  may lead to
 %	confusion if the heads relates to other   bodies.  For now it is
@@ -208,30 +208,38 @@ portray_clause(Term) :-
 	portray_clause(Out, Term).
 
 portray_clause(Stream, Term) :-
-	\+ \+ ( numbervars(Term, 0, _, [singletons(true)]), 
-		do_portray_clause(Stream, Term)
+	\+ \+ ( copy_term_nat(Term, Copy),
+		numbervars(Copy, 0, _,
+			   [ singletons(true)
+			   ]), 
+		do_portray_clause(Stream, Copy)
 	      ).
 
+do_portray_clause(Out, Var) :-
+	var(Var), !,
+	pprint(Out, Var, 1200).
 do_portray_clause(Out, (Head :- true)) :- !, 
-	portray_head(Out, Head), 
-	put(Out, 0'.), nl(Out).
+	pprint(Out, Head, 1200),
+	full_stop(Out).
 do_portray_clause(Out, (Head :- Body)) :- !, 
 	inc_indent(0, 1, Indent),
-	portray_head(Out, Head), 
+	infix_op((:-), LeftPri, RightPri),
+	pprint(Out, Head, LeftPri),
 	write(Out, ' :-'), 
 	(   nonvar(Body),
 	    Body = Module:LocalBody
 	->  nlindent(Out, Indent),
-	    format(Out, '~q:', [Module]),
+	    format(Out, '~q', [Module]),
+	    '$put_token'(Out, :),
 	    nlindent(Out, Indent),
 	    write(Out, '(   '),
-	    portray_body(LocalBody, 2, noindent, Out),
+	    portray_body(LocalBody, 2, noindent, 1200, Out),
 	    nlindent(Out, Indent),
 	    write(Out, ')')
 	;   inc_indent(0, 2, BodyIndent),
-	    portray_body(Body, BodyIndent, indent, Out)
+	    portray_body(Body, BodyIndent, indent, RightPri, Out)
 	),
-	put(Out, 0'.), nl(Out).
+	full_stop(Out).
 do_portray_clause(Out, (:-use_module(File, Imports))) :-
 	length(Imports, Len),
 	Len > 3, !,
@@ -244,106 +252,199 @@ do_portray_clause(Out, (:-module(Module, Exports))) :- !,
 	write(Out, ').\n').
 do_portray_clause(Out, (:-Directive)) :- !,
 	write(Out, ':- '), 
-	portray_body(Directive, 3, noindent, Out),
-	write(Out, '.\n').
+	portray_body(Directive, 3, noindent, 1199, Out),
+	full_stop(Out).
 do_portray_clause(Out, Fact) :-
-	do_portray_clause(Out, (Fact :- true)).
+	portray_body(Fact, 0, noindent, 1200, Out),
+	full_stop(Out).
 
-portray_head(Out, Head) :-
-	pprint(Out, Head).
+full_stop(Out) :-
+	'$put_token'(Out, '.'),
+	nl(Out).
 
-%%	portray_body(+Term, +Indent, +DoIndent, +Out)
+
+%%	portray_body(+Term, +Indent, +DoIndent, +Priority, +Out)
 %	
 %	Write Term at current indentation. If   DoIndent  is 'indent' we
 %	must first call nlindent/2 before emitting anything.
 
-portray_body(!, _, _, Out) :- !, 
+portray_body(Var, _, _, Pri, Out) :-
+	var(Var), !,
+	pprint(Out, Var, Pri).
+portray_body(!, _, _, _, Out) :- !, 
 	write(Out, ' !').
-portray_body((!, Clause), Indent, _, Out) :- !, 
+portray_body((!, Clause), Indent, _, Pri, Out) :-
+	\+ term_needs_braces((_,_), Pri), !,
 	write(Out, ' !,'), 
-	portray_body(Clause, Indent, indent, Out).
-portray_body(Term, Indent, indent, Out) :- !, 
+	portray_body(Clause, Indent, indent, 1000, Out).
+portray_body(Term, Indent, indent, Pri, Out) :- !, 
 	nlindent(Out, Indent), 
-	portray_body(Term, Indent, noindent, Out).
-portray_body((A, B), Indent, _, Out) :- !, 
-	portray_body(A, Indent, noindent, Out), 
-	write(Out, ','), 
-	portray_body(B, Indent, indent, Out).
-portray_body(Or, Indent, _, Out) :-
-	memberchk(Or, [(_;_), (_|_), (_->_), (_*->_)]), !, 
+	portray_body(Term, Indent, noindent, Pri, Out).
+portray_body(Or, Indent, _, _, Out) :-
+	or_layout(Or), !,
 	write(Out, '(   '), 
-	portray_or(Or, Indent, Out), 
+	portray_or(Or, Indent, 1200, Out), 
 	nlindent(Out, Indent), 
 	write(Out, ')').
-portray_body(Meta, Indent, _, Out) :-
+portray_body(Term, Indent, _, Pri, Out) :-
+	term_needs_braces(Term, Pri), !,
+	write(Out, '(   '), 
+	portray_body(Term, Indent, noindent, 1200, Out), 
+	nlindent(Out, Indent), 
+	write(Out, ')').
+portray_body((A,B), Indent, _, _Pri, Out) :- !, 
+	infix_op((,), LeftPri, RightPri),
+	portray_body(A, Indent, noindent, LeftPri, Out), 
+	write(Out, ','), 
+	portray_body(B, Indent, indent, RightPri, Out).
+portray_body(\+(Goal), Indent, _, _Pri, Out) :- !,
+	write(Out, (\+)), write(Out, ' '),
+	prefix_op((\+), ArgPri),
+	ArgIndent is Indent+3,
+	portray_body(Goal, ArgIndent, noindent, ArgPri, Out).
+portray_body(Meta, Indent, _, Pri, Out) :-
 	meta_call(Meta, N), !, 
-	portray_meta(Out, Meta, N, Indent).
-portray_body(Clause, _, _, Out) :-
-	pprint(Out, Clause).
+	portray_meta(Out, Meta, N, Indent, Pri).
+portray_body(Clause, _, _, Pri, Out) :-
+	pprint(Out, Clause, Pri).
+
+term_needs_braces(Term, Pri) :-
+	callable(Term),
+	functor(Term, Name, _Arity),
+	current_op(OpPri, _Type, Name),
+	OpPri > Pri, !.
+
+%%	portray_or(+Term, +Indent, +Priority, +Out) is det.
+
+portray_or(Term, Indent, Pri, Out) :-
+	term_needs_braces(Term, Pri), !,
+	inc_indent(Indent, 1, NewIndent),
+	write(Out, '(   '),
+	portray_or(Term, NewIndent, Out),
+	nlindent(Out, NewIndent),
+	write(Out, ')').
+portray_or(Term, Indent, _Pri, Out) :-
+	or_layout(Term), !,
+	portray_or(Term, Indent, Out).
+portray_or(Term, Indent, Pri, Out) :-
+	inc_indent(Indent, 1, NestIndent),
+	portray_body(Term, NestIndent, noindent, Pri, Out).
+
 
 portray_or((If -> Then ; Else), Indent, Out) :- !, 
 	inc_indent(Indent, 1, NestIndent),
-	portray_body(If, NestIndent, noindent, Out), 	
+	infix_op((->), LeftPri, RightPri),
+	portray_body(If, NestIndent, noindent, LeftPri, Out), 	
 	nlindent(Out, Indent),
 	write(Out, '->  '), 
-	portray_body(Then, NestIndent, noindent, Out), 
+	portray_body(Then, NestIndent, noindent, RightPri, Out), 
 	nlindent(Out, Indent), 
 	write(Out, ';   '), 
-	portray_or(Else, Indent, Out).
+	infix_op(;, _LeftPri, RightPri2),
+	portray_or(Else, Indent, RightPri2, Out).
 portray_or((If *-> Then ; Else), Indent, Out) :- !, 
 	inc_indent(Indent, 1, NestIndent),
-	portray_body(If, NestIndent, noindent, Out), 	
+	infix_op((*->), LeftPri, RightPri),
+	portray_body(If, NestIndent, noindent, LeftPri, Out), 	
 	nlindent(Out, Indent),
 	write(Out, '*-> '), 
-	portray_body(Then, NestIndent, noindent, Out), 
+	portray_body(Then, NestIndent, noindent, RightPri, Out), 
 	nlindent(Out, Indent), 
 	write(Out, ';   '), 
-	portray_or(Else, Indent, Out).
+	infix_op(;, _LeftPri, RightPri2),
+	portray_or(Else, Indent, RightPri2, Out).
 portray_or((If -> Then), Indent, Out) :- !, 
 	inc_indent(Indent, 1, NestIndent),
-	portray_body(If, NestIndent, noindent, Out), 	
+	infix_op((->), LeftPri, RightPri),
+	portray_body(If, NestIndent, noindent, LeftPri, Out), 	
 	nlindent(Out, Indent), 
 	write(Out, '->  '), 
-	portray_or(Then, Indent, Out).
+	portray_or(Then, Indent, RightPri, Out).
 portray_or((If *-> Then), Indent, Out) :- !, 
 	inc_indent(Indent, 1, NestIndent),
-	portray_body(If, NestIndent, noindent, Out), 	
+	infix_op((->), LeftPri, RightPri),
+	portray_body(If, NestIndent, noindent, LeftPri, Out), 	
 	nlindent(Out, Indent), 
 	write(Out, '*-> '), 
-	portray_or(Then, Indent, Out).
+	portray_or(Then, Indent, RightPri, Out).
 portray_or((A;B), Indent, Out) :- !, 
 	inc_indent(Indent, 1, NestIndent),
-	portray_body(A, NestIndent, noindent, Out), 
+	infix_op((;), LeftPri, RightPri),
+	portray_body(A, NestIndent, noindent, LeftPri, Out), 
 	nlindent(Out, Indent), 
 	write(Out, ';   '), 
-	portray_or(B, Indent, Out).
+	portray_or(B, Indent, RightPri, Out).
 portray_or((A|B), Indent, Out) :- !, 
 	inc_indent(Indent, 1, NestIndent),
-	portray_body(A, NestIndent, noindent, Out), 	
+	infix_op((|), LeftPri, RightPri),
+	portray_body(A, NestIndent, noindent, LeftPri, Out), 	
 	nlindent(Out, Indent), 
 	write(Out, '|   '), 
-	portray_or(B, Indent, Out).
-portray_or(A, Indent, Out) :-
-	inc_indent(Indent, 1, NestIndent),
-	portray_body(A, NestIndent, noindent, Out).
+	portray_or(B, Indent, RightPri, Out).
+
+
+%%	infix_op(+Op, -Left, -Right) is semidet.
+%
+%	True if Op is an infix operator and Left is the max priority of its
+%	left hand and Right is the max priority of its right hand.
+
+infix_op(Op, Left, Right) :-
+	current_op(Pri, Assoc, Op),
+	infix_assoc(Assoc, LeftMin, RightMin), !,
+	Left is Pri - LeftMin,
+	Right is Pri - RightMin.
+
+infix_assoc(xfx, 1, 1).
+infix_assoc(xfy, 1, 0).
+infix_assoc(yfx, 0, 1).
+infix_assoc(yfy, 0, 0).
+
+prefix_op(Op, ArgPri) :-
+	current_op(Pri, Assoc, Op),
+	pre_assoc(Assoc, ArgMin), !,
+	ArgPri is Pri - ArgMin.
+
+pre_assoc(fx, 1).
+pre_assoc(fy, 0).
+
+%%	or_layout(@Term) is semidet.
+%
+%	True if Term is a control structure for which we want to use clean
+%	layout.
+%	
+%	@tbd	Change name.
+
+or_layout(Var) :-
+	var(Var), !, fail.
+or_layout((_;_)).
+or_layout((_->_)).
+or_layout((_*->_)).
+
+%%	meta_call(+Goal, -Arg) is semidet.
+%
+%	True if Goal is a meta-predicate for which we wish to format the
+%	arguments.
+%	
+%	@tbd	This really must be  synchronised with meta_predicate
+%		to fix this.
 
 meta_call(call(_), 1).
 meta_call(once(_), 1).
 meta_call(not(_), 1).
-meta_call(\+(_), 1).
 meta_call(ignore(_), 1).
 
-portray_meta(Out, Term, N, Indent) :-
+portray_meta(Out, Term, N, Indent, _Pri) :-
 	arg(N, Term, Arg), 
-	memberchk(Arg, [(_, _), (_;_), (_->_), (_*->_)]), !, 
+	or_layout(Arg), !,
 	functor(Term, Name, _), 
-	write(Out, Name), write(Out, '(('), 
-	inc_indent(Indent, 1, NestIndent),
-	portray_body(Arg, NestIndent, indent, Out), 
-	nlindent(Out, NestIndent), 
+	format(Out, '~q((   ', [Name]),
+	atom_length(Name, Alen),
+	BraceIndent is Indent+Alen+1,
+	portray_or(Arg, BraceIndent, Out), 
+	nlindent(Out, BraceIndent), 
 	write(Out, '))').	
-portray_meta(Out, Term, _, _) :-
-	pprint(Out, Term).	
+portray_meta(Out, Term, _, _, Pri) :-
+	pprint(Out, Term, Pri).	
 
 %%	portray_list(+List, +Indent, +Out)
 %	
@@ -364,7 +465,7 @@ portray_list(List, Indent, Out) :-
 	write(Out, ']').
 
 portray_list_elements([H|T], EIndent, Out) :-
-	pprint(Out, H),
+	pprint(Out, H, 999),
 	(   T == []
 	->  true
 	;   nonvar(T), T = [_|_]
@@ -374,7 +475,7 @@ portray_list_elements([H|T], EIndent, Out) :-
 	;   Indent is EIndent - 2,
 	    nlindent(Out, Indent),
 	    write(Out, '| '),
-	    pprint(Out, T)
+	    pprint(Out, T, 999)
 	).
 
 %%	nlindent(+Out, +Indent)
@@ -403,12 +504,13 @@ put_tabs(_, _).
 inc_indent(Indent0, Inc, Indent) :-
 	Indent is Indent0 + Inc*4.
 
-%%	pprint(+Out, +Term)
+%%	pprint(+Out, +Term, +Priority)
 %	
 %	Print Term such that it can be read.
 
-pprint(Out, Term) :-
+pprint(Out, Term, Pri) :-
 	write_term(Out, Term,
 		   [ quoted(true),
-		     numbervars(true)
+		     numbervars(true),
+		     priority(Pri)
 		   ]).
