@@ -206,6 +206,7 @@ tnodebug(ThreadID) :-
 %	Profile the operation of Thread until the user hits a key.
 
 tprofile(Thread) :-
+	init_pce,
 	thread_signal(Thread, 
 		      (	  reset_profiler,
 			  profiler(_, true)
@@ -220,6 +221,17 @@ tprofile(Thread) :-
 		      )).
 
 
+%%	init_pce
+%
+%	Make sure XPCE is running if it is   attached, so we can use the
+%	graphical display using in_pce_thread/1.
+
+init_pce :-
+	current_prolog_flag(gui, true), !,
+	call(send(@(display), open)).	% avoid autoloading
+init_pce.
+
+
 		 /*******************************
 		 *	       HOOKS		*
 		 *******************************/
