@@ -29,8 +29,7 @@
     the GNU General Public License.
 */
 :- module(chr_compiler_utility,
-	[ is_variant/2
-	, time/2
+	[ time/2
 	, replicate/3
 	, pair_all_with/3
 	, conj2list/2
@@ -52,37 +51,22 @@
 	, set_elems/2
 	, instrument_goal/4
 	, sort_by_key/3
+	, arg1/3
+	, wrap_in_functor/3
+	, tree_set_empty/1
+	, tree_set_memberchk/2
+	, tree_set_add/3
 	]).
 
 :- use_module(pairlist).
 :- use_module(library(lists), [permutation/2]).
+:- use_module(library(assoc)).
 
 %% SICStus begin
 %% use_module(library(terms),[term_variables/2]).
 %% SICStus end
 
 
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-is_variant(A,B) :-
-	copy_term_nat(A,AC),
-	copy_term_nat(B,BC),
-	term_variables(AC,AVars), 
-	term_variables(BC,BVars),
-	AC = BC,
-	is_variant1(AVars),
-	is_variant2(BVars).
-
-is_variant1([]).
-is_variant1([X|Xs]) :-
-	var(X),
-	X = '$test',
-	is_variant1(Xs).
-	
-is_variant2([]).
-is_variant2([X|Xs]) :-
-	X == '$test',
-	is_variant2(Xs).
-
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % time(Phase,Goal) :-
 % 	statistics(runtime,[T1|_]),
@@ -300,3 +284,31 @@ sort_by_key(List,Keys,SortedList) :-
 	pairup(Keys,List,Pairs),
 	sort(Pairs,SortedPairs),
 	once(pairup(_,SortedList,SortedPairs)).	
+
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+arg1(Term,Index,Arg) :- arg(Index,Term,Arg).	
+
+wrap_in_functor(Functor,X,Term) :-
+	Term =.. [Functor,X].
+
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
+tree_set_empty(TreeSet) :- empty_assoc(TreeSet).
+tree_set_memberchk(Element,TreeSet) :- get_assoc(Element,TreeSet,_).
+tree_set_add(TreeSet,Element,NTreeSet) :- put_assoc(Element,TreeSet,x,NTreeSet).
+
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+:- dynamic
+	user:goal_expansion/2.
+:- multifile
+	user:goal_expansion/2.
+
+user:goal_expansion(arg1(Term,Index,Arg), arg(Index,Term,Arg)).
+user:goal_expansion(wrap_in_functor(Functor,In,Out), Goal) :-
+	( atom(Functor), var(Out) ->
+		Out =.. [Functor,In],
+		Goal = true
+	;
+		Goal = (Out =.. [Functor,In])
+	).
+
