@@ -1,11 +1,11 @@
-/*  $Id: form.c,v 1.4 2004/09/21 14:22:23 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2007, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -35,7 +35,7 @@
 #endif
 #include <string.h>
 #include "form.h"
-#ifdef WIN32
+#ifdef __WINDOWS__
 #include <io.h>
 #endif
 
@@ -59,10 +59,10 @@ dehex(int chr)
 }
 
 
-static int
-form_argument_decode(const char *in, int inlen, char *out, int outlen)
+static size_t
+form_argument_decode(const char *in, size_t inlen, char *out, size_t outlen)
 { const char *ein  = in+inlen;
-  int written = 0;
+  size_t written = 0;
 
   for(; in < ein; in++)
   { switch(*in)
@@ -76,12 +76,12 @@ form_argument_decode(const char *in, int inlen, char *out, int outlen)
 	  int h2 = dehex(*(++in));
 
 	  if ( h1 < 0 || h2 < 0 )
-	    return -1;
+	    return (size_t)-1;
 	  
 	  if ( ++written < outlen )
 	    *out++ = h1<<4|h2;
 	} else
-	  return -1;			/* syntax error */
+	  return (size_t)-1;		/* syntax error */
 	break;
       default:
 	if ( ++written < outlen )
@@ -109,9 +109,9 @@ break_form_argument(const char *formdata,
     char *eq = strchr(formdata, '=');
 
     if ( eq )
-    { int len = eq-formdata;
+    { size_t len = eq-formdata;
       char *end;
-      int vlen;
+      size_t vlen;
 
       if ( len > MAXNAME-1 )
 	return ERROR_NAME_TOO_LONG;
@@ -125,7 +125,7 @@ break_form_argument(const char *formdata,
 
       if ( (vlen=form_argument_decode(eq, end-eq, value, MAXVALUE)) >= MAXVALUE )
 	return ERROR_VALUE_TOO_LONG;
-      if ( vlen < 0 )
+      if ( vlen == (size_t)-1 )
 	return ERROR_SYNTAX_ERROR;
 
       (func)(name, value, closure);
@@ -143,7 +143,7 @@ break_form_argument(const char *formdata,
 
 static char *
 find_boundary(const char *data, const char *end, const char *boundary)
-{ int blen = strlen(boundary);
+{ size_t blen = strlen(boundary);
 
   while ( data < end &&
 	  !(strncmp(data, boundary, blen) == 0) )
@@ -166,7 +166,7 @@ Find a named attribute in a mime header of a multipart form
 static char *
 attribute_of_multipart_header(const char *name, char *header, char *endheader)
 { char *value;
-  int nlen = strlen(name);
+  size_t nlen = strlen(name);
 
   while( header < endheader &&
 	 !(header[nlen] == '=' && strncmp(header, name, nlen) == 0) )
@@ -224,11 +224,11 @@ next_line(const char *in)
 
 
 int
-break_multipart(char *formdata, int len,
+break_multipart(char *formdata, size_t len,
 		const char *boundary,
 		int (*func)(const char *name,
 			    const char *value,
-			    int valuelen,
+			    size_t valuelen,
 			    const char *filename,
 			    void *closure),
 		void *closure)
@@ -290,7 +290,7 @@ for lenp is the maximum acceptable content-length.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 char *
-get_raw_form_data(int *lenp)
+get_raw_form_data(size_t *lenp)
 { char *method;
   char *s;
 
@@ -298,13 +298,20 @@ get_raw_form_data(int *lenp)
        strcmp(method, "POST") == 0 )
   { char *lenvar = getenv("CONTENT_LENGTH");
     char *q;
-    int len;
+    long len;
 
     if ( !lenvar )
       return NULL;
-    len = atoi(lenvar);
+    len = atol(lenvar);
+    if ( len < 0 )
+    {
+#ifdef UTIL_H_INCLUDED
+	error("Negative content length");
+#endif
+	return NULL;
+    }
     if ( lenp )
-    { if ( *lenp && len > *lenp )
+    { if ( *lenp && (size_t)len > *lenp )
       {
 #ifdef UTIL_H_INCLUDED
 	error("Contents too long (accept max. %d Kbytes)", *lenp/1024);
