@@ -1,4 +1,4 @@
-/*  $Id: mime.c,v 1.11 2005/10/10 13:24:20 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -26,7 +26,7 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 #define HAVE_MALLOC_H 1
 #endif
 
@@ -249,7 +249,7 @@ get_character_data(term_t from, char **data, size_t *len, int *malloced)
 { atom_t name;
   int arity;
   char *buf;
-  unsigned int size;
+  size_t size;
 
   if ( PL_get_name_arity(from, &name, &arity) && arity > 0 )
   { if ( name == ATOM_stream )
