@@ -1,11 +1,11 @@
-/*  $Id: time.c,v 1.23 2006/01/19 20:33:37 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
     E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2006, University of Amsterdam
+    Copyright (C): 1985-2007, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -52,14 +52,17 @@
 #include <pthread.h>
 #endif
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-The WIN32 port uses the multimedia timers.   This  module must be linked
+The __WINDOWS__ port uses the multimedia timers.   This  module must be linked
 with winmm.lib
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #include <windows.h>
 #include <sys/timeb.h>
+#if (_MSC_VER < 1400) 
+typedef DWORD DWORD_PTR;
+#endif
 
 #ifndef SIGALRM
 #define SIGALRM 14
@@ -81,14 +84,14 @@ gettimeofday(struct timeval *tv, struct timezone *tz)
 { struct timeb tb;
 
   ftime(&tb);
-  tv->tv_sec  = tb.time;
+  tv->tv_sec  = (long)tb.time;
   tv->tv_usec = tb.millitm * 1000;
 
   return 0;
 }
 
 
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 
 #ifdef _REENTRANT
 #define SHARED_TABLE 1
@@ -97,7 +100,7 @@ gettimeofday(struct timeval *tv, struct timezone *tz)
 #include <time.h>
 #include <sys/time.h>
 
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 #ifdef O_DEBUG
 static int debuglevel = 0;
@@ -193,6 +196,7 @@ static void	on_alarm(int sig);
 
 static module_t	   MODULE_user;
 static atom_t	   ATOM_remove;
+static atom_t	   ATOM_install;
 static atom_t	   ATOM_done;
 static atom_t	   ATOM_next;
 static atom_t	   ATOM_scheduled;
@@ -206,6 +210,7 @@ static predicate_t PREDICATE_call1;
 #define EV_DONE		0x0001		/* Handled this one */
 #define EV_REMOVE	0x0002		/* Automatically remove */
 #define EV_FIRED	0x0004		/* Windows: got this one */
+#define EV_NOINSTALL	0x0008		/* Only allocate; do not install */
 
 typedef struct event
 { record_t	 goal;			/* Thing to call */
@@ -214,6 +219,7 @@ typedef struct event
   struct event  *previous;		/* idem */
   unsigned long  flags;			/* misc flags */
   long		 magic;			/* validate magic */
+  double	 time;			/* Specified time */
   struct timeval at;			/* Time to deliver */
 #ifdef SHARED_TABLE
   pthread_t	 thread_id;		/* Thread to call in */
@@ -221,7 +227,7 @@ typedef struct event
   int		 pl_thread_id;		/* Prolog thread ID */
 #endif
 #endif
-#ifdef WIN32
+#ifdef __WINDOWS__
   UINT		 mmid;			/* MultiMedia timer id */
   DWORD		 tid;			/* thread-id of Prolog thread */
 #endif
@@ -328,7 +334,7 @@ insertEvent(Event ev)
 	e->previous->next = ev;
       e->previous = ev;
 
-      if ( sched->first == e )			/* allocated as first */
+      if ( sched->first == e )		/* allocated as first */
 	sched->first = ev;
 
       return;
@@ -390,7 +396,7 @@ callEvent(Event ev)
 static void
 cleanupHandler()
 { 
-#ifndef WIN32
+#ifndef __WINDOWS__
   struct itimerval v;
 
   DEBUG(1, Sdprintf("Removed timer\n"));
@@ -428,7 +434,7 @@ cleanup()
 }
 
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 
 static void
 on_alarm(int sig)
@@ -453,7 +459,7 @@ on_alarm(int sig)
 
 
 static void CALLBACK
-callTimer(UINT id, UINT msg, DWORD dwuser, DWORD dw1, DWORD dw2)
+callTimer(UINT id, UINT msg, DWORD_PTR dwuser, DWORD_PTR dw1, DWORD_PTR dw2)
 { Event ev = (Event)dwuser;
 
   ev->flags |= EV_FIRED;
@@ -462,12 +468,12 @@ callTimer(UINT id, UINT msg, DWORD dwuser, DWORD dw1, DWORD dw2)
 
 
 static int
-installEvent(Event ev, double t)
+installEvent(Event ev)
 { MMRESULT rval;
 
   insertEvent(ev);
 
-  rval = timeSetEvent((int)(t*1000),
+  rval = timeSetEvent((int)(ev->time*1000),
 		      50,			/* resolution (milliseconds) */
 		      callTimer,
 		      (DWORD)ev,
@@ -498,7 +504,7 @@ uninstallEvent(Event ev)
 }
 
 
-#else /*WIN32*/
+#else /*__WINDOWS__*/
 
 #ifdef SHARED_TABLE
 
@@ -521,7 +527,8 @@ static void *
 alarm_loop(void * closure)
 { schedule *sched = TheSchedule();
 
-  LOCK();
+  pthread_mutex_lock(&mutex);		/* for condition variable */
+
   for(;;)
   { Event ev = nextEvent(sched);
 
@@ -543,8 +550,11 @@ alarm_loop(void * closure)
 	  ev->flags |= EV_FIRED;
 	  pthread_kill(ev->thread_id, SIGALRM);
 	  break;
+	case 0:
 	case EINTR:
 	  continue;
+	default:
+	  Sdprintf("alarm/4: pthread_cond_timedwait(): %s\n", strerror(rc));
       }
     } else
     { int rc = pthread_cond_wait(&cond, &mutex);
@@ -553,7 +563,6 @@ alarm_loop(void * closure)
 	continue;
     }
   }
-  UNLOCK();
 
   return NULL;
 }
@@ -604,7 +613,7 @@ on_alarm(int sig)
 
 
 static int
-installEvent(Event ev, double t)
+installEvent(Event ev)
 { LOCK();
 
   ev->thread_id = pthread_self();
@@ -727,7 +736,7 @@ on_alarm(int sig)
 
 
 static int
-installEvent(Event ev, double t)
+installEvent(Event ev)
 { insertEvent(ev);
   re_schedule();
 
@@ -746,7 +755,7 @@ uninstallEvent(Event ev)
 }
 
 #endif /*SHARED_TABLE*/
-#endif /*WIN32*/
+#endif /*__WINDOWS__*/
 
 
 		 /*******************************
@@ -827,7 +836,14 @@ alarm4(term_t time, term_t callable, term_t id, term_t options)
 	      return FALSE;
 	    if ( t )
 	      flags |= EV_REMOVE;
-	  }	    
+	  } else if ( name == ATOM_install )
+	  { int t = TRUE;
+
+	    if ( !pl_get_bool_ex(arg, &t) )
+	      return FALSE;
+	    if ( !t )
+	      flags |= EV_NOINSTALL;
+	  }
 	}
       }
     }
@@ -842,13 +858,14 @@ alarm4(term_t time, term_t callable, term_t id, term_t options)
   gettimeofday(&tv, NULL);
   tv.tv_usec += (long)((t-floor(t))*1000000);
   tv.tv_sec  += (long)t;
-  if ( tv.tv_usec > 1000000 )
+  if ( tv.tv_usec >= 1000000 )
   { tv.tv_usec -= 1000000;
     tv.tv_sec++;
   }
 
   if ( !(ev = allocEvent(&tv)) )
     return FALSE;
+  ev->time = t;
   if ( !unify_timer(id, ev) )
   { freeEvent(ev);
     return FALSE;
@@ -859,9 +876,11 @@ alarm4(term_t time, term_t callable, term_t id, term_t options)
   ev->module = m;
   ev->goal = PL_record(callable);
 
-  if ( !installEvent(ev, t) )
-  { freeEvent(ev);
-    return FALSE;
+  if ( !(ev->flags & EV_NOINSTALL) )
+  { if ( !installEvent(ev) )
+    { freeEvent(ev);
+      return FALSE;
+    }
   }
 
   return TRUE;
@@ -874,7 +893,23 @@ alarm3(term_t time, term_t callable, term_t id)
 }
 
 
-foreign_t
+static foreign_t
+install_alarm(term_t alarm)
+{ Event ev = NULL;
+
+  if ( !get_timer(alarm, &ev) )
+    return FALSE;
+
+  if ( !installEvent(ev) )
+  { freeEvent(ev);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+
+static foreign_t
 remove_alarm(term_t alarm)
 { Event ev = NULL;
 
@@ -887,7 +922,7 @@ remove_alarm(term_t alarm)
 }
 
 
-foreign_t
+static foreign_t
 current_alarms(term_t time, term_t goal, term_t id, term_t status,
 	       term_t matching)
 { Event ev;
@@ -976,6 +1011,7 @@ install()
   FUNCTOR_module2 = PL_new_functor(PL_new_atom(":"), 2);
 
   ATOM_remove	  = PL_new_atom("remove");
+  ATOM_install	  = PL_new_atom("install");
   ATOM_done	  = PL_new_atom("done");
   ATOM_next	  = PL_new_atom("next");
   ATOM_scheduled  = PL_new_atom("scheduled");
@@ -985,6 +1021,8 @@ install()
   PL_register_foreign("alarm",          4, alarm4,         PL_FA_TRANSPARENT);
   PL_register_foreign("alarm",          3, alarm3,         PL_FA_TRANSPARENT);
   PL_register_foreign("remove_alarm",   1, remove_alarm,   0);
+  PL_register_foreign("install_alarm",  1, install_alarm,  0);
+  PL_register_foreign("remove_alarm_notrace",1, remove_alarm,   PL_FA_NOTRACE);
   PL_register_foreign("current_alarms", 5, current_alarms, 0);
 #ifdef O_DEBUG
   PL_register_foreign("time_debug",	1, pl_time_debug,  0);
