@@ -1,11 +1,11 @@
-/*  $Id: time.pl,v 1.8 2006/01/13 14:33:52 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2007, University of Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -33,13 +33,15 @@
 	  [ alarm/3,			% +Time, :Callable, -Id
 	    alarm/4,			% +Time, :Callable, -Id, +Options
 	    remove_alarm/1,		% +Id
+	    install_alarm/1,		% +Id
 	    current_alarm/4,		% ?At, ?:Goal, ?Id, ?Status
 	    call_with_time_limit/2	% +Time, :Callable
 	  ]).
 :- use_module(library(lists)).
+:- set_prolog_flag(generate_debug_info, false).
 
-%	alarm(+Time, :Callable, -Id)
-%	alarm(+Time, :Callable, -Id, +Options)
+%%	alarm(+Time, :Callable, -Id) is det.
+%%	alarm(+Time, :Callable, -Id, +Options) is det.
 %	
 %	Set up an alarm to be signaled Time seconds from now. If the
 %	alarm expires, Callable is called asynchronously. Callable can
@@ -54,14 +56,28 @@
 %	options are:
 %	
 %		* remove(Bool)
-%		If Bool is true, remove the alarm-event (as
+%		If =true= (default =false=), remove the alarm-event (as
 %		remove_alarm/1) after it has been fired.
+%		* install(Bool)
+%		If =false= (default =true=) do not install the alarm.
+%		It must be installed seperately using install_alarm/1.
 
-%	remove_alarm(+Id)
+%%	install_alarm(+Id) is det.
+%
+%	Install an alarm allocated using alarm(Time, Goal, Id,
+%	[install(false)]).  Typically used with call_cleanup/2:
+%	
+%	==
+%		alarm(Time, Goal1, Id, [install(false)]),
+%		call_cleanup((install_alarm(Id), Goal2),
+%			     remove_alarm(Id)),
+%	==
+
+%%	remove_alarm(+Id) is det.
 %	
 %	Remove an alarm.  If it has not yet been fired, it never will.
 
-%	current_alarm(?Time, ?:Goal, ?Id, ?Status)
+%%	current_alarm(?Time, :Goal, ?Id, ?Status) is nondet.
 %	
 %	Enumerate the alarms in the schedule.  Time is the absolute time
 %	the event is scheduled for (see also get_time/1). Goal is the
@@ -73,17 +89,29 @@
 :- initialization
    load_foreign_library(foreign(time)).
 
-:- meta_predicate(call_with_time_limit(+, :)).
+:- module_transparent
+	call_with_time_limit/2.
 
-%	call_with_time_limit(+Time, :Goal)
+%%	call_with_time_limit(+Time, :Goal) is det.
 %	
-%	Call :Goal, while watching out for a (real-time) limit.  If this
-%	limit is exceeded, the exception time_limit_exceeded is raised.
+%	Call Goal, while watching out for   a (wall-time) limit. If this
+%	limit  is  exceeded,  the   exception  =time_limit_exceeded=  is
+%	raised. Goal is called as in once/1.
+%
+%	@throws =time_limit_exceeded=
 
 call_with_time_limit(Time, Goal) :-
-	alarm(Time, throw(time_limit_exceeded), Id),
-	call_cleanup(once(Goal), time:remove_alarm(Id)).
+	Time > 0, !,
+	strip_module(Goal, M, G),
+	call_with_time_limit2(Time, M:G).
+call_with_time_limit(_Time, _Goal) :-
+	throw(time_limit_exceeded).
 		     
+call_with_time_limit2(Time, Goal) :-
+	setup_and_call_cleanup(alarm(Time, throw(time_limit_exceeded), Id),
+			       once(Goal),
+			       remove_alarm_notrace(Id)).
+
 current_alarm(Time, Goal, Id, Status) :-
 	current_alarms(Time, Goal, Id, Status, List),
 	member(alarm(Time, Goal, Id, Status), List).
