@@ -117,9 +117,11 @@ static time_t timegm(struct tm *tm);
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Work around bug in MS SQL Server that reports NumCols of SQLColumns()
 as 19, while there are only 12.  Grrr!
+
+This bug appears fixed now, so we'll remove the work-around
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define SQL_SERVER_BUG	1
+/*#define SQL_SERVER_BUG	1*/
 
 static atom_t    ATOM_row;		/* "row" */
 static atom_t    ATOM_informational;	/* "informational" */
@@ -1947,30 +1949,27 @@ prepare_result(context *ctxt)
     switch (ptr_result->sqlTypeID)
     { case SQL_LONGVARCHAR:
       case SQL_LONGVARBINARY:
-      { if ( (int)columnSize > ctxt->max_nogetdata )
-	{ DEBUG(2,
-		Sdprintf("Wide SQL_LONGVAR* column %d: using SQLGetData()\n",
-			 i));
+      { if ( (int)columnSize > ctxt->max_nogetdata || columnSize == 0 )
+	{ use_sql_get_data:
+	  DEBUG(2,
+		Sdprintf("Wide SQL_LONGVAR* column %d: using SQLGetData()\n", i));
 	  ptr_result->ptr_value = NULL;	/* handle using SQLGetData() */
 	  continue;
 	}
-	ptr_result->len_value = sizeof(char)*columnSize+1;
+	ptr_result->len_value = sizeof(char)*(columnSize+1);
 	goto bind;
       }
     }
     
     switch (ptr_result->cTypeID)
     { case SQL_C_CHAR:
+	if ( columnSize == 0 )
+	  goto use_sql_get_data;
 	columnSize++;			/* one for decimal dot */
         /*FALLTHROUGH*/
       case SQL_C_BINARY:
-	if ( (int)columnSize > ctxt->max_nogetdata )
-	{ DEBUG(2,
-		Sdprintf("Wide SQL_C_BINARY column %d: using SQLGetData()\n",
-			 i));
-	  ptr_result->ptr_value = NULL;	/* handle using SQLGetData() */
-	  continue;
-	}
+	if ( (int)columnSize > ctxt->max_nogetdata || columnSize == 0 )
+	  goto use_sql_get_data;
         ptr_result->len_value = sizeof(char)*columnSize+1;
 	break;
       case SQL_C_SLONG:
