@@ -560,7 +560,8 @@ latex_section(Level, Attrs, Content) -->
 
 section_label(Attrs) -->
 	{ memberchk(name(Name), Attrs), !,
-	  atom_concat('sec:', Name, Label)
+	  delete_unsafe_label_chars(Name, SafeName),
+	  atom_concat('sec:', SafeName, Label)
 	},
 	latex(cmd(label(Label))).
 section_label(_) -->
@@ -577,6 +578,17 @@ deepen_section_level(Level0, Level1) :-
 	N1 is N + 1,
 	latex_section_level(Level1, N1).
 
+%%	delete_unsafe_label_chars(+LabelIn, -LabelOut)
+%
+%	delete unsafe characters from LabelIn. Currently only deletes _,
+%	as this appears  commonly  through   filenames,  but  cannot  be
+%	handled through the LaTeX processing chain.
+
+delete_unsafe_label_chars(LabelIn, LabelOut) :-
+	atom_chars(LabelIn, Chars),
+	delete(Chars, '_', CharsOut),
+	atom_chars(LabelOut, CharsOut).
+
 
 		 /*******************************
 		 *	   \ COMMANDS		*
@@ -653,14 +665,28 @@ tags_list(List) -->
 	latex(cmd(end(tags))),
 	[ nl(2) ].
 
-%%	tag(+Tag, +Value)// is det.
+%%	tag(+Tag, +Values:list)// is det.
 %
-%	Called from \tag(Name, Value) terms produced by doc_wiki.pl.
+%	Called from \tag(Name, Values) terms produced by doc_wiki.pl.
 
-tag(Tag, Value) -->
+tag(Tag, [One]) --> !,
 	{ doc_tag_title(Tag, Title) },
-	latex([cmd(tag(Title)), Value]).
+	latex([ cmd(tag(Title))
+	      | One
+	      ]).
+tag(Tag, More) -->
+	{ doc_tag_title(Tag, Title) },
+	latex([ cmd(mtag(Title)),
+		\tag_value_list(More)
+	      ]).
 
+tag_value_list([H|T]) -->
+	latex(['- '|H]),
+	(   { T \== [] }
+	->  [latex(' \\\\')],
+	    tag_value_list(T)
+	;   []
+	).
 
 %%	params(+Params:list) is det.
 %
@@ -720,7 +746,8 @@ file_title(Title, File, Options) -->
 	  Section =.. [Level,Title],
 	  file_base_name(File, BaseExt),
 	  file_name_extension(Base, _, BaseExt),
-	  atom_concat('sec:', Base, Label)
+	  delete_unsafe_label_chars(Base, SafeBase),
+	  atom_concat('sec:', SafeBase, Label)
 	},
 	latex(cmd(Section)),
 	latex(cmd(label(Label))).
@@ -1168,7 +1195,8 @@ print_latex_token(verb(Verb), Out) :-
 print_latex_token(verb(Verb), Out) :- !,
 	(   member(C, [$,'|',@,=,'"',^,!]),
 	    \+ sub_atom(Verb, _, _, _, C)
-	->  format(Out, '\\verb~w~w~w', [C,Verb,C])
+	->  atom_replace_char(Verb, '\n', ' ', Verb2),
+	    format(Out, '\\verb~w~w~w', [C,Verb2,C])
 	;   assertion(fail)
 	).
 print_latex_token(code(Code), Out) :- !,
@@ -1184,6 +1212,20 @@ print_latex_token(Rest, Out) :-
 	    write(Out, Rest)
 	).
 
+atom_replace_char(In, From, To, Out) :-
+	sub_atom(In, _, _, _, From), !,
+	atom_chars(In, CharsIn),
+	replace(CharsIn, From, To, CharsOut),
+	atom_chars(Out, CharsOut).
+atom_replace_char(In, _, _, In).
+
+replace([], _, _, []).
+replace([H|T0], H, N, [N|T]) :- !,
+	replace(T0, H, N, T).
+replace([H|T0], F, N, [H|T]) :-
+	replace(T0, F, N, T).
+
+
 %%	print_latex(+Out, +Text:atomic) is det.
 %
 %	Print Text, such that it comes out as normal LaTeX text.
@@ -1229,6 +1271,7 @@ print_char('$', Out) :- !, write(Out, '\\$').
 print_char('#', Out) :- !, write(Out, '\\#').
 print_char('%', Out) :- !, write(Out, '\\%').
 print_char('\\',Out) :- !, write(Out, '\\bsl{}').
+print_char('^', Out) :- !, write(Out, '$\\wedge$').
 print_char(C,   Out) :- put_char(Out, C).
 
 
