@@ -165,18 +165,32 @@ user:term_expansion((:- thread_local(PI)), (:- dynamic(PI))) :-
 %		When the tests are run.  Values are =manual=, =make=
 %		or make(all).
 %		
+%		* silent(+Bool)
+%		If =true= (default =false=), report successful tests
+%		using message level =silent=, only printing errors and
+%		warnings.
+%		
 %		* sto(+Bool)
 %		How to test whether code is subject to occurs check
 %		(STO).  If =false= (default), STO is not considered.
 %		If =true= and supported by the hosting Prolog, code
 %		is run in all supported unification mode and reported
 %		if the results are inconsistent.
-%		
-%	@tbd	Verify types	
 
 set_test_options(Options) :-
+	valid_options(Options, global_test_option),
 	set_test_flag(test_options, Options).
 
+global_test_option(load(Load)) :-
+	must_be(oneof([never,always,normal]), Load).
+global_test_option(run(When)) :-
+	must_be(oneof([manual,make,all]), When).
+global_test_option(silent(Bool)) :-
+	must_be(boolean, Bool).
+global_test_option(sto(Bool)) :-
+	must_be(boolean, Bool).
+
+
 %%	loading_tests
 %
 %	True if tests must be loaded.
@@ -435,6 +449,8 @@ test_option(all(_)).
 test_option(set(_)).
 test_option(nondet).
 test_option(fixme(_)).
+test_option(forall(X)) :-
+	must_be(callable, X).
 
 %%	test_option(+Option) is semidet.
 %
@@ -616,18 +632,27 @@ unification_capability(_) :-
 %	Run a single test.  
 
 run_test(Unit, Name, Line, Options, Body) :-
-	current_test_flag(test_options, Options),
-	option(sto(false), Options, false), !,
+	option(forall(Generator), Options), !,
+	unit_module(Unit, Module),
+	term_variables(Generator, Vars),
+	forall(Module:Generator,
+	       run_test_once(Unit, @(Name,Vars), Line, Options, Body)).
+run_test(Unit, Name, Line, Options, Body) :-
+	run_test_once(Unit, Name, Line, Options, Body).
+
+run_test_once(Unit, Name, Line, Options, Body) :-
+	current_test_flag(test_options, GlobalOptions),
+	option(sto(false), GlobalOptions, false), !,
 	run_test_6(Unit, Name, Line, Options, Body, Result),
 	report_result(Result, Options).
-run_test(Unit, Name, Line, Options, Body) :-
+run_test_once(Unit, Name, Line, Options, Body) :-
 	current_unit(Unit, _Module, _Supers, UnitOptions),
 	option(sto(Type), UnitOptions),
 	\+ option(sto(_), Options), !,
 	current_unification_capability(Cap0),
 	call_cleanup(run_test_cap(Unit, Name, Line, [sto(Type)|Options], Body),
 		     set_unification_capability(Cap0)).
-run_test(Unit, Name, Line, Options, Body) :-
+run_test_once(Unit, Name, Line, Options, Body) :-
 	current_unification_capability(Cap0),
 	call_cleanup(run_test_cap(Unit, Name, Line, Options, Body),
 		     set_unification_capability(Cap0)).
@@ -899,8 +924,13 @@ match_error(Expect, Rec) :-
 %	    If it is the setup handler for a test
 
 setup(Module, Context, Options) :-
+	option(condition(Condition), Options),
 	option(setup(Setup), Options), !,
-	(   catch(Module:Setup, E, true)
+	setup(Module, Context, [condition(Condition)]),
+	setup(Module, Context, [setup(Setup)]).
+setup(Module, Context, Options) :-
+	option(setup(Setup), Options), !,
+	(   catch(call_ex(Module, Setup), E, true)
 	->  (   var(E)
 	    ->	true
 	    ;	print_message(error, plunit(error(setup, Context, E))),
@@ -911,7 +941,7 @@ setup(Module, Context, Options) :-
 	).
 setup(Module, Context, Options) :-
 	option(condition(Setup), Options), !,
-	(   catch(Module:Setup, E, true)
+	(   catch(call_ex(Module, Setup), E, true)
 	->  (   var(E)
 	    ->	true
 	    ;	print_message(error, plunit(error(condition, Context, E))),
@@ -921,6 +951,14 @@ setup(Module, Context, Options) :-
 	).
 setup(_,_,_).
 
+%%	call_ex(+Module, +Goal)
+%
+%	Call Goal in Module after applying goal expansion.
+
+call_ex(Module, Goal) :-
+	Module:(expand_goal(Goal, GoalEx),
+		GoalEx).
+
 %%	cleanup(+Module, +Options) is det.
 %
 %	Call the cleanup handler and succeed.   Failure  or error of the
@@ -928,7 +966,7 @@ setup(_,_,_).
 
 cleanup(Module, Options) :-
 	option(cleanup(Cleanup), Options, true),
-	(   catch(Module:Cleanup, E, true)
+	(   catch(call_ex(Module, Cleanup), E, true)
 	->  (   var(E)
 	    ->	true
 	    ;	print_message(warning, E)
@@ -1248,7 +1286,8 @@ message(plunit(blocked(N))) -->
 	[ '~D tests are blocked:'-[N] ].
 message(plunit(blocked(Pos, Name, Reason))) -->
 	locationprefix(Pos),
-	[ 'test ~w: ~w'-[Name, Reason] ].
+	test_name(Name),
+	[ ': ~w'-[Reason] ].
 
 					% fail/success
 message(plunit(no_tests)) --> !,
@@ -1277,7 +1316,8 @@ message(plunit(fixme(Failed,Passed,Nondet))) -->
 message(plunit(failed(Unit, Name, Line, Failure))) -->
        { unit_file(Unit, File) },
        locationprefix(File:Line),
-       ['test ~w: '- [Name] ],
+       test_name(Name),
+       [': '-[] ],
        failure(Failure).
 					% Setup/condition errors
 message(plunit(error(Where, Context, Exception))) -->
@@ -1289,7 +1329,8 @@ message(plunit(error(Where, Context, Exception))) -->
 message(plunit(sto(Unit, Name, Line))) -->
 	{ unit_file(Unit, File) },
        locationprefix(File:Line),
-       ['test ~w is subject to occurs check (STO): '- [Name] ].
+       test_name(Name),
+       [' is subject to occurs check (STO): '-[] ].
 message(plunit(sto(Type, Result))) -->
 	sto_type(Type),
 	sto_result(Result).
@@ -1309,6 +1350,11 @@ message(interrupt(begin)) -->
 	'$messages':prolog_message(interrupt(begin)).
 :- endif.
 
+test_name(@(Name,Bindings)) --> !,
+	[ 'test ~w (forall bindings = ~p)'-[Name, Bindings] ].
+test_name(Name) --> !,
+	[ 'test ~w'-[Name] ].
+
 sto_type(sto_error_incomplete) -->
 	[ 'Finite trees (error checking): ' ].
 sto_type(rational_trees) -->
