@@ -29,30 +29,23 @@
 #include <SWI-Stream.h>
 #include <SWI-Prolog.h>
 #include "atom.h"
-#include "unicode_map.c"
+#include "murmur.h"
 #include <wchar.h>
 #include <wctype.h>
 #include <assert.h>
+
 #ifdef __WINDOWS__
 #define inline __inline
 #endif
 
+#include "unicode_map.c"
+
 
 		 /*******************************
 		 *	   TEXT HANDLING	*
 		 *******************************/
 
-typedef unsigned char charA;
-typedef wchar_t       charW;
-
-typedef struct text
-{ const charA *a;
-  const charW *w;
-  size_t length;
-} text;
-
-
-static int
+static inline int
 get_atom_text(atom_t atom, text *txt)
 { if ( (txt->a = (const charA*)PL_atom_nchars(atom, &txt->length)) )
   { txt->w = NULL;
@@ -73,57 +66,80 @@ fetch(const text *txt, int i)
 }
 
 
+static int
+fill_atom_info(atom_info *info)
+{ if ( !info->resolved )
+  { info->resolved = TRUE;
+
+    if ( !(info->rc=get_atom_text(info->handle, &info->text)) )
+    { info->text.a = NULL;
+      info->text.w = NULL;
+    }
+  }
+
+  return info->rc;
+}
+
+
 		 /*******************************
 		 *	      COMPARE		*
 		 *******************************/
 
 static inline int
 cmpA(int c1, int c2, int *dl2)
-{ int k1 = sort_pointA(c1);
-  int k2 = sort_pointA(c2);
-  int d;
-
-  if ( (d=((k1>>8)-(k2>>8))) == 0 )
-  { if ( *dl2 == 0 )
-      *dl2 = (k1&0xff) - (k2&0xff);
+{ if ( c1 == c2 )
+  { return 0;
+  } else
+  { int k1 = sort_pointA(c1);
+    int k2 = sort_pointA(c2);
+    int d;
+    
+    if ( (d=((k1>>8)-(k2>>8))) == 0 )
+    { if ( *dl2 == 0 )
+	*dl2 = (k1&0xff) - (k2&0xff);
+    }
+    
+    return d;
   }
-
-  return d;
 }
 
 
 static inline int
 cmpW(int c1, int c2, int *dl2)
-{ int k1 = sort_point(c1);
-  int k2 = sort_point(c2);
-  int d;
+{ if ( c1 == c2 )
+  { return 0;
+  } else
+  { int k1 = sort_point(c1);
+    int k2 = sort_point(c2);
+    int d;
 
-  if ( (d=((k1>>8)-(k2>>8))) == 0 )
-  { if ( *dl2 == 0 )
-      *dl2 = (k1&0xff) - (k2&0xff);
+    if ( (d=((k1>>8)-(k2>>8))) == 0 )
+    { if ( *dl2 == 0 )
+	*dl2 = (k1&0xff) - (k2&0xff);
+    }
+    
+    return d;
   }
-
-  return d;
 }
 
 
 int
-cmp_atoms(atom_t a1, atom_t a2)
-{ text t1, t2;
+cmp_atom_info(atom_info *info, atom_t a2)
+{ text t2;
   int i;
   int dl2 = 0;
   size_t n;
   
-  if ( a1 == a2 )
+  if ( info->handle == a2 )
     return 0;
 
-  if ( !get_atom_text(a1, &t1) ||
+  if ( !fill_atom_info(info) ||
        !get_atom_text(a2, &t2) )
   { goto cmphandles;			/* non-text atoms? */
   }
 
-  if ( t1.a && t2.a )
-  { const charA *s1 = t1.a;
+  if ( info->text.a && t2.a )
+  { const charA *s1 = info->text.a;
     const charA *s2 = t2.a;
     int d;
 
@@ -135,18 +151,18 @@ cmp_atoms(atom_t a1, atom_t a2)
     return d;
   }
 
-  n = (t1.length < t2.length ? t1.length : t2.length);
+  n = (info->text.length < t2.length ? info->text.length : t2.length);
 
-  if ( t1.w && t2.w )
-  { const charW *s1 = t1.w;
+  if ( info->text.w && t2.w )
+  { const charW *s1 = info->text.w;
     const charW *s2 = t2.w;
 
     for(;;s1++, s2++)
     { if ( n-- == 0 )
-      { if ( t1.length == t2.length )
+      { if ( info->text.length == t2.length )
 	  goto eq;
 	
-	return t1.length < t2.length ? -1 : 1;
+	return info->text.length < t2.length ? -1 : 1;
       } else
       { int d;
 
@@ -158,12 +174,12 @@ cmp_atoms(atom_t a1, atom_t a2)
   
   for(i=0; ; i++)
   { if ( n-- == 0 )
-    { if ( t1.length == t2.length )
+    { if ( info->text.length == t2.length )
 	  goto eq;
 	
-      return t1.length < t2.length ? -1 : 1;
+      return info->text.length < t2.length ? -1 : 1;
     } else
-    { wint_t c1 = fetch(&t1, i);
+    { wint_t c1 = fetch(&info->text, i);
       wint_t c2 = fetch(&t2, i);
       int d;
 
@@ -177,10 +193,86 @@ eq:
     return dl2;
 
 cmphandles:
-  return a1 < a2 ? -1 : 1;		/* == already covered */
+  return info->handle < a2 ? -1 : 1;		/* == already covered */
 }
 
 
+int
+cmp_atoms(atom_t a1, atom_t a2)
+{ atom_info info = {0};
+
+  if ( a1 == a2 )
+    return 0;
+
+  info.handle = a1;
+
+  return cmp_atom_info(&info, a2);
+}
+
+
+		 /*******************************
+		 *	       HASH		*
+		 *******************************/
+
+static unsigned int
+string_hashA(const char *s, size_t len)
+{ const unsigned char *t = (const unsigned char *)s;
+  unsigned int hash = 0;
+
+  while( len>0 )
+  { unsigned char buf[256];
+    unsigned char *o = buf-1;
+    int cp = len > 256 ? 256 : (int)len;
+    const unsigned char *e = t+cp;
+
+    t--;
+    while(++t<e)
+      *++o = sort_pointA(*t)>>8;
+    hash ^= rdf_murmer_hash(buf, cp, MURMUR_SEED);
+
+    len -= cp;
+  }
+
+  return hash;
+}
+
+
+static unsigned int
+string_hashW(const wchar_t *t, size_t len)
+{ unsigned int hash = 0;
+
+  while( len>0 )
+  { unsigned short buf[256];
+    unsigned short *o = buf;
+    int cp = len > 256 ? 256 : (int)len;
+    const wchar_t *e = t+cp;
+
+    while(t<e)
+      *o++ = (short)(sort_point(*t++)>>8);
+    hash ^= rdf_murmer_hash(buf, cp*sizeof(short), MURMUR_SEED);
+
+    len -= cp;
+  }
+
+  return hash;
+}
+
+
+unsigned int
+atom_hash_case(atom_t a)
+{ const char *s;
+  const wchar_t *w;
+  size_t len;
+
+  if ( (s = PL_atom_nchars(a, &len)) )
+    return string_hashA(s, len);
+  else if ( (w = PL_atom_wchars(a, &len)) )
+    return string_hashW(w, len);
+  else
+  { assert(0);
+    return 0;
+  }
+}
 
 
 		 /*******************************
