@@ -1,13 +1,25 @@
-/*  $Id: quote.c,v 1.8 2005/10/12 11:41:19 jan Exp $
+/*  $Id$
 
-    Part of SWI-Prolog SGML/XML parser
+    Part of SWI-Prolog
 
-    Author:  Jan Wielemaker
-    E-mail:  jan@swi.psy.uva.nl
-    WWW:     http://www.swi.psy.uva.nl/projects/SWI-Prolog/
-    Copying: LGPL-2.  See the file COPYING or http://www.gnu.org
+    Author:        Jan Wielemaker
+    E-mail:        wielemak@science.uva.nl
+    WWW:           http://www.swi-prolog.org
+    Copyright (C): 1985-2007, University of Amsterdam
 
-    Copyright (C) 1990-2002 SWI, University of Amsterdam. All rights reserved.
+    This library is free software; you can redistribute it and/or
+    modify it under the terms of the GNU Lesser General Public
+    License as published by the Free Software Foundation; either
+    version 2.1 of the License, or (at your option) any later version.
+
+    This library is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+    Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
 #include <SWI-Stream.h>			/* encoding */
@@ -34,7 +46,7 @@ typedef struct charbuf
 { char     buffer[1024];
   char    *bufp;
   char    *end;
-  int      size;
+  size_t   size;
 } charbuf;
 
 
@@ -53,8 +65,8 @@ free_buf(charbuf *b)
 
 
 static int
-room_buf(charbuf *b, int room)
-{ int used = b->end - b->bufp;
+room_buf(charbuf *b, size_t room)
+{ size_t used = b->end - b->bufp;
 
   if ( room + used > b->size )
   { if ( b->bufp == b->buffer )
@@ -78,7 +90,7 @@ room_buf(charbuf *b, int room)
 }
 
 
-static unsigned int
+static size_t
 used_buf(const charbuf *b)
 { return b->end - b->bufp;
 }
@@ -113,7 +125,7 @@ add_char_bufW(charbuf *b, int chr)
 
 static int
 add_str_buf(charbuf *b, const char *s)
-{ int len = strlen(s);
+{ size_t len = strlen(s);
 
   if ( room_buf(b, len+1) )
   { memcpy(b->end, s, len+1);
@@ -128,7 +140,7 @@ add_str_buf(charbuf *b, const char *s)
 
 static int
 add_str_bufW(charbuf *b, const char *s)
-{ int len = strlen(s);
+{ size_t len = strlen(s);
 
   if ( room_buf(b, len*sizeof(wchar_t)) )
   { wchar_t *p = (wchar_t*)b->end;
@@ -149,7 +161,7 @@ static foreign_t
 do_quote(term_t in, term_t quoted, char **map, int maxchr)
 { char *inA = NULL;
   wchar_t *inW = NULL;
-  unsigned len;
+  size_t len;
   const unsigned  char *s;
   charbuf buffer;
   int changes = 0;
@@ -248,6 +260,13 @@ get_max_chr(term_t t, int *maxchr)
 }
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+(*) xml_quote_attribute/3 assumes the attribute is   quoted using "" and
+does *not* escape '. Although escaping ' with &apos; is valid XML, it is
+*not* valid html, and this  routine  is   also  used  by  the html_write
+library.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 static foreign_t
 xml_quote_attribute(term_t in, term_t out, term_t encoding)
 { static char **map;
@@ -265,7 +284,7 @@ xml_quote_attribute(term_t in, term_t out, term_t encoding)
     map['<']  = "&lt;";
     map['>']  = "&gt;";
     map['&']  = "&amp;";
-    map['\''] = "&apos;";
+/*  map['\''] = "&apos;"; See (*) */
     map['"']  = "&quot;";
   }
 
@@ -306,7 +325,7 @@ static foreign_t
 xml_name(term_t in, term_t encoding)
 { char *ins;
   wchar_t *inW;
-  unsigned len;
+  size_t len;
   static dtd_charclass *map;
   unsigned int i;
   int maxchr;
