@@ -1,11 +1,11 @@
-/*  $Id: interface.c,v 1.83 2005/11/15 13:22:07 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        jan@swi.psy.uva.nl
-    WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (C): 1985-2002, University of Amsterdam
+    E-mail:        wielemak@science.uva.nl
+    WWW:           http://www.swi-prolog.org/packages/xpce/
+    Copyright (C): 1985-2007, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -22,19 +22,23 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#if !defined(__WIN32__) && defined(WIN32)
-#define __WIN32__ 1
-#endif
-
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #define HAVE_MALLOC_H 1
 #define HAVE_SIGNAL_H 1
+#include "../../src/md.h"
 #else
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 #endif
 
+#define SWI 1			/* SWI-Prolog version 2.5! and up */
+
+#ifdef SWI
+#include <SWI-Stream.h>			/* get intptr_t, etc first */
+#include <SWI-Prolog.h>
+#endif
+
 #define DEBUG(g) ((void)0)
 
 #include <stdio.h>
@@ -64,12 +68,10 @@
 #define UNLOCK()
 #endif
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #include <windows.h>
 #endif
 
-#define SWI 1			/* SWI-Prolog version 2.5! and up */
-
 #ifdef __GNUC__
 #define TermVector(name, size)  Term name[size]
 #define ObjectVector(name, size) PceObject name[size]
@@ -124,8 +126,6 @@ typedef SP_term_ref	Fid;
 #endif /*SICSTUS*/
 
 #ifdef SWI
-#include <SWI-Stream.h>
-#include <SWI-Prolog.h>
 
 typedef atom_t		Atom;
 typedef term_t		Term;
@@ -389,7 +389,7 @@ cToPceType(const char *name)
 #define CloseForeign(fid)	SP_reset_term_refs(fid)
 #define InstallPredicate(n, a, f, flags)
 
-#if defined(__WIN32__)
+#if defined(__WINDOWS__)
 #define PROLOG_INSTALL_DISPATCH_FUNCTION(f) {}
 #else
 #define PROLOG_INSTALL_DISPATCH_FUNCTION(f) \
@@ -494,13 +494,14 @@ static Functor  FUNCTOR_behaviour1;
 static predicate_t PREDICATE_send_implementation;
 static predicate_t PREDICATE_get_implementation;
 
-static long			pl_max_integer;
 static PL_dispatch_hook_t	old_dispatch_hook;
 
-#define PROLOG_MAX_INTEGER pl_max_integer
-
 #define AtomCharp(a)		PL_atom_chars((a))
-#define GetInteger(a, i)	PL_get_long((a), (i))
+#if SIZEOF_VOIDP == 8 && SIZEOF_VOIDP != SIZEOF_LONG
+#define GetInteger(a, i)	PL_get_int64((a), (i))
+#else
+#define GetInteger(a, i)	PL_get_long((a), (long*)(i))
+#endif
 #define GetAtom(a, n)		PL_get_atom((a), (n))
 #define GetFloat(a, f)		PL_get_float((a), (f))
 #define NewTerm()		PL_new_term_ref()
@@ -533,7 +534,7 @@ static PL_dispatch_hook_t	old_dispatch_hook;
 #define InstallPredicate(n, a, f, flags) \
 	PL_register_foreign(n, a, f, flags)
 
-#if defined(__WIN32__)
+#if defined(__WINDOWS__)
 #define PROLOG_INSTALL_DISPATCH_FUNCTION(f) {}
 #else
 #define PROLOG_INSTALL_DISPATCH_FUNCTION(f) \
@@ -548,9 +549,7 @@ static PL_dispatch_hook_t	old_dispatch_hook;
 
 static void
 initHostConstants()
-{ pl_max_integer = PL_query(PL_QUERY_MAX_INTEGER);
-  
-  FUNCTOR_behaviour1        = PL_new_functor(ATOM_behaviour, 1);
+{ FUNCTOR_behaviour1        = PL_new_functor(ATOM_behaviour, 1);
   FUNCTOR_error2  	    = PL_new_functor(ATOM_error, 2);
   FUNCTOR_existence_error2  = PL_new_functor(ATOM_existence_error, 2);
   FUNCTOR_get2    	    = PL_new_functor(ATOM_get, 2);
@@ -577,7 +576,7 @@ initHostConstants()
 }
 
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #include <console.h>
 
 static RlcUpdateHook old_update_hook;
@@ -648,7 +647,7 @@ uninstall_pl2xpce()
 		 old_dispatch_hook, old_update_hook));
 
   PL_dispatch_hook(old_dispatch_hook);
-#ifdef __WIN32__
+#ifdef __WINDOWS__
   indirect_rlc_update_hook(old_update_hook);
 #endif
   if ( exitpce_hook )
@@ -901,7 +900,7 @@ PceObject referenceToObject(Term a)
 
 static Atom
 nameToAtom(PceName name)
-{ unsigned int len;
+{ size_t len;
   const char *textA;
   const wchar_t *textW;
 
@@ -1164,7 +1163,7 @@ get_object_arg(term_t t, PceObject* obj)
       return TRUE;
     case PL_INTEGER:
       if ( val.i >= PCE_MIN_INT && val.i <= PCE_MAX_INT )
-	*obj = cToPceInteger((long)val.i);
+	*obj = cToPceInteger((intptr_t)val.i);
       else
 	*obj = cToPceReal((double)val.i);
       return TRUE;
@@ -1314,7 +1313,7 @@ unlinkProlog(PceObject hd)
 static PceObject
 getPrintNameProlog(PceObject hd)
 { char *buffer = NULL;
-  int   size   = 0;
+  size_t  size   = 0;
   PceObject rval;
   IOSTREAM *s;
 
@@ -1420,6 +1419,10 @@ termToObject(Term t, PceType type, Atom assoc, int new)
 { Atom functor;
   int arity;
 
+  DEBUG(Sdprintf("termToObject(");
+	PL_write_term(Soutput, t, 1200, 0);
+	Sdprintf(")\n"));
+
   if ( GetNameArity(t, &functor, &arity) )
   { 					/* Just an atom */
     if ( arity == 0 )
@@ -1458,7 +1461,7 @@ termToObject(Term t, PceType type, Atom assoc, int new)
     if ( functor == ATOM_string && arity == 1 )
     { char *s;
       wchar_t *sW;
-      unsigned int len;
+      size_t len;
       Term a = NewTerm();
       PceName pceassoc = atomToAssoc(assoc);
 
@@ -1477,7 +1480,7 @@ termToObject(Term t, PceType type, Atom assoc, int new)
     { PceObject h;
       Term a = NewTerm();
       double f;
-      long r;
+      intptr_t r;
       
       QGetArg(1, t, a);
 
@@ -1548,7 +1551,7 @@ termToObject(Term t, PceType type, Atom assoc, int new)
     }
   } else				/* not a term */
   { double f;
-    long r;
+    intptr_t r;
     
 					/* PL_get_integer() translates */
 					/* `whole' floats to integers */
@@ -1562,7 +1565,7 @@ termToObject(Term t, PceType type, Atom assoc, int new)
 #ifdef O_STRING
   { char *s;
     wchar_t *w;
-    unsigned int len;
+    size_t len;
 
     if ( PL_get_string(t, &s, &len) )	/* string object (if supported) */
       return cToPceStringA(atomToAssoc(assoc), s, len, FALSE);
@@ -1598,7 +1601,7 @@ unifyObject(Term t, PceObject obj, int top)
     case PCE_REAL:			/* float (real object) */
       return UnifyFloat(t, value.real);
     case PCE_NAME:			/* name */
-    { unsigned int len;
+    { size_t len;
       const char *textA;
       const wchar_t *textW;
 	
@@ -1635,7 +1638,7 @@ unifyObject(Term t, PceObject obj, int top)
   if ( pceIsString(obj) )	/* string: handle special */
   { const char *textA;
     const wchar_t *textW;
-    unsigned int len;
+    size_t len;
     Term a = NewTerm();
 
     if ( (textA = pceCharArrayToCA(obj, &len)) )
@@ -2137,7 +2140,7 @@ pl_object1(Term ref)
        arity == 1 )
   { Term a = NewTerm();
     Atom refname;
-    long refi;
+    intptr_t refi;
 
     QGetArg(1, ref, a);
     if ( GetAtom(a, &refname) )
@@ -2224,7 +2227,7 @@ PrologSend(PceObject prolog, PceObject sel, int argc, PceObject *argv)
     for(i=0; i<argc; i++)
       put_object(terms+i, argv[i]);
 
-    qid  = PL_open_query(m, DebugMode, pred, terms);
+    qid  = PL_open_query(m, DebugMode|PL_Q_PASS_EXCEPTION, pred, terms);
     rval = PL_next_solution(qid);
     PL_cut_query(qid);
   } else
@@ -2563,16 +2566,16 @@ pceContextModule()
 		 *******************************/
 
 #ifdef SWI
-#define fdFromHandle(h) ((int)((long)(h)))
+#define fdFromHandle(h) ((int)((intptr_t)(h)))
 
-static int
-Swrite_pce(void *handle, char *buf, int size)
+static ssize_t
+Swrite_pce(void *handle, char *buf, size_t size)
 { return pceWrite(fdFromHandle(handle), buf, size);
 }
 
 
-static int
-Sread_pce(void *handle, char *buf, int size)
+static ssize_t
+Sread_pce(void *handle, char *buf, size_t size)
 { return pceRead(fdFromHandle(handle), buf, size);
 }
 
@@ -2592,7 +2595,9 @@ Sclose_pce(void *handle)
 static int
 Scontrol_pce(void *handle, int cmd, void *closure)
 { switch(cmd)
-  { case SIO_SETENCODING:
+  { case SIO_FLUSHOUTPUT:
+      return 0;
+    case SIO_SETENCODING:
       return pceControl(fdFromHandle(handle), PCE_SETENCODING, closure);
   }
 
@@ -2705,7 +2710,7 @@ pce_dispatch(int fd)
 
 #ifdef SICSTUS
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 
 void *
 pl_malloc(unsigned int size)
@@ -2724,13 +2729,13 @@ pl_free(void *ptr)
 { SP_free(ptr);
 }
 
-#else /* __WIN32__ */
+#else /* __WINDOWS__ */
 
 #define pl_malloc SP_malloc
 #define pl_realloc SP_realloc
 #define pl_free SP_free
 
-#endif /*__WIN32__*/
+#endif /*__WINDOWS__*/
 
 
 		 /*******************************
@@ -3159,7 +3164,7 @@ do_reset(void)
 }
 
 
-#ifdef WIN32
+#ifdef __WINDOWS__
 static void
 do_redraw(void)
 { pceRedraw(FALSE);			/* FALSE: do not sync Xserver */
