@@ -163,8 +163,8 @@ open_object(_Emacs, Object:prolog, _NewWindow:new_window=[bool]) :->
 	edit(Object).
 
 show_bookmarks(_) :->
-	"Show PceEmacs bookmarks"::
-	send(@emacs_mark_list, open).
+	"Show PceEmacs bookmarks window"::
+	send(@emacs_mark_list, expose).
 
 
 		 /*******************************
@@ -278,7 +278,8 @@ server_start(Emacs, Force:[bool]) :->
 	    ;	get(@emacs_server, status, listen)
 	    )
 	->  (   current_prolog_flag(windows, true)
-	    ->	catch(start_emacs_dde_server(false), _, true)
+	    ->	Goal = start_emacs_dde_server(false), % fool xref
+	        catch(Goal, _, true)
 	    ;	true
 	    )
 	;   (	send(@emacs_server_address, exists, @off)
