@@ -1,4 +1,4 @@
-/*  $Id: buffer.pl,v 1.24 2003/06/29 08:58:16 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -59,7 +59,14 @@ variable(xref_generation,
 	 both,
 	 "Last generation we analysed").
 
-class_variable(undo_buffer_size, int, 40000).
+class_variable(undo_buffer_size,      int, 40000).
+class_variable(newline_existing_file, {posix,dos,detect}, detect).
+class_variable(newline_new_file,      {posix,dos},        posix).
+:- if(current_prolog_flag(windows, true)).
+class_variable(newline_new_file,      {posix,dos},        dos).
+:- else.
+class_variable(newline_new_file,      {posix,dos},        posix).
+:- endif.
 
 initialise(B, File:file*, Name:[name]) :->
 	"Create from file and name"::
@@ -114,10 +121,10 @@ scratch_text('% This buffer is for notes you don\'t want to save.\n\
 	     % then enter the text in that file\'s own buffer.\n\n').
 
 :- pce_global(@emacs_interpreter_regex,
-	      new(regex('#!\\(\\S +\\)\\s '))).
+	      new(regex('#!(\\S+)\\s'))).
 :- pce_global(@emacs_mode_regex,	% -*- Mode -*-
 					% -*- mode: Mode; ... -*-
-	      new(regex('.*-\\*-\\s *\\([Mm]ode:\\s *\\(\\w+\\);.*-\\*-\\|\\(\\w+\\)\\s *-\\*-\\)'))).
+	      new(regex('.*-\\*-\\s*([Mm]ode:\\s*(\\w+);.*-\\*-|(\\w+)\\s*-\\*-)'))).
 
 
 determine_initial_mode(B) :->
@@ -128,6 +135,8 @@ determine_initial_mode(B) :->
 	    get(Mode0?downcase, value, Mode),
 	    get(@pce, convert, Mode, emacs_mode, _ModeObject)
 	->  send(B, slot, mode, Mode)
+	;   content_from_mode(B, Mode)
+	->  send(B, slot, mode, Mode)
 	;   (   send(@emacs_interpreter_regex, match, B),
 		get(@emacs_interpreter_regex, register_value, B, 1, Match),
 		To = @emacs_interpreter_mode_list
@@ -142,6 +151,18 @@ determine_initial_mode(B) :->
 	).
 
 
+%	content_from_mode(+Buffer, -Mode)
+%	
+%	Search Buffer with the patterns from @emacs_content_mode_list
+
+content_from_mode(B, Mode) :-
+	get(@emacs_content_mode_list?members, find,
+	     message(@arg1?name?first, search, B,
+		     0, @arg1?name?second),
+	    Att),
+	get(Att, value, Mode).
+
+
 attach(B, E:editor) :->
 	"A new editor is attached.  Prepare it"::
 	get(B, editors, Editors),
@@ -228,12 +249,16 @@ file(B, File:file) :->
 	(   send(directory(File?name), exists)
 	->  send(File, error, open_file, read, 'is a directory')
 	;   send(File, exists)
-	->  ignore(send(B, insert_file, 0, File)),
+	->  get(B, newline_existing_file, OpenMode),
+	    send(File, newline_mode, OpenMode),
+	    ignore(send(B, insert_file, 0, File)),
 	    send(B, reset_undo),
 	    send(B, modified, @off),
 	    send(B, slot, time_stamp, File?time)
 	;   send(B, reset_undo),
-	    send(B, modified, @off)
+	    send(B, modified, @off),
+	    get(B, newline_new_file, Mode),
+	    send(File, newline_mode, Mode)
 	),
 	send(B, slot, file, File),
 	new(F2, file(File?absolute_path)),
@@ -536,7 +561,9 @@ properties(Buffer, V:view) :<-
 	),
 	(   File \== @nil
 	->  get(File, absolute_path, Path),
-	    send(V, appendf, 'Path:\t%s\n', Path)
+	    send(V, appendf, 'Path:\t%s\n', Path),
+	    send(V, appendf, 'Encoding:\t%s (BOM=%s, NL=%s)\n',
+		 File?encoding, File?bom, File?newline_mode)
 	;   send(V, appendf, 'Path:\t<No file>\n')
 	),
 	send(V, caret, 0),
