@@ -1,4 +1,4 @@
-/*  $Id: sgml_mode.pl,v 1.11 2003/08/19 09:24:10 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -75,7 +75,7 @@ elements that are closed by omited elements.
 		    % SYNTAX TABLE
 		    [ '"'  = string_quote('"'),
 		      '\'' = string_quote('\''),
-		      paragraph_end('\\s *$\\|^<p>\\|\\s +<')
+		      paragraph_end('\\s*$|^<p>|\\s+<')
 		    ]).
 
 class_variable(auto_colourise_size_limit, int, 100000,
@@ -131,7 +131,7 @@ load_dtd(M, Parser) :-
 	(   send(Re, search, TB)
 	->  get(Re, register_start, Start),
 	    pce_open(TB, read, In),
-	    seek(In, Start, bof, _),
+	    seek_to(In, Start),
 	    catch(sgml_parse(Parser,
 			     [ source(In),
 			       parse(declaration)
@@ -185,7 +185,8 @@ destroy_dtd(M) :->
 setup_mode(E) :->
 	"Switch editor into fill-mode"::
 	send(E, fill_mode, @on),
-	send(E, setup_styles).
+	send(E, setup_styles),
+	send_super(E, setup_mode).
 
 
 unlink(M) :->
@@ -281,8 +282,9 @@ colourise_element(M, Warn:[bool]) :->
 	    ->	From = F,
 		To = T
 	    )
-	->  send(M, remove_syntax_fragments, From, To),
-	    seek(In, From, bof, _),
+	->  debug(sgml_mode, 'Colouring ~D..~D', [From, To]),
+	    send(M, remove_syntax_fragments, From, To),
+	    seek_to(In, From),
 	    set_sgml_parser(Parser, charpos(From)),
 	    colourise(M, Parser,
 		      [ source(In),
@@ -333,7 +335,7 @@ find_element(M, Parser, _Re, In, Caret, Start, Start-To) :-
 	       and(message(@arg1, overlap, Start),
 		   @arg1?parsed == @off),
 	       _),
-	seek(In, Start, bof, _),
+	seek_to(In, Start),
 	set_sgml_parser(Parser, charpos(Start)),
 	retractall(caret(Caret)),
 	retractall(tag(_)),
@@ -423,12 +425,12 @@ colourise(M, Parser, Options) :-
 	      show_message(M, E)),
 	erase(Ref).
 	
-on_begin(_Tag, _Attributes, Parser) :-
+on_begin(Tag, _Attributes, Parser) :-
 	get_sgml_parser(Parser, file(File)),
 	current_tb(TB, File),
-%	format('Tag ~w, Attr = ~p~n', [Tag, Attributes]),
 	get_sgml_parser(Parser, charpos(From, To)),
 	get_sgml_parser(Parser, event_class(Class)),
+	debug(sgml_mode, 'begin ~w at ~d..~d', [Tag, From, To]),
 	Class \== omitted,
 	colour_item(tag(begin, Class), TB, From, To).
 on_end(_Tag, Parser) :-
@@ -711,10 +713,10 @@ insert_attribute(M, Att:name, Val:'[name|int|real]') :->
 	"Add attribute-value pair"::
 	get(M, text_buffer, TB),
 	get(M, caret, Caret),
-	new(Re, regex('\\(\\s +\\|[/>]\\)')),
+	new(Re, regex('(\\s+|[/>])')),
 	send(Re, search, TB, Caret),	% find place to insert
 	get(Re, register_start, 0, Where),
-	(   send(regex('\\s +'), match, TB, Where)
+	(   send(regex('\\s+'), match, TB, Where)
 	->  get(Re, register_end, 0, NewCaret), % after blanks
 	    send(M, caret, NewCaret)
 	;   send(M, caret, Where),
@@ -723,13 +725,13 @@ insert_attribute(M, Att:name, Val:'[name|int|real]') :->
 	(   Val == @default
 	->  send(M, format, '%s=""', Att),
 	    get(M, caret, C),
-	    (	send(M, looking_at, '\\s \\|[/>]')
+	    (	send(M, looking_at, '\\s|[/>]')
 	    ->  true
 	    ;	send(M, format, ' ')
 	    ),
 	    send(M, caret, C-1)
 	;   send(M, format, '%s="%s"', Att, Val),
-	    (	send(M, looking_at, '\\s \\|[/>]')
+	    (	send(M, looking_at, '\\s|[/>]')
 	    ->  true
 	    ;	send(M, format, ' ')
 	    )
@@ -831,7 +833,7 @@ prepare_insert(M) :->
 		Col is Col0+2,
 		get(M, text_buffer, TB),
 		get(TB, scan, Caret, line, 0, start, SOL),
-		(   new(Re, regex('\\s *')),
+		(   new(Re, regex('\\s*')),
 		    send(Re, match, TB, SOL, Caret),
 		    get(Re, register_end, Caret)
 		->  true		% at a blank line
@@ -897,7 +899,7 @@ insert_by_style(@default, M, Tag, End) :-
 	get(M, text_buffer, TB),
 	get(M, caret, Caret),
 	get(TB, scan, Caret, line, 0, start, SOL),
-	(   send(regex('\\s *$'), match, TB, SOL)
+	(   send(regex('\\s*$'), match, TB, SOL)
 	->  insert_by_style(block, M, Tag, End)
 	;   insert_by_style(inline, M, Tag, End)
 	).
@@ -946,7 +948,7 @@ required_content(A) -->
 
 
 looking_at_element(M, From:int, Elem:name) :<-
-	new(Re, regex('<\\([-_:a-zA-Z0-9]+\\)')),
+	new(Re, regex('<([-_:a-zA-Z0-9]+)')),
 	get(M, text_buffer, TB),
 	send(Re, match, TB, From),
 	get(Re, register_value, TB, 1, name, Elem).
@@ -969,7 +971,7 @@ allowed_elements(M, Allowed:prolog) :<-
 	    (	dtd_property(DTD, doctype(E))
 	    ;   dtd_property(DTD, element(E, omit(_, false), _))
 	    )
-	->  seek(In, From, bof, _),
+	->  seek_to(In, From),
 	    set_sgml_parser(Parser, charpos(From)),
 	    Len is Caret - From,
 	    catch(sgml_parse(Parser,
@@ -1088,3 +1090,16 @@ open_in_browser(M) :->
 variable(parsed,   bool := @on,	     both, "@off for unparsed fragments").
 
 :- pce_end_class.
+
+
+		 /*******************************
+		 *	       UTIL		*
+		 *******************************/
+
+%	seek_to(+Stream, +Pos)
+%	
+%	Seek to Pos on Stream related to an  XPCE object.
+
+seek_to(Stream, Pos) :-
+%	seek(Stream, 0, bof, _),		% Bug is fixed
+	seek(Stream, Pos, bof, _).
