@@ -1,4 +1,4 @@
-/*  $Id: pce_grapher.pl,v 1.4 2003/10/30 13:20:41 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -208,7 +208,7 @@ make_grapher(G) :-
 	send(new(G, grapher), open),
 	send(G, wait).
 
-%	grapher(+MessageOrList)
+%%	grapher(+MessageOrList)
 %	
 %	Send a message or list of messages to the grapher.  Leaves a
 %	choicepoint which undos the modifications if we backtrack into
