@@ -1,4 +1,4 @@
-/*  $Id: image.c,v 1.55 2005/09/30 19:22:50 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -890,7 +890,7 @@ getPostscriptFormatImage(Image image)
 		*       PREDEFINED IMAGES	*
 		********************************/
 
-#if defined(WIN32) || defined(HAVE_LIBXPM)
+#if defined(__WINDOWS__) || defined(HAVE_LIBXPM)
 #define XPM_PCEIMAGE 1			/* use an XPM image */
 #endif
 
@@ -1155,6 +1155,8 @@ static senddecl send_image[] =
      NAME_pixel, "Set pixel at x-y to bool or colour"),
   SM(NAME_setPixel, 2, T_xAint_yAint, setPixelImage,
      NAME_pixel, "Set pixel at x-y (to 1 or foreground)"),
+  SM(NAME_Postscript, 1, "{head,body}", drawPostScriptImage,
+     NAME_postscript, "Create PostScript"),
   SM(NAME_DrawPostScript, 1, "{head,body}", drawPostScriptImage,
      NAME_postscript, "Create PostScript"),
   SM(NAME_Xclose, 1, "display", XcloseImage,
