@@ -1,4 +1,4 @@
-/*  $Id: postscript.c,v 1.36 2005/09/30 19:22:50 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -57,7 +57,7 @@ StringObj
 getPostscriptObject(Any obj, Bool ls, Area a)
 { StringObj result;
   char *PostScript = NULL;
-  int  size = 0;
+  size_t size = 0;
 
   if ( isNil(documentFonts) )
     documentFonts = globalObject(NAME_DocumentFonts, ClassChain, EAV);
@@ -163,7 +163,7 @@ _output(char *fm, va_list args)
 			return;
 	  case 's':	putString(va_arg(args, char *));
 			continue;
-	  case 'd':	sprintf(tmp, "%ld", valInt(va_arg(args, Int)));
+	  case 'd':	sprintf(tmp, INTPTR_FORMAT, valInt(va_arg(args, Int)));
 			putString(tmp);
 			continue;
 	  case 'D':	sprintf(tmp, "%d", va_arg(args, int));
@@ -173,24 +173,26 @@ _output(char *fm, va_list args)
 			putString(tmp);
 			continue;
 	  case 'm':   { Graphical gr = va_arg(args, Graphical);
-			sprintf(tmp, "%ld %ld moveto", valInt(gr->area->x),
-						     valInt(gr->area->y));
+			sprintf(tmp, INTPTR_FORMAT " " INTPTR_FORMAT " moveto",
+				valInt(gr->area->x),
+				valInt(gr->area->y));
 			putString(tmp);
 			continue;
 		      }
 	  case 'c':   { Point p = va_arg(args, Point);
-			sprintf(tmp, "%ld %ld", valInt(p->x), valInt(p->y));
+			sprintf(tmp, INTPTR_FORMAT " " INTPTR_FORMAT,
+				valInt(p->x), valInt(p->y));
 			putString(tmp);
 			continue;
 		      }
 	  case 't':   { Figure f = va_arg(args, Figure);
-			sprintf(tmp, "%ld %ld translate",
+			sprintf(tmp, INTPTR_FORMAT " " INTPTR_FORMAT " translate",
 				valInt(f->offset->x), valInt(f->offset->y));
 			putString(tmp);
 			continue;
 		      }
 	  case 'p':   { Graphical gr = va_arg(args, Graphical);
-			sprintf(tmp, "%ld", valInt(gr->pen));
+			sprintf(tmp, INTPTR_FORMAT, valInt(gr->pen));
 			putString(tmp);
 			continue;
 		      }
@@ -244,25 +246,25 @@ _output(char *fm, va_list args)
 		      }
 	  case 'x':   { Graphical gr = va_arg(args, Graphical);
 
-			sprintf(tmp, "%ld", valInt(gr->area->x));
+			sprintf(tmp, INTPTR_FORMAT, valInt(gr->area->x));
 			putString(tmp);
 			continue;
 		      }
 	  case 'y':   { Graphical gr = va_arg(args, Graphical);
 
-			sprintf(tmp, "%ld", valInt(gr->area->y));
+			sprintf(tmp, INTPTR_FORMAT, valInt(gr->area->y));
 			putString(tmp);
 			continue;
 		      }
 	  case 'w':   { Graphical gr = va_arg(args, Graphical);
 
-			sprintf(tmp, "%ld", valInt(gr->area->w));
+			sprintf(tmp, INTPTR_FORMAT, valInt(gr->area->w));
 			putString(tmp);
 			continue;
 		      }
 	  case 'h':   { Graphical gr = va_arg(args, Graphical);
 
-			sprintf(tmp, "%ld", valInt(gr->area->h));
+			sprintf(tmp, INTPTR_FORMAT, valInt(gr->area->h));
 			putString(tmp);
 			continue;
 		      }
@@ -504,8 +506,8 @@ header(Any gr, Area area, Bool ls)
   { scale = 1.0;
   } else
   { scale = min( (float)w / (float)wgr, (float)h/(float)hgr );
-    paperW = (int)((float)paperW * scale);
-    paperH = (int)((float)paperH * scale);
+    paperW = (int)((float)paperW * scale + 0.999);
+    paperH = (int)((float)paperH * scale + 0.999);
   }
 
   if ( ls == ON )
@@ -1472,10 +1474,18 @@ drawPostScriptLine(Line ln, Name hb)
 		ln, ln, x1, y1, x2-x1, y2-y1);
   
     if ( adjustFirstArrowLine(ln) )
+    { Colour old = ln->first_arrow->colour;
+      ln->first_arrow->colour = ln->colour;
       postscriptGraphical(ln->first_arrow, hb);
+      ln->first_arrow->colour = old;
+    }
     if ( adjustSecondArrowLine(ln) )
+    { Colour old = ln->second_arrow->colour;
+      ln->second_arrow->colour = ln->colour;
       postscriptGraphical(ln->second_arrow, hb);
-  
+      ln->second_arrow->colour = old;
+    }
+
     ps_output("grestore\n");
   }
 
@@ -1635,6 +1645,9 @@ drawPostScriptImage(Image image, Name hb)
 }
 
 
+#define Wrapped(t)      ((t)->wrap == NAME_wrap || \
+			 (t)->wrap == NAME_wrapFixedWidth)
+
 status
 drawPostScriptText(TextObj t, Name hb)
 { String s = &t->string->data;
@@ -1691,7 +1704,7 @@ drawPostScriptText(TextObj t, Name hb)
       if ( t->underline == ON )
 	flags |= TXT_UNDERLINED;
 
-      if ( t->wrap == NAME_wrap )
+      if ( Wrapped(t) )
       { LocalString(buf, s->iswide, s->size + MAX_WRAP_LINES);
       
 	str_format(buf, s, valInt(t->margin), t->font);
