@@ -1,4 +1,4 @@
-/*  $Id: graphics.h,v 1.60 2003/09/17 09:08:59 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -596,7 +596,18 @@ NewClass(edit_text_gesture)
   Bool		activate;		/* @on: focus on ->terminate */
 End;
 
-
+NewClass(key_binding)
+  ABSTRACT_RECOGNISER
+  Name		name;			/* Global name of this table */
+  Sheet		bindings;		/* Key-name --> function */
+  Chain		defaults;		/* Default tables to be used */
+  Any		default_function;	/* Function if not defined */
+  Name 		prefix;			/* Parsed prefix */
+  Int		argument;		/* Universal argument */
+  Name		status;			/* {universal_argument}* */
+  Int		saved_column;		/* {next,previous}_line column */
+  Code		condition;		/* General condition for activation */
+End;
 
 		/********************************
 		*          BROWSERS		*
@@ -669,6 +680,17 @@ NewClass(colour_map)
   WsRef		ws_ref;			/* Window system reference */
 End;
 
+typedef struct
+{ enum
+  { XBM_DATA,
+    XPM_DATA
+  } type;
+  union
+  { unsigned char *xbm;
+    char **xpm;
+  } bits;
+} builtin_image_data;
+
 #define ABSTRACT_IMAGE \
   ABSTRACT_VISUAL \
   Name		name;			/* Name of the image */ \
@@ -683,7 +705,8 @@ End;
   BitmapObj	bitmap;			/* Bitmap for read-write's */ \
   Point		hot_spot;		/* Indication of hot-spot */ \
   Image		mask;			/* Masking image */ \
-  WsRef		ws_ref;			/* Window system reference */
+  WsRef		ws_ref;			/* Window system reference */ \
+  builtin_image_data *bits;		/* Data for built-in images */
 
 NewClass(image)
   ABSTRACT_IMAGE
@@ -738,7 +761,7 @@ NewClass(fontobj)
   Int		ex;			/* Width of an x in this font */
   Name          x_name;			/* X-Name of the font */
   Bool		fixed_width;		/* Is this a fixed width font? */
-  Bool		b16;			/* Font is 16-bit font */
+  Bool		iswide;			/* Font is 16-bit font */
   Name		postscript_font;	/* Name of PostScript font */
   Int		postscript_size;	/* Size in PostScript */
 End;
@@ -750,6 +773,7 @@ NewClass(displayobj)
   Name		address;		/* Display address specification */
   HashTable	font_table;		/* Table holding font-mappings */
   Chain		frames;			/* Created frames on this display */
+  Chain		monitors;		/* Physical monitors attached */
   Chain		inspect_handlers;	/* Event-handlers for inspector */
   Colour	foreground;		/* Window default foreground */
   Colour	background;		/* Window default background */
@@ -776,7 +800,7 @@ NewClass(application)
   Name		name;			/* name of the application */
   Chain		members;		/* its member frames */
   Name		kind;			/* {user,service} */
-  FrameObj	modal;			/* Modal frame */
+  Chain		modal;			/* Stack of modal frames */
 End;
 
 
@@ -887,6 +911,12 @@ NewClass(timer)
   WsRef		ws_ref;			/* Window System Reference */
 End;
 
+NewClass(monitor)
+  Any		name;			/* Name of the physical monitor */
+  Area		area;			/* Area on the display */
+  Area		work_area;		/* Application area */
+  Bool		primary;		/* Monitor is primary */
+End;
 
 		 /*******************************
 		 *	      EVENTS		*
@@ -1095,7 +1125,7 @@ GLOBAL  Chain grabbedWindows;		/* @grabbed_windows */
 		 *	    IMAGE TYPES		*
 		 *******************************/
 
-extern int image_type_from_data(char *data, int size);
+COMMON(int) image_type_from_data(char *data, int size);
 
 #define IMG_IS_UNKNOWN	0
 #define IMG_IS_JPEG	1
