@@ -1,4 +1,4 @@
-/*  $Id: unix.h,v 1.14 2005/04/08 15:13:40 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -77,13 +77,13 @@
 #define ABSTRACT_STREAM \
   Code		input_message;		/* Message forwarded on input */ \
   Any		record_separator;	/* Separate input records */ \
-  int		wrfd;			/* FD to write to process */ \
-  int		rdfd;			/* FD to read from process */ \
+  long		wrfd;			/* FD to write to process */ \
+  long		rdfd;			/* FD to read from process */ \
   FILE *	rdstream;		/* Stream to read from process */ \
   WsRef		ws_ref;			/* Window System Handle */ \
   unsigned char * input_buffer;		/* Input buffer */ \
-  int		input_allocated;	/* Allocated size of buffer */ \
-  int		input_p;		/* Pointer into input buffer */
+  long		input_allocated;	/* Allocated size of buffer */ \
+  long		input_p;		/* Pointer into input buffer */
 
 
 NewClass(fileobj)
@@ -94,6 +94,7 @@ NewClass(fileobj)
   Name		status;			/* current open mode */
   Name		filter;			/* I/O filter used */
   Bool		bom;			/* Use/has BOM marker? */
+  Name		newline_mode;		/* Newline mode {dos,posix} */
   IOSTREAM     *fd;			/* file descriptor */
 End;
 
