@@ -1,4 +1,4 @@
-/*  $Id: srcsink.c,v 1.19 2005/11/15 13:22:07 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -169,7 +169,7 @@ getContentsSourceSink(SourceSink ss, Int from, Int len)
 
       setStreamEncodingSourceSink(ss, fd);
 
-      while( n < valInt(len) && (c=Sgetcode(fd)) != EOF )
+      while( n < size && (c=Sgetcode(fd)) != EOF )
       { if ( c > 0xff && !s.iswide )
 	{ charW *w = pceMalloc(s.size*sizeof(charW));
 	  charW *t = w;
