@@ -1,4 +1,4 @@
-/*  $Id: class.c,v 1.45 2005/11/02 15:28:12 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -1280,7 +1280,7 @@ sourceClass(Class class, SendFunc f, char *file, char *rcs)
   { static char rev[] = "$Revision: ";
     char *s, *q;
     char buf[100];
-    int l;
+    size_t l;
 
     for(s=rcs, q=rev; *q && *s == *q; s++, q++)
       ;
@@ -2257,7 +2257,7 @@ getSubClassesClass(Class class)
 
 status
 makeClassClass(Class class)
-{ sourceClass(class, makeClassClass, __FILE__, "$Revision: 1.45 $");
+{ sourceClass(class, makeClassClass, __FILE__, "$Revision$");
 
   localClass(class, NAME_name, NAME_name, "name", NAME_get,
 	     "Name of the class");
