@@ -1,4 +1,4 @@
-/*  $Id: goodies.c,v 1.41 2005/11/15 13:22:07 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -341,13 +341,13 @@ characterName(Any chr)
     default:
     ctrl:
       if ( c < ' ' )
-      {	int l;
+      {	size_t l;
 
 	wcscat(buf, L"\\C-");
 	buf[l=wcslen(buf)] = tolower(c + '@');
 	buf[l+1] = EOS;
       } else
-      { int l;
+      { size_t l;
 
         buf[l=wcslen(buf)] = c;
 	buf[l+1] = EOS;
