@@ -1,4 +1,4 @@
-/*  $Id: name.c,v 1.26 2005/11/15 13:22:07 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -225,7 +225,7 @@ initNamesPass1(void)
 
   for( name=(Name)builtin_names; name->data.s_text != NULL; name++)
   { str_inithdr(&name->data, FALSE);
-    name->data.size = strlen((char *)name->data.s_text);
+    name->data.size = (int)strlen((char *)name->data.s_text);
   }
 }
 
