@@ -1,4 +1,4 @@
-/*  $Id: menu.c,v 1.39 2003/12/01 14:52:27 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -1087,28 +1087,8 @@ openComboBoxMenu(Menu m)
 
 
 static status
-executeMenu(Menu m, EventObj ev)
-{ MenuItem mi;
-
-  if ( m->feedback == NAME_showSelectionOnly )
-  { Any img = getClassVariableValueObject(m, NAME_cycleIndicator); /* TBD */
-
-    if ( img == NAME_comboBox )
-      return openComboBoxMenu(m);
-    else
-    { nextMenu(m);
-      flushGraphical(m);
-      if ( !send(m->device, NAME_modifiedItem, m, ON, EAV) )
-	forwardMenu(m, m->message, ev);
-      succeed;
-    }
-  }
-
-  if ( isDefault(ev) )
-    ev = EVENT->value;			/* @event */
-  TRY((mi = getItemFromEventMenu(m, ev)) && mi->active == ON);
-    
-  if ( m->multiple_selection == ON )
+executeMenuItem(Menu m, MenuItem mi, EventObj ev)
+{ if ( m->multiple_selection == ON )
   { toggleMenu(m, mi);
     flushGraphical(m);
     send(m->device, NAME_modifiedItem, m, ON, EAV);
@@ -1140,6 +1120,32 @@ executeMenu(Menu m, EventObj ev)
 }
 
 
+static status
+executeMenu(Menu m, EventObj ev)
+{ MenuItem mi;
+
+  if ( m->feedback == NAME_showSelectionOnly )
+  { Any img = getClassVariableValueObject(m, NAME_cycleIndicator); /* TBD */
+
+    if ( img == NAME_comboBox )
+      return openComboBoxMenu(m);
+    else
+    { nextMenu(m);
+      flushGraphical(m);
+      if ( !send(m->device, NAME_modifiedItem, m, ON, EAV) )
+	forwardMenu(m, m->message, ev);
+      succeed;
+    }
+  }
+
+  if ( isDefault(ev) )
+    ev = EVENT->value;			/* @event */
+  TRY((mi = getItemFromEventMenu(m, ev)) && mi->active == ON);
+    
+  return executeMenuItem(m, mi, ev);
+}
+
+
 		/********************************
 		*           SELECTION		*
 		********************************/
@@ -1225,8 +1231,18 @@ index_item_menu(Menu m, Any spec)
     }
   } else
   { Cell cell;
-    int n = 1;
+    int n;
     
+    n = 1;				/* exact match */
+    for_cell(cell, m->members)
+    { MenuItem mi = cell->value;
+
+      if ( mi->value == spec )
+        return n;
+      n++;
+    }
+
+    n = 1;				/* equal string representation */
     for_cell(cell, m->members)
     { if ( hasValueMenuItem(cell->value, spec) )
         return n;
@@ -1261,18 +1277,24 @@ clearSelectionMenu(Menu m)
 }
 
 
-#define MAX_ITEMS 1000
 #define IS_SET 1
 #define NEEDS_SET 2
 
 status
 selectionMenu(Menu m, Any selection)
 { Cell cell;
-  char is_set[MAX_ITEMS];
-  int n;
+  char *is_set;
+  int size = valInt(m->members->size);
+  int n, do_free;
 
   DEBUG(NAME_popup, Cprintf("selectionMenu(%s, %s)\n", pp(m), pp(selection)));
 
+  if ( !(is_set = alloca((size+1)*sizeof(char))) )
+  { is_set = pceMalloc((size+1)*sizeof(char));
+    do_free = TRUE;
+  } else
+    do_free = FALSE;
+
   is_set[0] = 0;
   n = 1;
   for_cell(cell, m->members)
@@ -1299,17 +1321,36 @@ selectionMenu(Menu m, Any selection)
     if ( is_set[n] & NEEDS_SET )
     { if ( !(is_set[n] & IS_SET) )
       { assign(mi, selected, ON);
-	ChangedItemMenu(m, mi);
       }
     } else
     { if ( is_set[n] & IS_SET )
-      { assign(((MenuItem) cell->value), selected, OFF);
-	ChangedItemMenu(m, mi);
+      { assign(mi, selected, OFF);
       }
     }
     n++;
   }
 
+					/* 2nd pass to avoid a call-back */
+					/* through ensureSingleSelectionMenu */
+  n = 1;
+  for_cell(cell, m->members)
+  { MenuItem mi = cell->value;
+
+    if ( is_set[n] & NEEDS_SET )
+    { if ( !(is_set[n] & IS_SET) )
+      { ChangedItemMenu(m, mi);
+      }
+    } else
+    { if ( is_set[n] & IS_SET )
+      { ChangedItemMenu(m, mi);
+      }
+    }
+    n++;
+  }
+
+  if ( do_free )
+    pceFree(is_set);
+
   succeed;
 }
 
@@ -1600,8 +1641,7 @@ kindMenu(Menu m, Name kind)
       assign(m, feedback, NAME_box);
       assign(m, pen, ONE);
       assign(m, border, TWO);
-      assign(m, multiple_selection, kind == NAME_toggle ? ON : OFF);
-
+      multipleSelectionMenu(m, kind == NAME_toggle ? ON : OFF);
       assign(m, kind, kind);
       return requestComputeGraphical(m, DEFAULT);
     } else if ( kind == NAME_marked )
@@ -1610,7 +1650,7 @@ kindMenu(Menu m, Name kind)
       assign(m, feedback, NAME_image);
       assign(m, pen, ZERO);
       assign(m, border, TWO);
-      assign(m, multiple_selection, OFF);
+      multipleSelectionMenu(m, OFF);
       assign(m, kind, kind);
 
       return requestComputeGraphical(m, DEFAULT);
@@ -1624,7 +1664,7 @@ kindMenu(Menu m, Name kind)
       assign(m, feedback, NAME_image);
       assign(m, pen, ZERO);
       assign(m, border, toInt(3));
-      assign(m, multiple_selection, kind == NAME_toggle ? ON : OFF);
+      multipleSelectionMenu(m, kind == NAME_toggle ? ON : OFF);
 
       assign(m, kind, kind);
       return requestComputeGraphical(m, DEFAULT);
@@ -1634,7 +1674,7 @@ kindMenu(Menu m, Name kind)
       assign(m, feedback, NAME_box);
       assign(m, pen, ONE);
       assign(m, border, TWO);
-      assign(m, multiple_selection, OFF);
+      multipleSelectionMenu(m, OFF);
 
       assign(m, kind, kind);
       return requestComputeGraphical(m, DEFAULT);
@@ -1653,7 +1693,7 @@ kindMenu(Menu m, Name kind)
       assign(m, feedback, NAME_image);
       assign(m, pen, ZERO);
       assign(m, border, toInt(3));
-      assign(m, multiple_selection, kind == NAME_toggle ? ON : OFF);
+      multipleSelectionMenu(m, kind == NAME_toggle ? ON : OFF);
 
       assign(m, kind, kind);
       return requestComputeGraphical(m, DEFAULT);
@@ -1663,7 +1703,7 @@ kindMenu(Menu m, Name kind)
       assign(m, feedback, NAME_box);
       assign(m, pen, ONE);
       assign(m, border, TWO);
-      assign(m, multiple_selection, OFF);
+      multipleSelectionMenu(m, OFF);
 
       assign(m, kind, kind);
       return requestComputeGraphical(m, DEFAULT);
@@ -1755,7 +1795,9 @@ ensureSingleSelectionMenu(Menu m)
 
   if ( sel == 0 )
   { if ( notNil(first) )
+    { DEBUG(NAME_popup, Cprintf("%s: selecting first\n", pp(m)));
       return selectionMenu(m, first);
+    }
 
     fail;
   }
@@ -1766,7 +1808,11 @@ ensureSingleSelectionMenu(Menu m)
 
 static status
 multipleSelectionMenu(Menu m, Bool val)
-{ return assignGraphical(m, NAME_multipleSelection, val);
+{ assignGraphical(m, NAME_multipleSelection, val);
+  
+  get(m, NAME_selection, EAV);		/* update <-selection */
+
+  succeed;
 }
 
 
@@ -1896,15 +1942,21 @@ getModifiedMenu(Menu m)
     answer((mi && m->selection == mi->value) ? OFF : ON);
   } else
   { Cell cell;
-    char is_set[MAX_ITEMS];
-    int n;
+    int size = valInt(m->members->size);
+    char *is_set;
+    int n, do_free;
 
     if ( !instanceOfObject(m->selection, ClassChain) )
       answer(ON);
 
+    if ( !(is_set = alloca(size*sizeof(char))) )
+    { is_set = pceMalloc(size*sizeof(char));
+      do_free = TRUE;
+    } else
+      do_free = FALSE;
+
     n = 1;
     for_cell(cell, m->members)
-
       is_set[n++] = 0;
     
     for_cell(cell, (Chain) m->selection)
@@ -1919,10 +1971,17 @@ getModifiedMenu(Menu m)
 
       if ( (is_set[n] && mi->selected == OFF) ||
 	   (!is_set[n] && mi->selected == ON) )
+      { if ( do_free )
+	  pceFree(is_set);
+
 	answer(ON);
+      }
       n++;
     }
 
+    if ( do_free )
+      pceFree(is_set);
+
     answer(OFF);
   }
 }
@@ -1989,7 +2048,23 @@ applyMenu(Menu m, Bool always)
 
 static status
 assignAcceletatorsMenu(Menu m)
-{ return assignAccelerators(m->members, NAME_, NAME_label);
+{ return assignAccelerators(m->members, CtoName("\\e"), NAME_label);
+}
+
+
+static status
+keyMenu(Menu m, Name key)
+{ Cell cell;
+
+  for_cell(cell, m->members)
+  { MenuItem mi = cell->value;
+
+    if ( mi->accelerator == key )
+    { return executeMenuItem(m, mi, EVENT->value);
+    }
+  }
+
+  fail;
 }
 
 
@@ -2082,6 +2157,8 @@ static senddecl send_menu[] =
      DEFAULT, "Unlink from menu-items"),
   SM(NAME_assignAccelerators, 0, NULL, assignAcceletatorsMenu,
      NAME_accelerator, "Assign accelerators for the items"),
+  SM(NAME_key, 1, "key=name", keyMenu,
+     NAME_accelerator, "Handle accelerator key `name'"),
   SM(NAME_activeAllItems, 1, "active=bool", activeAllItemsMenu,
      NAME_active, "(De)activate all items in the menu"),
   SM(NAME_activeItem, 2, T_activeItem, activeItemMenu,
