@@ -1,4 +1,4 @@
-/*  $Id: mswindow.c,v 1.41 2006/02/24 10:44:52 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -24,7 +24,7 @@
 
 #include "include.h"
 
-static int WINAPI window_wnd_proc(HWND win, UINT msg, UINT wP, LONG lP);
+static LRESULT WINAPI window_wnd_proc(HWND w, UINT m, WPARAM wP, LPARAM lP);
 
 static int clearing_update;		/* from ws_redraw_window() */
 static int invert_window = FALSE;	/* invert the window */
@@ -266,7 +266,12 @@ do_window_wnd_proc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
 	
 	if ( oproc )			/* refining alien window */
 					/* see winHandleWindow() below */
-	{ SetWindowLong(hwnd, GWL_WNDPROC, (LONG) oproc);
+	{
+#if (_MSC_VER < 1400)
+	  SetWindowLong(hwnd, GWL_WNDPROC, (LONG) oproc);
+#else
+	  SetWindowLongPtr(hwnd, GWLP_WNDPROC, (LONG_PTR) oproc);
+#endif
 	  return CallWindowProc(oproc, hwnd, message, wParam, lParam);
 	}
 
@@ -374,8 +379,8 @@ cascade:
 }
 
 
-static int WINAPI
-window_wnd_proc(HWND hwnd, UINT message, UINT wParam, LONG lParam)
+static LRESULT WINAPI
+window_wnd_proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 { int rval;
 
   if ( InSendMessage() )
@@ -844,8 +849,13 @@ winHandleWindow(PceWindow sw, Int handle)
   setHwndWindow(sw, hwnd);
   assocObjectToHWND(hwnd, sw);
   w = sw->ws_ref;
+#if (_MSC_VER < 1400)
   w->saved_window_procedure = (WNDPROC)GetWindowLong(hwnd, GWL_WNDPROC);
   SetWindowLong(hwnd, GWL_WNDPROC, (LONG) window_wnd_proc);
+#else
+  w->saved_window_procedure = (WNDPROC)GetWindowLongPtr(hwnd, GWLP_WNDPROC);
+  SetWindowLongPtr(hwnd, GWLP_WNDPROC, (LONG_PTR) window_wnd_proc);
+#endif
   GetWindowRect(hwnd, &rect);
   ServiceMode(is_service_window(sw),
 	      { Area a = sw->area;
