@@ -1,4 +1,4 @@
-/*  $Id: chararray.c,v 1.38 2005/11/02 09:51:57 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -773,7 +773,7 @@ initCharArrays(void)
 CharArray
 CtoScratchCharArray(const char *s)
 { CharArray name = scratch_char_arrays;
-  int len = strlen(s);
+  size_t len = strlen(s);
   int n;
 
   for(n = 0; n < SCRATCH_CHAR_ARRAYS; n++, name++)
