@@ -1,4 +1,4 @@
-/*  $Id: str.c,v 1.29 2005/11/15 13:22:07 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -86,7 +86,7 @@ str_ring_alloc(String s)
   } else
   { str_ring[str_ring_ptr] = pceRealloc(str_ring[str_ring_ptr], size);
   }
-  s->s_textA = str_ring[str_ring_ptr];
+  s->s_textA = (charA*)str_ring[str_ring_ptr];
   s->readonly = TRUE;
 
   if ( ++str_ring_ptr == STR_RING_SIZE )
@@ -123,12 +123,12 @@ fstr_inithdr(String s, int iswide, void *data, int len)
 
 
 status
-str_set_n_ascii(String str, int len, char *text)
+str_set_n_ascii(String str, size_t len, char *text)
 { if ( len > STR_MAX_SIZE )
     return errorPce(NIL, NAME_stringTooLong, toInt(len));
 
   str_inithdr(str, FALSE);
-  str->size = len;
+  str->size = (int)len;
   str->s_textA = (charA *) text;
 
   succeed;
@@ -136,12 +136,12 @@ str_set_n_ascii(String str, int len, char *text)
 
 
 status
-str_set_n_wchar(String str, int len, wchar_t *text)
+str_set_n_wchar(String str, size_t len, wchar_t *text)
 { if ( len > STR_MAX_SIZE )
     return errorPce(NIL, NAME_stringTooLong, toInt(len));
 
   str_inithdr(str, TRUE);
-  str->size = len;
+  str->size = (int)len;
   str->s_textW = text;
 
   succeed;
@@ -150,7 +150,7 @@ str_set_n_wchar(String str, int len, wchar_t *text)
 
 status
 str_set_ascii(String str, char *text)
-{ int len = strlen(text);
+{ size_t len = strlen(text);
 
   return str_set_n_ascii(str, len, text);
 }
@@ -193,14 +193,14 @@ str_set_static(): initialise a string from a static C-string
 
 status
 str_set_static(String str, const char *text)
-{ int len = strlen(text);
+{ size_t len = strlen(text);
 
   if ( len > STR_MAX_SIZE )
     return errorPce(NIL, NAME_stringTooLong, toInt(len));
 
   str_inithdr(str, FALSE);
   str->readonly = TRUE;
-  str->size = len;
+  str->size = (int)len;
   str->s_textA = (charA *) text;
 
   succeed;
@@ -650,7 +650,7 @@ str_next_index(String s, int from, wint_t chr)
   { charW *d = &s->s_textW[from];
     
     for(i=from; i<n; i++, d++)
-      if ( *d == chr )
+      if ( (wint_t)*d == chr )
 	return i;
   }
 
@@ -672,7 +672,7 @@ str_next_rindex(String s, int from, wint_t chr)
   { charW *d = &s->s_textW[from];
     
     for(i=from; i >= 0; i--, d--)
-      if ( *d == chr )
+      if ( (wint_t)*d == chr )
 	return i;
   }
 
@@ -707,7 +707,7 @@ str_count_chr(String s, int from, int to, wint_t chr)
   { charW *d = &s->s_textW[from];
     
     for(i=from; i<to; i++, d++)
-      if ( *d == chr )
+      if ( (wint_t)*d == chr )
 	count++;
   }
 
@@ -756,7 +756,7 @@ str_from_char16(String s, int c)
   text[0] = c;
   text[1] = '\0';
 
-  str_inithdr(s, FALSE);
+  str_inithdr(s, TRUE);
   s->s_textW = text;
   s->size     = 1;
 }
@@ -828,19 +828,36 @@ str_strip(String s)
     charA *t = s->s_textA;
     charA *e = &s->s_textA[size];
     
-    while( f < e && isblank(*f) )
+    while( f < e && iswspace(*f) )	/* ISO-Latin-1 */
       f++;
 
     do
-    { while( f < e && !isblank(*f) )
+    { while( f < e && !iswspace(*f) )
 	*t++ = *f++;
-      while( f < e && isblank(*f) )
+      while( f < e && iswspace(*f) )
 	f++;
       if ( f < e )
 	*t++ = ' ';
     } while( f < e );
+    s->size = t - s->s_textA;
   } else
-    errorPce(CtoName("str_strip()"), NAME_notSupportedForChar16);
+  { charW *f = s->s_textW;
+    charW *t = s->s_textW;
+    charW *e = &s->s_textW[size];
+    
+    while( f < e && iswspace(*f) )
+      f++;
+
+    do
+    { while( f < e && !iswspace(*f) )
+	*t++ = *f++;
+      while( f < e && iswspace(*f) )
+	f++;
+      if ( f < e )
+	*t++ = ' ';
+    } while( f < e );
+    s->size = t - s->s_textW;
+  }
 }
 
 
