@@ -1,4 +1,4 @@
-/*  $Id: syntax.c,v 1.15 2005/10/10 14:15:16 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -73,7 +73,7 @@ swapBytesTable(SyntaxTable t)
 {
 #define swapchr(x, y)	{ unsigned char z; z=x; x=y; y=z; }
   { unsigned char *s;
-    int i = FLAGS_SIZE(t);
+    size_t i = FLAGS_SIZE(t);
 
     for(s = (unsigned char *)t->table; i > 0; s += 2, i-= 2)
     { swapchr(s[0], s[1]);
@@ -370,7 +370,7 @@ makeClassSyntaxTable(Class class)
 	     "comment_start,comment_end,"
 	     "letter,word,layout}");
 
-  sourceClass(class, makeClassSyntaxTable, __FILE__, "$Revision: 1.15 $");
+  sourceClass(class, makeClassSyntaxTable, __FILE__, "$Revision$");
 
   localClass(class, NAME_name, NAME_name, "name*", NAME_get,
 	     "Name of this syntax table");
