@@ -1,4 +1,4 @@
-/*  $Id: file.c,v 1.39 2003/09/01 09:40:16 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -49,19 +49,29 @@
 #include <linux/limits.h>
 #endif
 
-static Sheet FileFilters;
+static status	kindFile(FileObj f, Name encoding);
+static Sheet	FileFilters;
 
 static status
-initialiseFile(FileObj f, Name name, Name kind)
-{ initialiseSourceSink((SourceSink)f);
+initialiseFile(FileObj f, Name name, Name encoding)
+{ Name fn;
 
-  if ( isDefault(kind) )
-    kind = NAME_text;
+  initialiseSourceSink((SourceSink)f);
+
+  if ( isDefault(encoding) )
+    encoding = NAME_text;
 
   assign(f, status, NAME_closed);
+  assign(f, bom, DEFAULT);
   assign(f, path, DEFAULT);
-  assign(f, kind, kind);
+#ifdef __WINDOWS__
+  assign(f, newline_mode, NAME_dos);
+#else
+  assign(f, newline_mode, NAME_posix);
+#endif
   f->fd = NULL;
+  
+  kindFile(f, encoding);
 
   if ( isDefault(name) )
   {
@@ -78,7 +88,7 @@ initialiseFile(FileObj f, Name name, Name kind)
 
     if ( (fileno = mkstemp(namebuf)) < 0 )
       return errorPce(f, NAME_openFile, NAME_write, getOsErrorPce(PCE));
-    if ( (f->fd = fdopen(fileno, "w")) == NULL )
+    if ( (f->fd = Sfdopen(fileno, "w")) == NULL )
     { close(fileno);
       return errorPce(f, NAME_openFile, NAME_write, getOsErrorPce(PCE));
     }
@@ -97,7 +107,7 @@ initialiseFile(FileObj f, Name name, Name kind)
     { return errorPce(f, NAME_noTempFile, getOsErrorPce(PCE));
     }
 #else
-#ifdef HAVE_TEMPNAM			/* Prefer this on WIN32 */
+#ifdef HAVE_TEMPNAM			/* Prefer this on __WINDOWS__ */
     char *s = tempnam("c:\\tmp", "xpce");
 
     if ( s )
@@ -115,42 +125,70 @@ initialiseFile(FileObj f, Name name, Name kind)
 #endif
   }
 
-#if O_XOS
-  { char buf[MAXPATHLEN];
-    char lng[MAXPATHLEN];
-    
-    _xos_long_file_name(strName(name), lng);
-    _xos_canonical_filename(lng, buf);
-    assign(f, name, CtoName(buf));
-  }
-#else
-  assign(f, name, name);
-#endif
-
-
-  succeed;
+  if ( (fn=expandFileName(name)) )
+  { assign(f, name, fn);
+    succeed;
+  } else
+    fail;
 }
 
 
+Name
+expandFileName(Name in)
+{ wchar_t expanded[MAXPATHLEN];
+  int len;
+
+  if ( (len=expandFileNameW(charArrayToWC((CharArray)in, NULL),
+			    expanded, MAXPATHLEN)) > 0 )
+  {
+#if O_XOS
+     wchar_t lng[MAXPATHLEN];
+     char buf[MAXPATHLEN];
+  
+     if ( _xos_long_file_nameW(expanded, lng, MAXPATHLEN) &&
+	  _xos_canonical_filenameW(lng, buf, sizeof(buf), 0) )
+     { return UTF8ToName(buf);
+     } else
+     { errorPce(in, NAME_representation, NAME_nameTooLong);
+       fail;
+     }    
+#else
+     return WCToName(expanded, len);
+#endif
+  }
+
+  fail;
+}
+
+
+
 static status
-kindFile(FileObj f, Name kind)
+kindFile(FileObj f, Name encoding)
 { if ( f->status != NAME_closed )
     return errorPce(f, NAME_noChangeAfterOpen);
 
-  assign(f, kind, kind);
-
+  if ( encoding == NAME_text )
+  { if ( !isName(f->encoding) )
+      assign(f, encoding, getClassVariableValueObject(f, NAME_encoding));
+    assign(f, kind, NAME_text);
+  } else if ( encoding == NAME_binary || encoding == NAME_octet )
+  { assign(f, kind, NAME_binary);
+    assign(f, encoding, NAME_octet);
+  } else
+  { assign(f, encoding, encoding);
+    assign(f, kind, NAME_text);
+  }
+  
   succeed;
 }
 
 
 Name
 getOsNameFile(FileObj f)
-{ char bin[MAXPATHLEN];
+{ if ( notDefault(f->path) )
+    answer(f->path);
 
-  if ( notDefault(f->path) )
-    answer(CtoName(strName(f->path)));
-
-  answer(CtoName(expandFileName(strName(f->name), bin)));
+  answer(f->name);
 }
 
 
@@ -172,9 +210,20 @@ loadFile(FileObj f, IOSTREAM *fd, ClassDef def)
 
   if ( isNil(f->path) )
     assign(f, path, DEFAULT);		/* backward compatibility load */
-  if ( isNil(f->kind) )
+  if ( !isName(f->kind) )
     assign(f, kind, NAME_binary);	/* same */
-
+  if ( !isName(f->encoding) )
+    assign(f, encoding, (f->kind == NAME_binary ? NAME_octet : NAME_text));
+  if ( !isDefault(f->bom) && !isBoolean(f->bom) )
+    assign(f, bom, DEFAULT);
+  if ( !isName(f->newline_mode) )
+  {
+#ifdef __WINDOWS__
+    assign(f, newline_mode, NAME_dos);
+#else
+    assign(f, newline_mode, NAME_posix);
+#endif
+  }
   assign(f, status, NAME_closed);
   f->fd = NULL;
 
@@ -193,12 +242,7 @@ closeFile(FileObj f)
 { if ( f->status != NAME_closed )
   { status rval = checkErrorFile(f);
 
-#ifdef HAVE_POPEN
-    if ( notNil(f->filter) )
-      pclose(f->fd);
-    else
-#endif
-      fclose(f->fd);
+    Sclose(f->fd);
     f->fd = NULL;
     assign(f, status, NAME_closed);
 
@@ -211,26 +255,21 @@ closeFile(FileObj f)
 
 status
 existsFile(FileObj f, Bool mustbefile)
-{ struct stat buf;
-  Name name;
+{ STAT_TYPE buf;
+  const char *fn = charArrayToFN((CharArray)f->name);
 
-  if ( (name = getOsNameFile(f)) )
-  { DEBUG(NAME_file, Cprintf("name=\"%s\"\n", strName(name)));
 #ifdef HAVE_ACCESS
-    if ( mustbefile == OFF )
-    { if ( access(strName(name), F_OK) == 0 )
-	succeed;
-      fail;
-    }
-#endif
-    if ( stat(strName(name), &buf) == -1 )
-      fail;
-    if ( mustbefile != OFF && (buf.st_mode & S_IFMT) != S_IFREG )
-      fail;
-    succeed;
+  if ( mustbefile == OFF )
+  { if ( access(fn, F_OK) == 0 )
+      succeed;
+    fail;
   }
-
-  fail;
+#endif
+  if ( STAT_FUNC(fn, &buf) == -1 )
+    fail;
+  if ( mustbefile != OFF && (buf.st_mode & S_IFMT) != S_IFREG )
+    fail;
+  succeed;
 }
 
 
@@ -240,15 +279,7 @@ sameOsPath(const char *s1, const char *s2)
     succeed;
 
 #if O_XOS
-					/* verylongfilename == verylong !!! */
-  { char b1[MAXPATHLEN];
-    char b2[MAXPATHLEN];
-
-    _xos_limited_os_filename(s1, b1);
-    _xos_limited_os_filename(s2, b2);
-    if ( streq(b1, b2) )
-      succeed;
-  }
+  return _xos_same_file(s1, s2);
 #endif
 
 #if __unix__
@@ -282,58 +313,35 @@ sameFile(FileObj f1, FileObj f2)
 
 static status
 absolutePathFile(FileObj f)
-{ char bin[MAXPATHLEN];
-  char *path = absolutePath(expandFileName(strName(f->name), bin));
+{ char path[MAXPATHLEN];
 
-  if ( path )
-  { assign(f, path, CtoName(path));
+  if ( absolutePath(charArrayToUTF8((CharArray)f->name), path, sizeof(path)) > 0 )
+  { assign(f, path, UTF8ToName(path));
     succeed;
   }
 
-  fail;
+  return errorPce(f, NAME_representation, NAME_nameTooLong);
 }
 
 
 Name
 getAbsolutePathFile(FileObj f)
-{ char bin[MAXPATHLEN];
+{ char path[MAXPATHLEN];
 
   if ( notDefault(f->path) )
     answer(f->path);
 
-  answer(CtoName(absolutePath(expandFileName(strName(f->name), bin))));
-}
-
-
-static int
-is_absolute_name(const char *s)
-{
-#ifdef __WIN32__
-  if ( isletter(s[0]) && s[1] == ':' )
-    succeed;
-  if ( IsDirSep(s[0]) && IsDirSep(s[1]) )
-    succeed;
-#endif
-  if ( IsDirSep(s[0]) )
-    succeed;
+  if ( absolutePath(charArrayToUTF8((CharArray)f->name), path, sizeof(path)) > 0 )
+    return UTF8ToName(path);
 
+  errorPce(f, NAME_representation, NAME_nameTooLong);
   fail;
 }
 
+
 status
 isAbsoluteFile(FileObj f)
-{ char bin[MAXPATHLEN];
-  char *name = strName(f->name);
-  int n;
-
-  for(n=0; n < 2; n++)
-  { if ( is_absolute_name(name) )
-      succeed;
-
-    name = expandFileName(name, bin);
-  }
-
-  fail;
+{ return isAbsolutePath(charArrayToUTF8((CharArray)f->name));
 }
 
 
@@ -343,7 +351,7 @@ isAbsoluteFile(FileObj f)
 #define O_BINARY 0
 #endif
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #include <fcntl.h>
 #endif
 
@@ -357,20 +365,17 @@ open_file(FileObj f, int access, ...)
 { va_list args;
   int mode;
   int fd = -1;
-  Name name;
   
   va_start(args, access);
   mode = va_arg(args, int);
   va_end(args);
 
-  if ( (name = getOsNameFile(f)) )
-  { fd = open(strName(name), access, mode);
+  fd = open(charArrayToFN((CharArray)f->name), access, mode);
 
-    if ( fd < 0 )
-      errorPce(f, NAME_openFile,
-	       (access & O_RDONLY) ? NAME_read : NAME_write,
-	       getOsErrorPce(PCE));
-  }
+  if ( fd < 0 )
+    errorPce(f, NAME_openFile,
+	     (access & O_RDONLY) ? NAME_read : NAME_write,
+	     getOsErrorPce(PCE));
 
   return fd;
 }
@@ -419,90 +424,29 @@ out:
 }
 
 
-#ifdef __WIN32__
 static status
-dos_backup_name(char *old, char *ext, char *bak)
-{ char base[100];
-  char dir[MAXPATHLEN];
-
-  strcpy(base, baseName(old));
-  strcpy(dir, dirName(old));
-
-  if ( ext[0] == '.' )
-  { char *s;
-
-    if ( (s = strchr(base, '.')) )
-      strcpy(s, ext);
-  } else
-  { char *s;
-
-    if ( (s = strchr(base, '.')) )
-    { int l, e;
-
-      s++;				/* skip the '.' */
-      l = strlen(s);
-      e = strlen(ext);
-      if ( (l + e) > 3 )
-      { l = 3-e;
-	if ( l < 0 )
-	  l = 0;
-	strncpy(&s[l], ext, 3);
-	s[3] = EOS;
-      } else
-	strcat(s, ext);
-    } else
-    { int l;
-
-      strcat(base, ext);
-      if ( (l=strlen(base)) > 8 )
-      { memmove(&base[9], &base[8], l-8);
-	base[8] = '.';
-	base[l+1] = EOS;
-      }
-    }
-  }
-
-  if ( dir[0] )
-    sprintf(bak, "%s/%s", dir, base);
-  else
-    strcpy(bak, base);
-
-  DEBUG(NAME_backup, Cprintf("Backup %s in %s\n", old, bak));
-
-  succeed;
-}
-
-status
-backup_name(char *old, char *ext, char *bak)
-{ if ( iswin32s() )
-    return dos_backup_name(old, ext, bak);
-  else
+backup_name(const char *old, const char *ext, char *bak, size_t len)
+{ if ( strlen(old) + strlen(ext) + 1 <= len )
   { sprintf(bak, "%s%s", old, ext);
-
     succeed;
+  } else
+  { errno = ENAMETOOLONG;
+    fail;
   }
 }
 
-#else /*__WIN32__*/
-
-status
-backup_name(char *old, char *ext, char *bak)
-{ sprintf(bak, "%s%s", old, ext);
-
-  succeed;
-}
-
-#endif /*__WIN32__*/
 
 static Name
 getBackupFileNameFile(FileObj f, Name ext)
-{ char bak[MAXPATHLEN];
+{ char bak[MAXPATHLEN*2];
 
-  backup_name(strName(getOsNameFile(f)),
-	      isDefault(ext) ? "~" : strName(ext),
-	      bak);
+  if ( backup_name(nameToUTF8(f->name),
+		   isDefault(ext) ? "~" : nameToUTF8(ext),
+		   bak, sizeof(bak)) )
+    answer(UTF8ToName(bak));
 
-  answer(CtoName(bak));
+  errorPce(f, NAME_representation, NAME_nameTooLong);
+  fail;
 }
 
 
@@ -510,13 +454,13 @@ static status
 backupFile(FileObj f, Name ext)
 { if ( existsFile(f, ON) )
   { Name newname = get(f, NAME_backupFileName, ext, EAV);
-    char *new;
-    char *old = strName(getOsNameFile(f));
+    const char *new;
+    const char *old = nameToFN(getOsNameFile(f));
     int fdfrom = -1, fdto = -1;
     status rval = FAIL;
 
     if ( newname )
-      new = strName(newname);
+      new = nameToFN(newname);
     else
       fail;				/* or succeed? */
 
@@ -583,7 +527,7 @@ getFilterFile(FileObj f)
   { char path[MAXPATHLEN];
     Attribute a = cell->value;
     Name extension = a->name;
-    struct stat buf;
+    STAT_TYPE buf;
 
     if ( !isName(extension) )
     { errorPce(extension, NAME_unexpectedType, TypeName);
@@ -591,7 +535,7 @@ getFilterFile(FileObj f)
     }
 
     sprintf(path, "%s%s", strName(f->name), strName(extension));
-    if ( stat(path, &buf) == 0 &&
+    if ( STAT_FUNC(path, &buf) == 0 &&
 	 (buf.st_mode & S_IFMT) == S_IFREG )
     { if ( !isName(a->value) )
       { errorPce(a->value, NAME_unexpectedType, TypeName);
@@ -606,6 +550,39 @@ getFilterFile(FileObj f)
 }
 
 
+status
+doBOMFile(FileObj f)
+{ assert(f->fd);			/* must be open */
+
+  if ( f->kind == NAME_text )
+  { if ( f->status == NAME_read )
+    { if ( f->bom != OFF )
+      { if ( ScheckBOM(f->fd) < 0 )
+	{ error:
+  
+	  reportErrorFile(f);
+	  closeFile(f);
+  
+	  fail;
+	}
+  
+	assign(f, bom, (f->fd->flags & SIO_BOM) ? ON : OFF);
+	if ( f->bom == ON )
+	  assign(f, encoding, encoding_to_name(f->fd->encoding));
+      }
+    } else				/* write */
+    { if ( f->bom == ON )
+      { if ( SwriteBOM(f->fd) < 0 )
+	{ goto error;
+	}
+      }
+    }
+  }
+
+  succeed;
+}
+
+
 status
 openFile(FileObj f, Name mode, Name filter, CharArray extension)
 { CharArray path;
@@ -640,11 +617,7 @@ openFile(FileObj f, Name mode, Name filter, CharArray extension)
     fdmode[0] = 'r';
 
   if ( f->kind == NAME_text )
-#ifdef __WATCOMC__
-    fdmode[1] = 't';
-#else
     fdmode[1] = '\0';
-#endif
   else
     fdmode[1] = 'b';
 
@@ -653,22 +626,25 @@ openFile(FileObj f, Name mode, Name filter, CharArray extension)
   if ( isNil(filter) )
   { DEBUG(NAME_file, Cprintf("Opening %s (%s) using mode %s\n",
 			     pp(f->name), pp(f), fdmode));
-    f->fd = fopen(strName(path), fdmode);
+    f->fd = Sopen_file(charArrayToFN(path), fdmode);
   } else
 #ifndef HAVE_POPEN
   { return errorPce(f, NAME_noPopen);
   }
 #else
   { char cmd[LINESIZE];
+    const char *fn = nameToFN(filter);
+    const char *pn = charArrayToFN(path);
+    const char *rn = (mode == NAME_read ? "<" : mode == NAME_write ? ">" : ">>");
 
     if ( fdmode[0] == 'a' )
       fdmode[0] = 'w';
 
-    sprintf(cmd, "%s %s %s",
-	    strName(filter), 
-	    mode == NAME_read ? "<" : mode == NAME_write ? ">" : ">>",
-	    strName(path));
-    f->fd = popen(cmd, fdmode);
+    if ( strlen(fn)+strlen(pn)+7 > LINESIZE )
+      return errorPce(f, NAME_representation, NAME_nameTooLong);
+
+    sprintf(cmd, "%s %s \"%s\"", fn, rn, pn);
+    f->fd = Sopen_pipe(cmd, fdmode);
   }
 #endif /*HAVE_POPEN*/
 
@@ -686,10 +662,27 @@ openFile(FileObj f, Name mode, Name filter, CharArray extension)
     return errorPce(f, NAME_openFile, mode, getOsErrorPce(PCE));
   }
 
-  assign(f, filter, filter);
   if ( mode == NAME_append )
     mode = NAME_write;
   assign(f, status, mode);
+  assign(f, filter, filter);
+
+  if ( mode == NAME_read )
+  { if ( !doBOMFile(f) )
+      fail;
+    if ( !setStreamEncodingSourceSink((SourceSink)f, f->fd) )
+    { closeFile(f);
+      fail;
+    }
+  } else
+  { if ( !setStreamEncodingSourceSink((SourceSink)f, f->fd) )
+    { closeFile(f);
+      fail;
+    }
+
+    if ( mode != NAME_append && !doBOMFile(f) )
+      fail;
+  }
 
   succeed;
 }
@@ -699,12 +692,9 @@ status
 removeFile(FileObj f)
 { Name name = getOsNameFile(f);
 
-  if ( !name )
-    fail;
-
   closeFile(f);				/* Ok? */
 
-  if ( unlink(strName(name)) == 0 )
+  if ( remove(nameToFN(name)) == 0 )
     succeed;
   if ( existsFile(f, OFF) )
     return errorPce(f, NAME_removeFile, getOsErrorPce(PCE));
@@ -716,26 +706,27 @@ removeFile(FileObj f)
 static status
 nameFile(FileObj f, Name name)
 { int rval;
-  Name nm = getOsNameFile(f);
-  char *old, new[MAXPATHLEN];
+  Name ofn = getOsNameFile(f);
+  Name nfn = expandFileName(name);
 
-  if ( !nm || !expandFileName(strName(name), new) )
+  if ( !nfn )
     fail;
-  old = strName(nm);
 
   if ( existsFile(f, OFF) )
-  {
+  { const char *ofns = nameToFN(ofn);
+    const char *nfns = nameToFN(nfn);
+
 #ifdef HAVE_RENAME
-    remove(new);
-    rval = rename(old, new);
+    remove(nfns);
+    rval = rename(ofns, nfns);
 #else
-    unlink(new);
-    if ((rval = link(old, new)) == 0 && (rval = unlink(old)) != 0)
-      unlink(new);
+    unlink(nfns);
+    if ((rval = link(ofns, nfns)) == 0 && (rval = unlink(ofns)) != 0)
+      unlink(nfns);
 #endif /*__unix__*/
 
     if ( rval == 0 )
-    { assign(f, name, name);
+    { assign(f, name, nfn);
       succeed;
     }
 
@@ -762,7 +753,7 @@ static Int
 getIndexFile(FileObj f)
 { TRY( check_file(f, NAME_open) );
 
-  answer(toInt(ftell(f->fd)));
+  answer(toInt(Stell(f->fd)));
 }
 
 
@@ -775,7 +766,7 @@ seekFile(FileObj f, Int index, Name whence)
   if ( isDefault(whence) )
     whence = NAME_start;
 
-  if ( fseek(f->fd, valInt(index), whence == NAME_start ? 0 :
+  if ( Sseek(f->fd, valInt(index), whence == NAME_start ? 0 :
 				   whence == NAME_here ? 1 :
 				   2 ) == -1 )
     return errorPce(f, NAME_seekFile, index, whence, getOsErrorPce(PCE));
@@ -787,10 +778,32 @@ seekFile(FileObj f, Int index, Name whence)
 static status
 append_file(FileObj f, String str)
 { TRY( check_file(f, NAME_write) );
-  fwrite(str->s_text,
-	 isstr8(str) ? sizeof(char8) : sizeof(char16),
-	 str->size, 
-	 f->fd);
+
+  if ( f->encoding == NAME_octet )
+  { if ( Sfwrite(str->s_text,
+		 isstrA(str) ? sizeof(charA) : sizeof(charW),
+		 str->size, 
+		 f->fd) != str->size )
+      return reportErrorFile(f);
+  } else
+  { if ( isstrA(str) )
+    { const charA *s = str->s_textA;
+      const charA *e = &s[str->size];
+  
+      for(; s<e; s++)
+      { if ( Sputcode(*s, f->fd) < 0 )
+	  return reportErrorFile(f);
+      }
+    } else
+    { const charW *s = str->s_textW;
+      const charW *e = &s[str->size];
+  
+      for(; s<e; s++)
+      { if ( Sputcode(*s, f->fd) < 0 )
+	  return reportErrorFile(f);
+      }
+    }
+  }  
 
   succeed;
 }
@@ -823,28 +836,42 @@ formatFile(FileObj f, CharArray fmt, int argc, Any *argv)
 static status
 flushFile(FileObj f)
 { if ( f->fd )
-    fflush(f->fd);
+    Sflush(f->fd);
 
   succeed;
 }
 
 
-static Int
-getSizeFile(FileObj f)
-{ struct stat buf;
-  int rval;
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+We can use the faster fstat() here, but confirmed by various messages on
+the web, MS-Windows implementation  of   _fstat()  is  broken, returning
+EBADF for perfectly valid filedescriptors.  Shouldn't make a difference,
+only slow ...
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-  if ( f->fd != NULL )
-    rval = fstat(fileno(f->fd), &buf);
-  else
+static int
+statFile(FileObj f, STAT_TYPE *buf)
+{ 
+#ifndef __WINDOWS__
+  int fno;
+
+  if ( f->fd != NULL && (fno = Sfileno(f->fd)) >= 0)
+  { return FSTAT_FUNC(fno, buf);
+  } else
+#endif
   { Name name = getOsNameFile(f);
 
-    if ( !name )
-      fail;
-    rval = stat(strName(name), &buf);
+    return STAT_FUNC(nameToFN(name), buf);
   }
+}
 
-  if ( rval == -1 )
+
+
+static Int
+getSizeFile(FileObj f)
+{ STAT_TYPE buf;
+
+  if ( statFile(f, &buf) == -1 )
   { errorPce(f, NAME_cannotStat, getOsErrorPce(PCE));
     fail;
   }
@@ -855,21 +882,17 @@ getSizeFile(FileObj f)
 
 static Date
 getTimeFile(FileObj f, Name which)
-{ struct stat buf;
-  Name name = getOsNameFile(f);
-
-  if ( !name )
-    fail;
+{ STAT_TYPE buf;
 
   if ( isDefault(which) )
     which = NAME_modified;
 
-  if ( stat(strName(name), &buf) < 0 )
+  if ( statFile(f, &buf) < 0 )
   { errorPce(f, NAME_cannotStat, getOsErrorPce(PCE));
     fail;
   }
   
-  if ( equalName(which, NAME_modified) )
+  if ( which == NAME_modified )
     answer(CtoDate(buf.st_mtime));
   else
     answer(CtoDate(buf.st_atime));
@@ -878,58 +901,47 @@ getTimeFile(FileObj f, Name which)
 
 Name
 getBaseNameFile(FileObj f)
-{ char bin[MAXPATHLEN];
+{ const char *ufn = nameToUTF8(f->name);
 
-  answer(CtoName(baseName(expandFileName(strName(f->name), bin))));
+  answer(UTF8ToName(baseName(ufn)));
 }
 
 
 static Name
 getDirectoryNameFile(FileObj f)
-{ char bin[MAXPATHLEN];
+{ char dir[MAXPATHLEN];
 
-  answer(CtoName(dirName(expandFileName(strName(f->name), bin))));
+  dirName(nameToUTF8(getOsNameFile(f)), dir, sizeof(dir));
+
+  answer(UTF8ToName(dir));
 }
 
 
 static StringObj
 getReadLineFile(FileObj f)
-{ char tmp[1024];
-  char *s = tmp;
-  int allocated = sizeof(tmp);
-  int len = 0;
-  string str;
+{ tmp_string tmp;
   StringObj rval;
 
   TRY( check_file(f, NAME_read) );
 
+  str_tmp_init(&tmp);
+
   for(;;)
-  { int c = getc(f->fd);
+  { int c = Sgetcode(f->fd);
 
     if ( c == EOF )
-    { if ( len == 0 )
+    { if ( tmp.s.size == 0 )
 	fail;
       break;
     }
-    if ( len >= allocated )
-    { allocated *= 2;
 
-      if ( s == tmp )
-      { s = pceMalloc(allocated);
-	memcpy(s, tmp, sizeof(tmp));
-      } else
-      { s = pceRealloc(s, allocated);
-      }
-    }
-    s[len++] = c;
+    str_tmp_put(&tmp, (wint_t)c);
     if ( c == '\n' )
       break;
   }
       
-  str_set_n_ascii(&str, len, s);
-  rval = StringToString(&str);
-  if ( s != tmp )
-    pceFree(s);
+  rval = StringToString(&tmp.s);
+  str_tmp_done(&tmp);
 
   return rval;
 }
@@ -937,8 +949,7 @@ getReadLineFile(FileObj f)
 
 static StringObj
 getReadFile(FileObj f, Int n)
-{ int size;
-  int m;
+{ size_t size;
   StringObj s;
 
   TRY( check_file(f, NAME_read) );
@@ -957,19 +968,32 @@ getReadFile(FileObj f, Int n)
     fail;
   }
 
-  s = answerObject(ClassString, EAV);
-  str_unalloc(&s->data);
-  str_inithdr(&s->data, ENC_ASCII);
-  s->data.size = size;
-  str_alloc(&s->data);
+  if ( f->encoding == NAME_octet )
+  { size_t m;
 
-  if ( (m = fread(s->data.s_text8, 1, size, f->fd)) != size )
-  { if ( m >= 0 )
-      deleteString(s, toInt(m), DEFAULT);
-    else
-    { errorPce(f, NAME_ioError, getOsErrorPce(PCE), 0);
+    s = answerObject(ClassString, EAV);
+    str_unalloc(&s->data);
+    str_inithdr(&s->data, FALSE);
+    s->data.size = (int)size;
+    str_alloc(&s->data);
+
+    if ( (m = Sfread(s->data.s_textA, 1, size, f->fd)) != size )
+    { deleteString(s, toInt(m), DEFAULT); 			/* TBD: error? */
+    }
+  } else
+  { tmp_string tmp;
+    int c;
+
+    str_tmp_init(&tmp);
+    while(tmp.s.size < size && (c = Sgetcode(f->fd)) != EOF )
+    { str_tmp_put(&tmp, (wint_t)c);
+    }
+    if ( !checkErrorFile(f) )
+    { str_tmp_done(&tmp);
       fail;
     }
+    s = StringToString(&tmp.s);
+    str_tmp_done(&tmp);
   }
 
   answer(s);
@@ -981,10 +1005,10 @@ getCharacterFile(FileObj f)
 { int chr;
 
   TRY( check_file(f, NAME_read) );
-  if ( feof(f->fd) )
+  if ( Sfeof(f->fd) )
     fail;
 
-  chr = getc(f->fd);
+  chr = Sgetcode(f->fd);
   
   answer(toInt(chr));
 }
@@ -1006,7 +1030,7 @@ checkErrorFile(FileObj f)
 { if ( f->fd == NULL )
     succeed;
 
-  if ( ferror(f->fd) )
+  if ( Sferror(f->fd) )
     return reportErrorFile(f);
 
   succeed;
@@ -1015,18 +1039,21 @@ checkErrorFile(FileObj f)
 
 status
 storeCharFile(FileObj f, int c)
-{ putc(c, f->fd);
+{ if ( f->encoding == NAME_octet )
+    Sputc(c, f->fd);
+  else
+    Sputcode(c, f->fd);
 
   return checkErrorFile(f);
 }
 
 
 void
-putstdw(unsigned long w, FILE *fd)
+putstdw(unsigned long w, IOSTREAM *fd)
 {
 #ifndef WORDS_BIGENDIAN
   union
-  { unsigned long         l;
+  { unsigned long l;
     unsigned char c[4];
   } cvrt;
   unsigned long rval;
@@ -1036,9 +1063,9 @@ putstdw(unsigned long w, FILE *fd)
          (cvrt.c[1] << 16) |
 	 (cvrt.c[2] << 8) |
 	  cvrt.c[3];
-  putw(rval, fd);
+  Sputw(rval, fd);
 #else /*WORDS_BIGENDIAN*/
-  putw(w, fd);
+  Sputw(w, fd);
 #endif /*WORDS_BIGENDIAN*/
 }
 
@@ -1050,13 +1077,73 @@ storeWordFile(FileObj f, Any w)
   return checkErrorFile(f);
 }
 
+#ifdef WORDS_BIGENDIAN
+static const int double_byte_order[] = { 7,6,5,4,3,2,1,0 };
+#else
+static const int double_byte_order[] = { 0,1,2,3,4,5,6,7 };
+#endif
+
+#define BYTES_PER_DOUBLE (sizeof(double_byte_order)/sizeof(int))
 
 status
-storeCharpFile(FileObj f, char *s)
-{ int l = strlen(s);
+storeDoubleFile(FileObj file, double f)
+{ unsigned char *cl = (unsigned char *)&f;
+  unsigned int i;
 
-  TRY(storeWordFile(f, (Any) (long)l));
-  fwrite(s, sizeof(char), l, f->fd);
+  for(i=0; i<BYTES_PER_DOUBLE; i++)
+    Sputc(cl[double_byte_order[i]], file->fd);
+
+  return checkErrorFile(file);
+}
+
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+storeStringFile() stores a string to a   file. For compatibility reasons
+the format is somewhat strange. If the string  is 8-bit, it is stored as
+length followed by  the  character  data.   Otherwise  it  is  stores as
+NEGATIVE length followed by a sequence of UTF-8 character codes.
+
+Note that if the string is wide but need not be, it is saved as if it is
+an ISO Latin-1 string.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+status
+storeStringFile(FileObj f, String s)
+{ if ( isstrA(s) )
+  { TRY(storeWordFile(f, (Any) (long)s->size));
+    Sfwrite(s->s_textA, sizeof(char), s->size, f->fd);
+
+    DEBUG(NAME_save, Cprintf("Saved ISO string, %ld chars\n", s->size));
+  } else if ( !str_iswide(s) )
+  { const charW *w = s->s_textW;
+    const charW *e = &w[s->size];
+
+    TRY(storeWordFile(f, (Any) (long)s->size));
+    for( ; w<e; w++)
+    { if ( Sputc(*w, f->fd) < 0 )
+	return checkErrorFile(f);
+    }
+
+    DEBUG(NAME_save,
+	  Cprintf("Saved converted ISO string, %ld chars\n", s->size));
+  } else
+  { IOENC oenc;
+    const charW *w = s->s_textW;
+    const charW *e = &w[s->size];
+
+    TRY(storeWordFile(f, (Any) -(long)s->size));
+    oenc = f->fd->encoding;
+    f->fd->encoding = ENC_UTF8;
+    for( ; w<e; w++)
+    { if ( Sputcode(*w, f->fd) < 0 )
+      { f->fd->encoding = oenc;
+	return checkErrorFile(f);
+      }
+    }
+    f->fd->encoding = oenc;
+
+    DEBUG(NAME_save, Cprintf("Saved wide string, %ld chars\n", s->size));
+  }
   
   return checkErrorFile(f);
 }
@@ -1064,7 +1151,7 @@ storeCharpFile(FileObj f, char *s)
 
 status
 storeNameFile(FileObj f, Name n)
-{ return storeCharpFile(f, strName(n));
+{ return storeStringFile(f, &n->data);
 }
 
 
@@ -1073,29 +1160,44 @@ storeIntFile(FileObj f, Int i)
 { return storeWordFile(f, (Any) valInt(i));
 }
 
+
 		/********************************
 		*             PATHS		*
 		********************************/
 
+static int
+waccess(const wchar_t *name, int m)
+{ string s;
+  const char *ufn;
+
+  str_set_n_wchar(&s, wcslen(name), (wchar_t *)name);
+  ufn = stringToFN(&s);
+  
+  DEBUG(NAME_find, Cprintf("find: trying \"%s\"\n", ufn));
+
+  return access(ufn, m);
+}
+
+
 #ifndef X_OK
 #define X_OK 0
 #endif
 
 status
 findFile(FileObj f, CharArray path, Name mode)
-{ char bin[MAXPATHLEN];
-  char *exp = expandFileName(strName(f->name), bin);
-  char base[MAXPATHLEN];
-  char name[MAXPATHLEN];
-  char *pathstr;
+{ wchar_t *base;
+  wchar_t basebuf[MAXPATHLEN];
+  const wchar_t *pathstr;
+  size_t bl;
   int m;
 
-  if ( !exp )
-    fail;
-
-  if ( isAbsolutePath(exp) || streq(exp, ".") )
+  if ( isAbsolutePath(nameToUTF8(f->name)) )
     succeed;
 
+  base = charArrayToWC((CharArray)f->name, &bl);
+  if ( base[0] == '.' )
+    succeed;
+  
   if ( isDefault(mode) || mode == NAME_read )
     m = R_OK;
   else if ( mode == NAME_write || mode == NAME_append )
@@ -1103,57 +1205,57 @@ findFile(FileObj f, CharArray path, Name mode)
   else /*if ( mode == NAME_execute )*/
     m = X_OK;
 
-  if ( notDefault(f->path) && access(strName(f->path), m) == 0 )
+  if ( notDefault(f->path) && access(nameToFN(f->path), m) == 0 )
     succeed;
 
-  strcpy(base, exp);
-  if ( is_absolute_name(base) )
-  { if ( access(base, m) == 0 )
-    { assign(f, path, CtoName(base));
-      succeed;
-    }
-
-    goto nofind;
-  }
+  if ( bl+1 > MAXPATHLEN )
+    return errorPce(f, NAME_representation, NAME_nameTooLong);
+  wcscpy(basebuf, base);
+  base = basebuf;
 
   if ( isDefault(path) )
-    pathstr = ".";
+    pathstr = L".";
   else
-    pathstr = strName(path);
+    pathstr = charArrayToWC(path, NULL);
 
   while( pathstr && *pathstr )
-  { char *end = pathstr;
+  { wchar_t name[MAXPATHLEN];
+    wchar_t bin[MAXPATHLEN];
+    const wchar_t *end = pathstr;
+    size_t l;
 
-#ifdef __WIN32__
-    if ( isletter(end[0]) && end[1] == ':' )
+#ifdef __WINDOWS__
+    if ( end[0] < 0x80 && isalpha(end[0]) && end[1] == ':' )
       end += 2;
 #endif
 
-    if ( (end = strchr(end, ':')) == NULL )
-    { strcpy(name, pathstr);
+    if ( (end = wcschr(end, ':')) == NULL )
+    { wcscpy(name, pathstr);
       pathstr = NULL;
     } else
-    { strncpy(name, pathstr, end-pathstr);
+    { wcsncpy(name, pathstr, end-pathstr);
       name[end-pathstr] = EOS;
       pathstr = &end[1];
     }
-    if ( (exp = expandFileName(name, bin)) )
-      strcpy(name, exp);
-    else
-      continue;
 
-    strcat(name, "/");
-    strcat(name, base);
+    if ( wcschr(name, L'$') || name[0] == L'~' )
+    { if ( (l=expandFileNameW(name, bin, MAXPATHLEN)) > 0 )
+	wcsncpy(name, bin, l);
+      else
+	continue;
+    } else
+    { l = wcslen(name);
+    }
 
-    DEBUG(NAME_find, Cprintf("%s->find: trying %s\n", pp(f), name));
+    name[l] = '/';
+    wcscpy(&name[l+1], base);
 
-    if ( access(name, m) == 0 )
-    { assign(f, path, CtoName(name));
+    if ( waccess(name, m) == 0 )
+    { assign(f, path, WCToName(name, wcslen(name)));
       succeed;
     }
   }
 
-nofind:
   return errorPce(f, NAME_cannotFindFile, path);
 }
 
@@ -1174,7 +1276,9 @@ static char *T_open[] =
 static char *T_find[] =
         { "path=[char_array]", "access=[{read,write,append,execute}]" };
 static char *T_initialise[] =
-        { "path=[name]", "kind=[{text,binary}]" };
+        { "path=[name]",
+	  "encoding=[{text,binary,iso_latin_1,utf8,unicode_be,unicode_le}]"
+	};
 
 /* Instance Variables */
 
@@ -1189,6 +1293,10 @@ static vardecl var_file[] =
      NAME_open, "(How) opened or closed?"),
   IV(NAME_filter, "command=name*", IV_BOTH,
      NAME_filter, "Name of input/output filter used"),
+  IV(NAME_bom, "[bool]", IV_BOTH,
+     NAME_encoding, "Byte Order Mark"),
+  IV(NAME_newlineMode, "{posix,dos,detect}", IV_BOTH,
+     NAME_encoding, "Newline representation"),
   IV(NAME_fd, "alien:FILE *", IV_NONE,
      NAME_internal, "Unix file (stream) handle")
 };
@@ -1200,6 +1308,8 @@ static senddecl send_file[] =
      DEFAULT, "Create from name and kind"),
   SM(NAME_unlink, 0, NULL, unlinkFile,
      DEFAULT, "Close file"),
+  SM(NAME_encoding, 1, "{text,binary,iso_latin_1,utf8,unicode_be,unicode_le}",
+     kindFile, NAME_fileType, "Specify text or binary encoding type"),
   SM(NAME_backup, 1, "extension=[name]", backupFile,
      NAME_copy, "Make a backup by adding extension (~)"),
   SM(NAME_copy, 1, "from=file", copyFile,
@@ -1287,17 +1397,14 @@ makeClassFile(Class class)
 { declareClass(class, &file_decls);
   setLoadStoreFunctionClass(class, loadFile, storeFile);
 
-#if defined(__WIN32__)
-{ int w32s = iswin32s();
-
-  featureClass(class, NAME_caseSensitive, OFF);
-  featureClass(class, NAME_casePreserving, w32s ? OFF : ON);
-  featureClass(class, NAME_8plus3names, w32s ? ON : OFF);
-}
-#else
-  featureClass(class, NAME_caseSensitive, ON);
+#if defined(__WINDOWS__)
+  featureClass(class, NAME_caseSensitive,  OFF);
   featureClass(class, NAME_casePreserving, ON);
-  featureClass(class, NAME_8plus3names, OFF);
+  featureClass(class, NAME_8plus3names,    OFF);
+#else
+  featureClass(class, NAME_caseSensitive,  ON);
+  featureClass(class, NAME_casePreserving, ON);
+  featureClass(class, NAME_8plus3names,    OFF);
 #endif
 
   FileFilters = globalObject(NAME_compressionFilters, ClassSheet,
