@@ -1,4 +1,4 @@
-/*  $Id: socket.c,v 1.40 2005/11/21 19:38:59 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -482,7 +482,11 @@ bindSocket(Socket s, Bool reuse)
 	   (instanceOfObject(s->address, ClassTuple) &&
 	    ((Tuple)s->address)->second == ZERO) )
       { struct sockaddr_in addr;
+#ifdef __WINDOWS__
+	DWORD len = sizeof(addr);
+#else
 	socklen_t len = sizeof(addr);
+#endif
 
 	if ( getsockname(SocketHandle(s), (struct sockaddr *) &addr, &len) )
 	{ return errorPce(s, NAME_socket, NAME_getsockname, SockError());
@@ -490,7 +494,7 @@ bindSocket(Socket s, Bool reuse)
 	{ if ( s->address == ZERO )
 	    assign(s, address, toInt(ntohs(addr.sin_port)));
 	  else
-	    assign((Tuple)s->address, second, toInt(ntohs(address.sin_port)));
+	    assign((Tuple)s->address, second, toInt(ntohs(addr.sin_port)));
 	}
       }
     }
@@ -521,7 +525,11 @@ acceptSocket(Socket s)
   } else /*if ( s->domain == NAME_inet )*/
 #endif
   { struct sockaddr_in address;
+#ifdef __WINDOWS__
+    DWORD len = sizeof(address);
+#else
     socklen_t len = sizeof(address);
+#endif
 
     if ( (id2=accept(SocketHandle(s), (struct sockaddr *) &address, &len))<0 )
       errorPce(s, NAME_socket, NAME_accept, SockError());
@@ -670,7 +678,11 @@ getPeerNameSocket(Socket s)
   } else /* if ( s->domain = NAME_inet ) */
 #endif /*UNIX_DOMAIN_SOCKETS*/
   { struct sockaddr_in address;
+#ifdef __WINDOWS__
+    DWORD len = sizeof(address);
+#else
     socklen_t len = sizeof(address);
+#endif
     int port;
     unsigned long addr;
     char aname[3*4+4];
