@@ -1,4 +1,4 @@
-/*  $Id: include.h,v 1.45 2006/04/13 12:22:03 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -73,6 +73,8 @@ error XPCE cannot be build for X version 10.  Sorry.
 #endif
 
 #ifdef USE_XFT
+#include <ft2build.h>
+#include FT_FREETYPE_H
 #include <X11/Xft/Xft.h>
 #endif
 
