@@ -1,4 +1,4 @@
-/*  $Id: xdraw.c,v 1.58 2003/07/07 09:42:39 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -87,6 +87,10 @@ s_string(s, f, x, y, w, h, had, vad)
 
 static void	clip_area(int *, int *, int *, int *);
 static void	r_andpattern(Image i);
+#ifdef USE_XFT
+static XftDraw *xftDraw();
+#endif
+
 
 		/********************************
 		*       DEVICE FUNCTIONS	*
@@ -109,7 +113,13 @@ static struct d_context
 { DContext	parent;			/* saved parent context */
   DrawContext	gcs;			/* The X GC's */                  
   Display      *display;		/* Current drawing display */     
+  int		screen;			/* X screen */
+  Visual       *visual;			/* X visual */
+  Colormap      colormap;		/* X colourmap */
   Drawable	drawable;		/* X Object we are drawing on */  
+#ifdef USE_XFT
+  XftDraw      *xft_draw;		/* XFT drawable */
+#endif
   Name		kind;			/* Drawable kind */               
   int		depth;			/* depth of drawable */           
   DisplayObj	pceDisplay;		/* PCE display object */          
@@ -136,6 +146,7 @@ static struct d_context
 #define X(x) ((x) + context.offset_x)
 #define Y(y) ((y) + context.offset_y)
 #define Translate(x, y)	 { (x) = X(x); (y) = Y(y); }
+#define InvTranslate(x, y) { x -= context.offset_x; y -= context.offset_y; }
 #define Clip(x, y, w, h) clip_area(&x, &y, &w, &h)
 
 #include <gra/graphstate.c>
@@ -189,7 +200,17 @@ d_push_context(void)
 
 static void
 d_pop_context()
-{ if ( context.parent != NULL )
+{
+#ifdef USE_XFT
+  if ( context.xft_draw )
+  { if ( !context.parent || context.parent->xft_draw != context.xft_draw )
+    { XftDrawDestroy(context.xft_draw);
+      context.xft_draw = NULL;
+    }
+  }
+#endif
+
+  if ( context.parent != NULL )
   { DContext ctx = context.parent;
 
     context = *ctx;			/* structure copy! */
@@ -273,6 +294,9 @@ d_display(DisplayObj d)
 
     context.pceDisplay = d;
     context.display    = r->display_xref;
+    context.screen     = r->screen;
+    context.visual     = r->visual;
+    context.colormap   = r->colour_map;
     context.depth      = r->depth;
     context.gcs	       = r->pixmap_context;
   }
@@ -317,6 +341,9 @@ d_window(PceWindow sw, int x, int y, int w, int h, int clear, int limit)
   context.origin_x	     = context.offset_x;
   context.origin_y	     = context.offset_y;
   context.drawable	     = (Drawable) XtWindow(widgetWindow(sw));
+#ifdef USE_XFT
+  context.xft_draw	     = NULL;
+#endif
   context.kind		     = NAME_window;
 
   env++;
@@ -357,6 +384,9 @@ d_window(PceWindow sw, int x, int y, int w, int h, int clear, int limit)
       context.cache = d->cache;
       context.window = context.drawable;
       context.drawable = (Drawable) getXrefObject(context.cache, d);
+#ifdef USE_XFT
+      context.xft_draw = NULL;
+#endif
       assign(context.cache, foreground, sw->colour);
       assign(context.cache, background, sw->background);
       context.kind = NAME_pixmap;
@@ -412,7 +442,9 @@ d_image(Image i, int x, int y, int w, int h)
 
   if ( i->kind == NAME_bitmap )
   { DisplayWsXref r = d->ws_ref;
-    context.gcs = r->bitmap_context;
+
+    context.gcs   = r->bitmap_context;
+    context.depth = 1;
   }
 
   if ( isDefault(i->foreground) )
@@ -424,6 +456,9 @@ d_image(Image i, int x, int y, int w, int h)
   else
     context.default_background = i->background;
   context.drawable	     = (Drawable) image;
+#ifdef USE_XFT
+  context.xft_draw	     = NULL;
+#endif
   context.kind		     = i->kind;
 
   if ( i->kind == NAME_pixmap )
@@ -457,6 +492,9 @@ d_xwindow(DisplayObj d, Window win, int x, int y, int w, int h)
   context.origin_x	     = 0;
   context.origin_y	     = 0;
   context.drawable	     = (Drawable) win;
+#ifdef USE_XFT
+  context.xft_draw	     = NULL;
+#endif
   context.default_background = d->background;
   context.kind		     = NAME_window;
 
@@ -502,6 +540,59 @@ d_frame(FrameObj fr, int x, int y, int w, int h)
 }
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Actually set the clipping region. If we are  using Xft, we need to use a
+Region (as far  as  I  understand   a  monochrome  pixmap).  This  seems
+expensive. I have the impression that eventually   the clip-mask of a GC
+is a monochrome pixmap anyway.  A   simple  test  reveals no significant
+performance difference between the two  approaches,   so  we'll  use the
+Region based approach regardless of Xft   for easier maintenance. Can we
+get better efficiency by keeping the region around?
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+static void
+do_clip(int x, int y, int w, int h)
+{
+#if 1					/* was USE_XFT */
+  XPoint pts[5];
+  Region r;
+
+  pts[0].x = x;   pts[0].y = y;
+  pts[1].x = x;   pts[1].y = y+h;
+  pts[2].x = x+w; pts[2].y = y+h;
+  pts[3].x = x+w; pts[3].y = y;
+  pts[4].x = x;   pts[4].y = y;
+
+  r = XPolygonRegion(pts, 5, WindingRule);
+  XSetRegion(context.display, context.gcs->workGC, r);
+  XSetRegion(context.display, context.gcs->fillGC, r);
+  XSetRegion(context.display, context.gcs->shadowGC, r);
+  XSetRegion(context.display, context.gcs->reliefGC, r);
+#ifdef USE_XFT
+  XftDrawSetClip(xftDraw(), r);
+#endif
+  XDestroyRegion(r);
+
+#else /*USE_XFT*/
+  XRectangle rect;
+
+# define CLIP(x) XSetClipRectangles(context.display, x, 0, 0, &rect, \
+				    1, Unsorted)
+  rect.x      = x;
+  rect.y      = y;
+  rect.width  = w;
+  rect.height = h;
+
+  CLIP(context.gcs->workGC);
+  CLIP(context.gcs->fillGC);
+  CLIP(context.gcs->shadowGC);
+  CLIP(context.gcs->reliefGC);
+
+#undef CLIP
+#endif /*USE_XFT*/
+}
+
+
 void
 d_clip(int x, int y, int w, int h)
 { XRectangle rect;
@@ -526,12 +617,7 @@ d_clip(int x, int y, int w, int h)
 
   DEBUG(NAME_redraw, Cprintf("clip to %d %d %d %d\n", x, y, w, h));
 
-# define CLIP(x) XSetClipRectangles(context.display, x, 0, 0, &rect, \
-				    1, Unsorted)
-  CLIP(context.gcs->workGC);
-  CLIP(context.gcs->fillGC);
-  CLIP(context.gcs->shadowGC);
-  CLIP(context.gcs->reliefGC);
+  do_clip(x, y, w, h);
 }
 
 
@@ -572,21 +658,8 @@ d_clip_done(void)
 
   assert(env >= environments);		/* stack underflow */
 
-  if ( env->level != 0 )		/* outermost: no use */
-  { XRectangle rect;
-
-    rect.x      = env->area.x;
-    rect.y      = env->area.y;
-    rect.width  = env->area.w;
-    rect.height = env->area.h;
-
-    CLIP(context.gcs->workGC);
-    CLIP(context.gcs->fillGC);
-    CLIP(context.gcs->shadowGC);
-    CLIP(context.gcs->reliefGC);
-  } 
-
-#   undef CLIP
+  if ( env >= environments && env->level != 0 ) /* outermost: no use */
+    do_clip(env->area.x, env->area.y, env->area.w, env->area.h);
 }
 
 
@@ -2421,6 +2494,330 @@ r_get_pixel(int x, int y)
 		*       TEXT MANIPULATION	*
 		********************************/
 
+#ifdef USE_XFT
+		 /*******************************
+		 *	    XFT VERSION		*
+		 *******************************/
+
+static XftDraw *
+xftDraw()
+{ if ( !context.xft_draw )
+  { if ( context.depth == 1 )
+    { context.xft_draw = XftDrawCreateBitmap(context.display,
+					     (Pixmap)context.drawable);
+    } else
+    { context.xft_draw = XftDrawCreate(context.display,
+				       context.drawable,
+				       context.visual,
+				       context.colormap);
+    }
+  }
+
+  return context.xft_draw;
+}
+
+static void
+s_font(FontObj f)
+{ if ( f )
+  { d_ensure_display();
+
+    if ( context.gcs->font != f )
+    { XpceFontInfo info;
+      context.gcs->font = f;
+    
+      info = (XpceFontInfo) getXrefObject(f, context.pceDisplay);
+      context.gcs->xft_font = info->xft_font;
+    }
+  }
+}
+
+
+int
+s_has_char(FontObj f, unsigned int c)
+{ s_font(f);
+
+  return XftCharExists(context.display, context.gcs->xft_font, c);
+}
+
+
+void
+f_domain(FontObj f, Name which, int *x, int *y)
+{ *x = 0;
+  *y = 0xffff;
+}
+
+
+int
+s_default_char(FontObj font)
+{ return 'X';				/* for now */
+}
+
+
+int
+s_ascent(FontObj font)
+{ s_font(font);
+
+  return context.gcs->xft_font->ascent;
+}
+
+
+int
+s_descent(FontObj font)
+{ s_font(font);
+
+  return context.gcs->xft_font->descent;
+}
+
+
+int
+c_width(wint_t c, FontObj font)
+{ FcChar32 s[1];
+  XGlyphInfo info;
+
+  s_font(font);
+  s[0] = c;
+
+  XftTextExtents32(context.display, context.gcs->xft_font, s, 1, &info);
+
+  return info.xOff;
+}
+
+
+static int
+s_advance(String s, int from, int to)
+{ int len = to-from;
+  XGlyphInfo info;
+
+  if ( len <= 0 )
+    return 0;
+
+  if ( isstrA(s) )
+  { XftTextExtents8(context.display, context.gcs->xft_font, 
+		    s->s_textA+from, len, &info);
+  } else if ( sizeof(charW) == 2 )
+  { XftTextExtents16(context.display, context.gcs->xft_font, 
+		     (FcChar16*)s->s_textW+from, len, &info);
+  } else if ( sizeof(charW) == 4 )
+  { XftTextExtents32(context.display, context.gcs->xft_font, 
+		     (FcChar32*)s->s_textW+from, len, &info);
+  }
+
+  return info.xOff;			/* Xoff? */
+}
+
+
+static int
+lbearing(wint_t c)
+{ XGlyphInfo info;
+  FcChar32 s[1];
+
+  s[0] = c;
+  XftTextExtents32(context.display, context.gcs->xft_font, 
+		   s, 1, &info);
+
+  return info.x;
+}
+
+
+static void
+xft_color(XftColor *c)
+{ if ( instanceOfObject(context.gcs->colour, ClassColour) )
+  { Colour cobj = context.gcs->colour;
+
+    c->pixel = getPixelColour(cobj, context.pceDisplay);
+    c->color.red   = valInt(cobj->red);
+    c->color.green = valInt(cobj->green);
+    c->color.blue  = valInt(cobj->blue);
+  } else				/* Pixmap colour.  Use black */
+  { c->color.red   = 0;
+    c->color.green = 0;
+    c->color.blue  = 0;
+  }
+  c->color.alpha = 0xffff;
+}
+
+
+
+void
+s_printA(charA *s, int l, int x, int y, FontObj f)
+{ if ( l > 0 )
+  { XftColor color;
+
+    xft_color(&color);
+    Translate(x, y);
+    s_font(f);
+    XftDrawString8(xftDraw(), &color, context.gcs->xft_font, x, y, s, l);
+  }
+}
+
+
+void
+s_printW(charW *s, int l, int x, int y, FontObj f)
+{ if ( l > 0 )
+  { XftColor color;
+
+    xft_color(&color);
+    Translate(x, y);
+    s_font(f);
+    
+    if ( sizeof(charW) == 2 )
+    { XftDrawString16(xftDraw(), &color, context.gcs->xft_font, x, y, 
+		      (FcChar16*)s, l);
+    } else if ( sizeof(charW) == 4 )
+    { XftDrawString32(xftDraw(), &color, context.gcs->xft_font, x, y, 
+		      (FcChar32*)s, l);
+    } else
+    { assert(0);
+    }
+  }
+}
+
+#else /*USE_XFT*/
+
+		 /*******************************
+		 *    Xwc* Function version	*
+		 *******************************/
+
+#ifdef USE_XFONTSET
+
+static void
+s_font(FontObj f)
+{ if ( f )
+  { d_ensure_display();
+
+    if ( context.gcs->font != f )
+    { XpceFontInfo info;
+      context.gcs->font = f;
+    
+      info = (XpceFontInfo) getXrefObject(f, context.pceDisplay);
+      context.gcs->font_set = info->font_set;
+    }
+  }
+}
+
+
+int
+s_has_char(FontObj f, unsigned int c)
+{ return TRUE;
+}
+
+
+void
+f_domain(FontObj f, Name which, int *x, int *y)
+{ *x = 0;
+  *y = 0xffff;
+}
+
+
+int
+s_default_char(FontObj font)
+{ return 'X';				/* for now */
+}
+
+
+int
+s_ascent(FontObj font)
+{ XFontSetExtents *exts;
+
+  s_font(font);
+  exts = XExtentsOfFontSet(context.gcs->font_set);
+  
+  return -exts->max_logical_extent.y;
+}
+
+
+int
+s_descent(FontObj font)
+{ XFontSetExtents *exts;
+
+  s_font(font);
+  exts = XExtentsOfFontSet(context.gcs->font_set);
+  
+  return exts->max_logical_extent.y + exts->max_logical_extent.height;
+}
+
+
+int
+c_width(wint_t c, FontObj font)
+{ wchar_t s[1];
+
+  s_font(font);
+  s[0] = c;
+
+  return XwcTextEscapement(context.gcs->font_set, s, 1);
+}
+
+
+static int
+s_advance(String s, int from, int to)
+{ int len = to-from;
+
+  if ( len <= 0 )
+    return 0;
+
+  if ( isstrA(s) )
+  { wchar_t *ws = alloca(len*sizeof(wchar_t));
+    wchar_t *t = ws;
+    const charA *f = &s->s_textA[from];
+    const charA *e = &f[len];
+    
+    while(f<e)
+      *t++ = *f++;
+
+    return XwcTextEscapement(context.gcs->font_set, ws, len);
+  } else
+  { return XwcTextEscapement(context.gcs->font_set, s->s_textW+from, len);
+  }
+}
+
+
+static int
+lbearing(wint_t c)
+{ XRectangle ink;
+  wchar_t s[1];
+
+  s[0] = c;
+  XwcTextExtents(context.gcs->font_set, s, 1, &ink, NULL);
+
+  return -ink.x;
+}
+
+
+void
+s_printA(charA *s, int l, int x, int y, FontObj f)
+{ if ( l > 0 )
+  { wchar_t *ws = alloca(l*sizeof(wchar_t));
+    wchar_t *t = ws;
+    const charA *e = &s[l];
+
+    while(s<e)
+      *t++ = *s++;
+
+    Translate(x, y);
+    s_font(f);
+    XwcDrawString(context.display, context.drawable,
+		  context.gcs->font_set,
+		  context.gcs->workGC,
+		  x, y, ws, l);
+  }
+}
+
+
+void
+s_printW(charW *s, int l, int x, int y, FontObj f)
+{ if ( l > 0 )
+  { Translate(x, y);
+    s_font(f);
+    XwcDrawString(context.display, context.drawable,
+		  context.gcs->font_set,
+		  context.gcs->workGC,
+		  x, y, s, l);
+  }
+}
+
+
+#else /*USE_XFONTSET*/
+
 #define MAX_CHAR 256
 static void s_font(FontObj f);
 
@@ -2492,18 +2889,21 @@ s_default_char(FontObj font)
 
 static void
 s_font(FontObj f)
-{ d_ensure_display();
+{ if ( f )
+  { d_ensure_display();
 
-  if ( context.gcs->font != f )
-  { XpceFontInfo info;
-    context.gcs->font = f;
-
-    info = (XpceFontInfo) getXrefObject(f, context.pceDisplay);
-    context.gcs->font_info = info->info;
-    context.gcs->char_widths = info->widths;
-
-    XSetFont(context.display, context.gcs->workGC,
-	     context.gcs->font_info->fid);
+    if ( context.gcs->font != f )
+    { XpceFontInfo info;
+      context.gcs->font = f;
+  
+      info = (XpceFontInfo) getXrefObject(f, context.pceDisplay);
+      context.gcs->font_info   = info->info;
+      context.gcs->char_widths = info->widths;
+      context.gcs->maxchar     = info->maxchar;
+  
+      XSetFont(context.display, context.gcs->workGC,
+	       context.gcs->font_info->fid);
+    }
   }
 }
 
@@ -2524,34 +2924,17 @@ s_descent(FontObj f)
 }
 
 
-int
-s_height(FontObj f)
-{ s_font(f);
-
-  return context.gcs->font_info->ascent + context.gcs->font_info->descent;
-}
-
-
 static int
-lbearing(unsigned int c, XFontStruct *info)
-{ XCharStruct *def = s_char_struct(info, c);
+lbearing(wint_t c)
+{ XFontStruct *info = context.gcs->font_info;
+  XCharStruct *def = s_char_struct(info, c);
 
   return def ? -def->lbearing : 0;
 }
 
 
-#if 0
-static int
-rbearing(unsigned int c, XFontStruct *info)
-{ XCharStruct *def = s_char_struct(info, c);
-
-  return def ? def->rbearing : info->max_bounds.width;
-}
-#endif
-
-
 int
-c_width(unsigned int c, FontObj font)
+c_width(wint_t c, FontObj font)
 { s_font(font);
 
   if ( context.gcs->char_widths )
@@ -2561,131 +2944,35 @@ c_width(unsigned int c, FontObj font)
 }
 
 
-String
-str_bits_as_font(String s, FontObj f, int *shift)
-{ static string s2;
-
-  Bool b16 = getB16Font(f);
-  if ( b16 == ON && isstr8(s) )
-  { s2 = *s;
-    s2.b16 = TRUE;
-    s2.size /= 2;
-    if ( shift )
-      *shift = -1;
-    return &s2;
-  } else if ( b16 != ON && !isstr8(s) )
-  { s2 = *s;
-    s2.b16 = FALSE;
-    s2.size *= 2;
-    if ( shift )
-      *shift = 1;
-    return &s2;
-  }
-  
-  if ( shift )
-    *shift = 0;
-  return s;
-}
-
-
 static int
-s_width_(String s, int from, int to)
+s_advance(String s, int from, int to)
 { if ( !context.gcs->char_widths )
-  { return context.gcs->font_info->max_bounds.width * (to-from);
-  } else
-  { cwidth *widths = context.gcs->char_widths;
-    int width;
-    int n = to-from;
-
-    if ( from >= to )
-      return 0;
-
-    if ( isstr8(s) )
-    { char8 *q = &s->s_text8[from];
-
-      width = lbearing(*q, context.gcs->font_info);
-#if 0
-      for(; n-- > 1; q++)
-	width += widths[*q];
-      width += rbearing(*q, context.gcs->font_info);
-#else
-      for(; n-- > 0; q++)
-	width += widths[*q];
-#endif
-    } else
-    { char16 *q = &s->s_text16[from];
-
-      width = lbearing(*q, context.gcs->font_info);
-#if 0
-      for(; n-- > 1; q++)
-	width += widths[*q];
-      width += rbearing(*q, context.gcs->font_info);
-#else
-      for(; n-- > 0; q++)
-	width += widths[*q];
-#endif
-    }
-
-    return width;
-  }
-}
-
-
-int
-str_width(String s, int from, int to, FontObj f)
-{ string s2;
-
-  s_font(f);
-
-  if ( f->b16 == ON && isstr8(s) )
-  { s2 = *s;
-    s2.b16 = TRUE;
-    s2.size /= 2;
-    from /= 2;
-    to /= 2;
-    s = &s2;
-  } else if ( f->b16 != ON && !isstr8(s) )
-  { s2 = *s;
-    s2.b16 = FALSE;
-    s2.size *= 2;
-    from *= 2;
-    to *= 2;
-    s = &s2;
-  }
-
-  if ( from < 0 )
-    from = 0;
-  if ( from >= s->size || to <= from )
-    return 0;
-  if ( to > s->size )
-    to = s->size;
-
-  return s_width_(s, from, to);
-}
-
-
-int
-str_advance(String s, int from, int to, FontObj f)
-{ if ( f ) 
-    s_font(f);
-
-  if ( !context.gcs->char_widths )
   { return context.gcs->font_info->max_bounds.width * (to-from);
   } else
   { cwidth *widths = context.gcs->char_widths;
     int width = 0;
     int n = to-from;
 
-    if ( isstr8(s) )
-    { char8 *q = &s->s_text8[from];
+    if ( from >= to )
+      return 0;
+
+    if ( isstrA(s) )
+    { charA *q = &s->s_textA[from];
 
       for(; n-- > 0; q++)
 	width += widths[*q];
     } else
-    { char16 *q = &s->s_text16[from];
+    { charW *q = &s->s_textW[from];
+      wint_t maxchr = context.gcs->maxchar;
 
       for(; n-- > 0; q++)
-	width += widths[*q];
+      { wint_t c = *q;
+
+	if ( c > maxchr )
+	  c = ' ';
+
+	width += widths[c];
+      }
     }
 
     return width;
@@ -2694,7 +2981,7 @@ str_advance(String s, int from, int to, FontObj f)
 
 
 void
-s_print8(char8 *s, int l, int x, int y, FontObj f)
+s_printA(charA *s, int l, int x, int y, FontObj f)
 { if ( l > 0 )
   { Translate(x, y);
     s_font(f);
@@ -2705,22 +2992,69 @@ s_print8(char8 *s, int l, int x, int y, FontObj f)
 
 
 void
-s_print16(char16 *s, int l, int x, int y, FontObj f)
+s_printW(charW *s, int l, int x, int y, FontObj f)
 { if ( l > 0 )
-  { Translate(x, y);
+  { XChar2b *ws = alloca(l*sizeof(XChar2b));
+    int i;
+
+    Translate(x, y);
     s_font(f);
+
+    for(i=0; i<l; i++)
+    { ws[i].byte1 = (s[i]>>8) & 0xff;
+      ws[i].byte2 = s[i] & 0xff;
+    }
+
     XDrawString16(context.display, context.drawable, context.gcs->workGC,
-		  x, y, (XChar2b *)s, l);
+		  x, y, ws, l);
   }
 }
 
+#endif /*USE_XFONTSET*/
+#endif /*USE_XFT*/
+
+int
+s_height(FontObj f)
+{ return s_ascent(f) + s_descent(f);
+}
+
+
+int
+str_width(String s, int from, int to, FontObj f)
+{ s_font(f);
+
+  if ( from < 0 )
+    from = 0;
+  if ( from >= s->size || to <= from )
+    return 0;
+  if ( to > s->size )
+    to = s->size;
+
+  if ( to > from )
+  { int w = lbearing(str_fetch(s, from));
+
+    w += s_advance(s, from, to);
+    return w;
+  }
+
+  return 0;
+}
+
+
+int
+str_advance(String s, int from, int to, FontObj font)
+{ s_font(font);
+
+  return s_advance(s, from, to);
+}
+
 
 void
 s_print(String s, int x, int y, FontObj f)
-{ if ( isstr8(s) )
-    s_print8(s->s_text8, s->size, x, y, f);
+{ if ( isstrA(s) )
+    s_printA(s->s_textA, s->size, x, y, f);
   else
-    s_print16(s->s_text16, s->size, x, y, f);
+    s_printW(s->s_textW, s->size, x, y, f);
 }
 
 
@@ -2729,28 +3063,49 @@ s_print_aligned(String s, int x, int y, FontObj f)
 { if ( s->size > 0 )
   { s_font(f);
     
-    x += lbearing(str_fetch(s, 0), context.gcs->font_info);
-    Translate(x, y);
-
-    XDrawString(context.display, context.drawable, context.gcs->workGC,
-		x, y, s->s_text8, s->size);
+    x += lbearing(str_fetch(s, 0));
+    s_print(s, x, y, f);
   } 
 }
 
 
+static void
+str_draw_text(String s, int offset, int len, int x, int y)
+{ if ( offset >= s->size )
+    return;
+
+  if ( offset < 0 )
+  { len += offset;
+    offset = 0;
+  }
+
+  if ( offset + len > s->size )
+    len = s->size - offset;
+  
+  if ( s->size > 0 )
+  { InvTranslate(x, y);			/* Hack */
+
+    if ( isstrA(s) )
+    { s_printA(s->s_textA+offset, len, x, y, NULL);
+    } else
+    { s_printW(s->s_textW+offset, len, x, y, NULL);
+    }
+  }
+}
+
+
 static void
 str_stext(String s, int f, int len, int x, int y, Style style)
 { if ( len > 0 )
-  { string s2;
-    Any ofg = NULL;
+  { Any ofg = NULL;
     int w = 0;				/* make compiler happy */
 
     if ( notNil(style) )
-    { w = str_advance(s, f, f+len, NULL);
+    { w = s_advance(s, f, f+len);
 
       if ( notDefault(style->background) )
-      { int a = context.gcs->font_info->ascent;
-	int b = context.gcs->font_info->descent;
+      { int a = s_ascent(NULL);
+	int b = s_descent(NULL);
 	
 	r_fillpattern(style->background, NAME_foreground);
 	XFillRectangle(context.display, context.drawable, context.gcs->fillGC,
@@ -2760,29 +3115,7 @@ str_stext(String s, int f, int len, int x, int y, Style style)
 	ofg = r_colour(style->colour);
     }
 
-    if ( context.gcs->font->b16 == ON )
-    { if ( isstr8(s) )
-      { s2 = *s;
-	s2.size /= 2;
-	s2.b16 = TRUE;
-	s = &s2;
-	len /= 2;
-	f /= 2;
-      }
-      XDrawString16(context.display, context.drawable, context.gcs->workGC,
-		    x, y, (XChar2b *)s->s_text16+f, len);
-    } else
-    { if ( isstr16(s) )
-      { s2 = *s;
-	s2.size *= 2;
-	s2.b16 = FALSE;
-	s = &s2;
-	len *= 2;
-	f *= 2;
-      }
-      XDrawString(context.display, context.drawable, context.gcs->workGC,
-		  x, y, s->s_text8+f, len);
-    }
+    str_draw_text(s, f, len, x, y);
 
     if (  notNil(style) && style->attributes & TXT_UNDERLINED )
       XDrawLine(context.display, context.drawable, context.gcs->workGC,
@@ -2797,29 +3130,9 @@ str_stext(String s, int f, int len, int x, int y, Style style)
 static void
 str_text(String s, int x, int y)
 { if ( s->size > 0 )
-  { string s2;
+  { x += lbearing(str_fetch(s, 0));
 
-    if ( context.gcs->font->b16 == ON )
-    { if ( isstr8(s) )
-      { s2 = *s;
-	s2.size /= 2;
-	s2.b16 = TRUE;
-	s = &s2;
-      }
-      x += lbearing(str_fetch(s, 0), context.gcs->font_info);
-      XDrawString16(context.display, context.drawable, context.gcs->workGC,
-		    x, y, (XChar2b *)s->s_text16, s->size);
-    } else
-    { if ( isstr16(s) )
-      { s2 = *s;
-	s2.size *= 2;
-	s2.b16 = FALSE;
-	s = &s2;
-      }
-      x += lbearing(str_fetch(s, 0), context.gcs->font_info);
-      XDrawString(context.display, context.drawable, context.gcs->workGC,
-		  x, y, s->s_text8, s->size);
-    }
+    str_draw_text(s, 0, s->size, x, y);
   }
 }
 
@@ -2840,9 +3153,10 @@ typedef struct
 
 
 static void
-str_break_into_lines(String s, strTextLine *line, int *nlines)
+str_break_into_lines(String s, strTextLine *line, int *nlines, int maxlines)
 { int here = 0;
   int size = s->size;
+  int nls = 0;
 
   *nlines = 0;
 
@@ -2854,7 +3168,7 @@ str_break_into_lines(String s, strTextLine *line, int *nlines)
     return;
   }
 
-  for( ; here < size; line++, (*nlines)++ )
+  for( ; here < size && nls < maxlines; line++, nls++ )
   { int el;
 
     str_cphdr(&line->text, s);
@@ -2864,7 +3178,7 @@ str_break_into_lines(String s, strTextLine *line, int *nlines)
     { line->text.size = el - here;
       here = el + 1;
       if ( here == size )		/* last char is newline: add a line */
-      { line++, (*nlines)++;
+      { line++, nls++;
 	str_cphdr(&line->text, s);
 	line->text.s_text = str_textp(s, here);
 	line->text.size = 0;
@@ -2874,6 +3188,8 @@ str_break_into_lines(String s, strTextLine *line, int *nlines)
       here = size;
     }
   }
+
+  *nlines = nls;
 }
 
 
@@ -2914,28 +3230,20 @@ str_size(String s, FontObj font, int *width, int *height)
   strTextLine *line;
   int nlines, n;
   int w = 0;
-  string s2;
 
   s_font(font);
 
-  if ( font->b16 == ON && isstr8(s) )
-  { s2 = *s;
-    s2.b16 = TRUE;
-    s2.size /= 2;
-    s = &s2;
-  } else if ( font->b16 != ON && !isstr8(s) )
-  { s2 = *s;
-    s2.b16 = FALSE;
-    s2.size *= 2;
-    s = &s2;
-  }
-
-  str_break_into_lines(s, lines, &nlines);
+  str_break_into_lines(s, lines, &nlines, MAX_TEXT_LINES);
   for(n = 0, line = lines; n++ < nlines; line++)
-  { int lw = s_width_(&line->text, 0, line->text.size);
+  { if ( line->text.size > 0 )
+    { int lw;
+
+      lw = lbearing(str_fetch(&line->text, 0));
+      lw += s_advance(&line->text, 0, line->text.size);
     
-    if ( w < lw )
-      w = lw;
+      if ( w < lw )
+	w = lw;
+    }
   }
   
   *width  = w;
@@ -2956,8 +3264,8 @@ str_string(String s, FontObj font, int x, int y, int w, int h,
 
   Translate(x, y);
   s_font(font);
-  baseline = context.gcs->font_info->ascent;
-  str_break_into_lines(s, lines, &nlines);
+  baseline = s_ascent(font);
+  str_break_into_lines(s, lines, &nlines, MAX_TEXT_LINES);
   str_compute_lines(lines, nlines, font, x, y, w, h, hadjust, vadjust);
 
   if ( flags & TXT_UNDERLINED )
@@ -2991,14 +3299,14 @@ str_selected_string(String s, FontObj font,
 
   Translate(x, y);
   s_font(font);
-  baseline = context.gcs->font_info->ascent;
-  str_break_into_lines(s, lines, &nlines);
+  baseline = s_ascent(font);
+  str_break_into_lines(s, lines, &nlines, MAX_TEXT_LINES);
   str_compute_lines(lines, nlines, font, x, y, w, h, hadjust, vadjust);
 
   for(n=0, line = lines; n++ < nlines; line++)
   { int len = line->text.size;
 
-    line->x += lbearing(str_fetch(&line->text, 0), context.gcs->font_info);
+    line->x += lbearing(str_fetch(&line->text, 0));
 
     if ( t <= here || f >= here+len )	/* outside */
       str_stext(&line->text, 0, len, line->x, line->y+baseline, NIL);
@@ -3007,13 +3315,13 @@ str_selected_string(String s, FontObj font,
 
       sf = (f <= here     ?      0 : f-here);
       sl = (t >= here+len ? len-sf : t-here-sf);
-      sx = str_advance(&line->text, 0, sf, NULL);
+      sx = s_advance(&line->text, 0, sf);
       
       str_stext(&line->text, 0,  sf, line->x,    line->y+baseline, NIL);
       str_stext(&line->text, sf, sl, line->x+sx, line->y+baseline, style);
       if ( sf+sl < len )
       { int a  = sf+sl;
-	int ax = sx + str_advance(&line->text, sf, a, NULL);
+	int ax = sx + s_advance(&line->text, sf, a);
 
 	str_stext(&line->text, a, len-a, line->x+ax, line->y+baseline, NIL);
       }
@@ -3041,8 +3349,8 @@ ps_string(String s, FontObj font, int x, int y, int w, Name format, int flags)
   s_font(font);
   ps_font(font);
 
-  baseline = context.gcs->font_info->ascent;
-  str_break_into_lines(s, lines, &nlines);
+  baseline = s_ascent(font);
+  str_break_into_lines(s, lines, &nlines, MAX_TEXT_LINES);
   str_compute_lines(lines, nlines, font, x, y, w, 0, format, NAME_top);
 
   for(n=0, line = lines; n++ < nlines; line++)
@@ -3070,7 +3378,7 @@ str_draw_text_lines(int acc, FontObj font,
 		    int ox, int oy)
 { strTextLine *line;
   int n;
-  int baseline = context.gcs->font_info->ascent;
+  int baseline = s_ascent(font);
 
   for(n=0, line = lines; n++ < nlines; line++)
   { str_text(&line->text, line->x+ox, line->y+baseline+oy);
@@ -3079,13 +3387,13 @@ str_draw_text_lines(int acc, FontObj font,
     { int cx = line->x;
       int cn;
 
-      cx += lbearing(str_fetch(&line->text, 0), context.gcs->font_info);
+      cx += lbearing(str_fetch(&line->text, 0));
 
       for(cn=0; cn<line->text.size; cn++)
       { int c  = str_fetch(&line->text, cn);
 	int cw = c_width(c, font);
 
-	if ( tolower(c) == (unsigned)acc )
+	if ( (int)tolower(c) == acc )
 	{			/* not r_line to avoid double Translate() */
 	  XDrawLine(context.display, context.drawable, context.gcs->workGC,
 		    cx, line->y+baseline+1, cx+cw-2, line->y+baseline+1);
@@ -3124,7 +3432,7 @@ str_label(String s, int acc, FontObj font, int x, int y, int w, int h,
 
   Translate(x, y);
   s_font(font);
-  str_break_into_lines(s, lines, &nlines);
+  str_break_into_lines(s, lines, &nlines, MAX_TEXT_LINES);
   str_compute_lines(lines, nlines, font, x, y, w, h, hadjust, vadjust);
   if ( acc )
   { r_dash(NAME_none);
