@@ -109,6 +109,9 @@ linkVal__LD(Word p ARG_LD)
     w = *p;
   }
 
+#ifdef LIFE_GC
+  SECURE(assert(w != ATOM_garbage_collected));
+#endif
   return w;
 }
 
@@ -3402,8 +3405,9 @@ PL_open_resource(Module m,
 int
 PL_raise(int sig)
 { GET_LD
+
   if ( sig > 0 && sig <= MAXSIGNAL && LD )
-  { LD->pending_signals |= (1L << (sig-1));
+  { LD->pending_signals |= ((int64_t)1 << (sig-1));
     return TRUE;
   }
 
@@ -3411,6 +3415,25 @@ PL_raise(int sig)
 }
 
 
+int
+PL_pending__LD(int sig ARG_LD)
+{ if ( sig > 0 && sig <= MAXSIGNAL && LD )
+    return (LD->pending_signals & ((int64_t)1 << (sig-1))) ? TRUE : FALSE;
+
+  return -1;
+}
+
+
+int
+PL_clearsig__LD(int sig ARG_LD)
+{ if ( sig > 0 && sig <= MAXSIGNAL && LD )
+  { LD->pending_signals &= ~((int64_t)1 << (sig-1));
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 		/********************************
 		*         RESET (ABORTS)	*
 		********************************/
