@@ -249,6 +249,7 @@ COMMON(int) 		openFileDescriptors(unsigned char *buf, int size);
 COMMON(void) 		protocol(const char *s, size_t n);
 COMMON(bool) 		getInputStream__LD(term_t t, IOSTREAM **s ARG_LD);
 COMMON(bool) 		getOutputStream(term_t t, IOSTREAM **s);
+COMMON(bool)	        reportStreamError(IOSTREAM *s);
 COMMON(bool) 		streamStatus(IOSTREAM *s);
 COMMON(atom_t) 		fileNameStream(IOSTREAM *s);
 COMMON(int) 		getSingleChar(IOSTREAM *s, int signals);
@@ -408,6 +409,8 @@ COMMON(void)            bindExtensions(const char *module,
 				       const PL_extension *ext);
 COMMON(void)		initForeign(void);
 COMMON(int)		PL_rethrow(void);
+COMMON(int)		PL_pending__LD(int sig ARG_LD);
+COMMON(int)		PL_clearsig__LD(int sig ARG_LD);
 
 
 /* pl-fmt.c */
@@ -934,3 +937,5 @@ COMMON(void)    promoteNumber(Number n1, numtype type);
 COMMON(int)	cmpNumbers(Number n1, Number n2);
 COMMON(void)	cpNumber(Number to, Number from);
 
+/* pl-version.h */
+COMMON(void)	setGITVersion(void);
