@@ -124,10 +124,10 @@ char tmp[256];				/* for calling print_val(), etc. */
 #define ldomark(p)	{ *(p) |= MARK_MASK; }
 #define domark(p)	{ if ( is_marked(p) ) \
 			    sysError("marked twice: %p (*= 0x%lx), gTop = %p", p, *(p), gTop); \
+			  DEBUG(3, char b[64]; Sdprintf("\tdomarked(%p = %s)\n", p, print_val(*p, b))); \
 			  *(p) |= MARK_MASK; \
 			  total_marked++; \
 			  recordMark(p); \
-			  DEBUG(4, Sdprintf("is_marked(%p)\n", p)); \
 			}
 #define unmark(p)	(*(p) &= ~MARK_MASK)
 
@@ -164,7 +164,9 @@ char tmp[256];				/* for calling print_val(), etc. */
 forwards void		mark_variable(Word ARG_LD);
 forwards void		sweep_foreign(void);
 static void		sweep_global_mark(Word *m ARG_LD);
+#ifndef LIFE_GC
 forwards QueryFrame	mark_environments(LocalFrame, Code PC);
+#endif
 forwards void		update_relocation_chain(Word, Word ARG_LD);
 forwards void		into_relocation_chain(Word, int stg ARG_LD);
 forwards void		alien_into_relocation_chain(void *addr,
@@ -254,18 +256,39 @@ print_adr(Word adr, char *buf)
 
 static char *
 print_val(word val, char *buf)
-{ char *tag_name[] = { "var", "float", "int", "atom",
-		       "string", "list", "term", "ref" };
-  char *stg_name[] = { "static/inline/trail", "global", "local", "reserved" };
+{ GET_LD
+  char *tag_name[] = { "var", "attvar", "float", "int", "atom",
+		       "string", "term", "ref" };
+  char *stg_name[] = { "static", "global", "local", "reserved" };
+  char *o = buf;
 
-  Ssprintf(buf, "%s at %s(%ld)",
-	   tag_name[tag(val)],
-	   stg_name[storage(val) >> 3],
-	   (val >> LMASK_BITS)/sizeof(word));
-  if ( val & MARK_MASK )
-    strcat(buf, "M");
-  if ( val & FIRST_MASK )
-    strcat(buf, "F");
+  if ( val & (MARK_MASK|FIRST_MASK) )
+  { *o++ = '[';
+    if ( val & MARK_MASK )
+      *o++ = 'M';
+    if ( val & FIRST_MASK )
+      *o++ = 'F';
+    *o++ = ']';
+    val &= ~(word)(MARK_MASK|FIRST_MASK);
+  }
+
+  if ( isVar(val) )
+    strcpy(o, "VAR");
+  else if ( isTaggedInt(val) )
+    Ssprintf(o, "int(%ld)", valInteger(val));
+  else if ( isAtom(val) )
+  { const char *s = stringAtom(val);
+    if ( strlen(s) > 10 )
+    { strncpy(o, s, 10);
+      strcat(o, "...");
+    } else
+    { strcpy(o, s);
+    }
+  } else
+    Ssprintf(o, "%s at %s(%ld)",
+	     tag_name[tag(val)],
+	     stg_name[storage(val) >> 3],
+	     (val >> LMASK_BITS)/sizeof(word));
 
   return buf;
 }
@@ -427,7 +450,9 @@ mark_variable(Word start ARG_LD)
   word val;				/* old value of current cell */
   Word next;				/* cell to be examined */
 
-  DEBUG(3, Sdprintf("marking %p\n", start));
+  DEBUG(3,
+	char b[64];
+	Sdprintf("marking %p (=%s)\n", start, print_val(*start, b)));
 
   if ( is_marked(start) )
     sysError("Attempt to mark twice");
@@ -568,6 +593,12 @@ static void
 mark_term_refs()
 { GET_LD
   FliFrame fr = fli_context;
+#if O_DEBUG
+  long gmarked = 0;
+  long lmarked = 0;
+#endif
+
+  DEBUG(3, Sdprintf("Marking term references ...\n"));
 
   for( ; fr; fr = fr->parent )
   { Word sp = refFliP(fr, 0);
@@ -578,14 +609,19 @@ mark_term_refs()
     { SECURE(assert(!is_marked(sp)));
 
       if ( isGlobalRef(*sp) )
-      { mark_variable(sp PASS_LD);
+      { DEBUG(3, gmarked++);
+	mark_variable(sp PASS_LD);
       } else
-      { ldomark(sp);      
+      { DEBUG(3, lmarked++);
+	ldomark(sp);      
       }
     }
 
     SECURE(check_marked("After marking foreign frame"));
   }
+
+  DEBUG(3, Sdprintf("Marked %ld global and %ld local term references\n",
+		    gmarked, lmarked));
 }
 
 
@@ -788,6 +824,22 @@ slotsInFrame(LocalFrame fr, Code PC)
 }
 
 
+static inline void
+check_call_residue(LocalFrame fr ARG_LD)
+{
+#ifdef O_CALL_RESIDUE
+  if ( fr->predicate == PROCEDURE_call_residue_vars2->definition )
+  { if ( !LD->gc.marked_attvars )
+    { mark_attvars();
+      LD->gc.marked_attvars = TRUE;
+    }
+  }
+#endif
+}
+
+
+#ifndef LIFE_GC
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Mark environments and all data that can   be  reached from them. Returns
 the QueryFrame that started this environment,  which provides use access
@@ -815,14 +867,7 @@ mark_environments(LocalFrame fr, Code PC)
 		      levelFrame(fr), predicateName(fr->predicate)));
 
     clearUninitialisedVarsFrame(fr, PC);
-#ifdef O_CALL_RESIDUE
-    if ( fr->predicate == PROCEDURE_call_residue_vars2->definition )
-    { if ( !LD->gc.marked_attvars )
-      { mark_attvars();
-	LD->gc.marked_attvars = TRUE;
-      }
-    }
-#endif    
+    check_call_residue(fr PASS_LD);
 
     slots  = slotsInFrame(fr, PC);
 #if O_SECURE
@@ -846,6 +891,7 @@ mark_environments(LocalFrame fr, Code PC)
   }
 }
 
+#endif /*not LIFE_GC*/
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 If multiple TrailAssignment() calls happen on  the same address within a
@@ -892,15 +938,6 @@ mergeTrailedAssignments(GCTrailEntry top, GCTrailEntry mark,
 
   DEBUG(2, Sdprintf("Scanning %d trailed assignments\n", assignments));
 
-#if O_SECURE
-  for(te=mark; te <= top; te++)
-  { if ( ttag(te[1].address) == TAG_TRAILVAL )
-    { Word p = val_ptr(te->address);
-      assert(!is_first(p));
-    }
-  }
-#endif
-
   for(te=mark; te <= top; te++)
   { if ( ttag(te[1].address) == TAG_TRAILVAL )
     { Word p = val_ptr(te->address);
@@ -936,12 +973,8 @@ normal trail-pointer, while the  second   is  flagged  with TAG_TRAILVAL
 is encountered, this value is restored  at   the  location  of the first
 trail-cell.
 
-If the trail cell has become garbage,   we  can destroy both cells. Note
-that mark_trail() already has marked the   replaced  value, so that will
-not be garbage collected during this pass. We cannot reverse marking the
-stacks and the trail-stack as one trailed   value might make another one
-non-garbage. It isn't too bad however as the   next GC will take care of
-the term.
+If the trail cell  has  become  garbage,   we  can  destroy  both cells,
+otherwise we must mark the value.
 
 Early reset of trailed  assignments  is   another  issue.  If  a trailed
 location has not yet been  marked  it   can  only  be accessed by frames
@@ -971,12 +1004,36 @@ early_reset_vars(mark *m, Word top, GCTrailEntry te ARG_LD)
 	te->address = 0;
 	trailcells_deleted += 2;
       } else if ( is_marked(tard) )
-      { assignments++;
+      { Word gp = val_ptr(te->address);
+
+	assert(onGlobal(gp));
+	assert(!is_first(gp));
+	if ( !is_marked(gp) )
+	{ total_marked++;			/* fix counters */
+	  local_marked--;
+
+	  DEBUG(2,
+		char b1[64]; char b2[64]; char b3[64];
+		Sdprintf("Marking assignment at %s (%s --> %s)\n",
+			 print_adr(tard, b1),
+			 print_val(*gp, b2),
+			 print_val(*tard, b3)));
+
+	  mark_variable(gp PASS_LD);
+	  assert(is_marked(gp));
+	}
+
+	assignments++;
       } else
-      { Word gp;
-	DEBUG(3, Sdprintf("Early reset of assignment at %p (*=0x%lx)\n",
-		 tard, *tard));
-	gp = val_ptr(te->address);
+      { Word gp = val_ptr(te->address);
+
+	DEBUG(4,
+	      char b1[64]; char b2[64]; char b3[64];
+	      Sdprintf("Early reset of assignment at %s (%s --> %s)\n",
+		       print_adr(tard, b1),
+		       print_val(*tard, b2),
+		       print_val(*gp, b3)));
+
 	assert(onGlobal(gp));
 	*tard = *gp;
 	unmark(tard);
@@ -994,11 +1051,20 @@ early_reset_vars(mark *m, Word top, GCTrailEntry te ARG_LD)
       { SECURE(assert(ttag(te[1].address) != TAG_TRAILVAL));
 	te->address = 0;
 	trailcells_deleted++;
-      } else if ( !is_marked(tard) )	/* garbage */
-      { setVar(*tard);
-	DEBUG(3, Sdprintf("Early reset at %p\n", tard));
+      } else if ( !is_marked(tard) )
+      { DEBUG(3,
+	      char b1[64]; char b2[64]; 
+	      Sdprintf("Early reset at %s (%s)\n", print_adr(tard, b1), print_val(*tard, b2)));
+	setVar(*tard);
 	te->address = 0;
 	trailcells_deleted++;
+      } else if ( isVar(get_value(tard)) )
+      { if ( tard == valTermRef(LD->attvar.tail) ||
+	     tard == valTermRef(LD->attvar.head) )
+	{ DEBUG(3, Sdprintf("Trailed wakeup list already reset %p\n", tard));
+	  te->address = 0;
+	  trailcells_deleted++;
+	}
       }
     }
   }
@@ -1029,6 +1095,7 @@ mark_foreign_frame(FliFrame fr, GCTrailEntry te)
 }
 
 
+#ifndef LIFE_GC
 static GCTrailEntry
 mark_choicepoints(Choice ch, GCTrailEntry te, FliFrame *flictx)
 { GET_LD
@@ -1057,12 +1124,12 @@ mark_choicepoints(Choice ch, GCTrailEntry te, FliFrame *flictx)
     alien_into_relocation_chain(&ch->mark.trailtop,
 				STG_TRAIL, STG_LOCAL PASS_LD);
     SECURE(trailtops_marked--);
-
     mark_environments(fr, ch->type == CHP_JUMP ? ch->value.PC : NULL);
   }
 
   return te;
 }
+#endif
 
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -1080,6 +1147,9 @@ We must first mark all environments,   including  those in outer queries
 variables in outer queries.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
+#ifdef LIFE_GC
+#include "pl-lifegc.c"
+#else
 static void
 mark_stacks(LocalFrame fr, Choice ch)
 { GET_LD
@@ -1113,35 +1183,7 @@ mark_stacks(LocalFrame fr, Choice ch)
 
   DEBUG(2, Sdprintf("Trail stack garbage: %ld cells\n", trailcells_deleted));
 }
-
-
-#ifdef O_DESTRUCTIVE_ASSIGNMENT
-static void
-mark_trail()
-{ GET_LD
-  GCTrailEntry te = (GCTrailEntry)tTop - 1;
-
-  for( ; te >= (GCTrailEntry)tBase; te-- )
-  { Word gp;
-
-    if ( ttag(te->address) == TAG_TRAILVAL )
-    { gp = val_ptr(te->address);
-
-      DEBUG(3, Sdprintf("mark_trail(): trailed value from %p at %p (*=%p)\n",
-			&te->address, gp, *gp));
-
-      assert(onGlobal(gp));
-      if ( !is_marked(gp) )
-      {	total_marked++;			/* fix counters */
-	local_marked--;
-
-	mark_variable(gp PASS_LD);
-	assert(is_marked(gp));
-      }
-    }
-  }
-}
-#endif /*O_DESTRUCTIVE_ASSIGNMENT*/
+#endif
 
 
 #if O_SECURE
@@ -1162,9 +1204,6 @@ mark_phase(LocalFrame fr, Choice ch)
 
   SECURE(check_marked("Before mark_term_refs()"));
   mark_term_refs();
-#ifdef O_DESTRUCTIVE_ASSIGNMENT
-  mark_trail();
-#endif
   mark_stacks(fr, ch);
 #if O_SECURE
   if ( !scan_global(TRUE) )
@@ -1215,7 +1254,7 @@ update_relocation_chain(Word current, Word dest ARG_LD)
 { Word head = current;
   word val = get_value(current);
 
-  DEBUG(3, Sdprintf("unwinding relocation chain at %p to %p\n",
+  DEBUG(4, Sdprintf("unwinding relocation chain at %p to %p\n",
 		    current, dest));
 
   do
@@ -1250,7 +1289,7 @@ into_relocation_chain(Word current, int stg ARG_LD)
   set_value(current, get_value(head));
   set_value(head, consPtr(current, stg|tag(val)));
 
-  DEBUG(2, Sdprintf("Into relocation chain: %p (head = %p)\n",
+  DEBUG(4, Sdprintf("Into relocation chain: %p (head = %p)\n",
 		    current, head));
 
   if ( is_first(head) )
@@ -1535,6 +1574,18 @@ sweep_environments(LocalFrame fr, Code PC)
 	  into_relocation_chain(sp, STG_LOCAL PASS_LD);
 	}
       }
+#ifdef LIFE_GC
+      else
+      { if ( isGlobalRef(*sp) )
+	{ DEBUG(1, char b[64];
+		Sdprintf("[%ld] %s: GC VAR(%d) (=%s)\n",
+			 levelFrame(fr), predicateName(fr->predicate),
+			 sp-argFrameP(fr, 0),
+			 print_val(*sp, b)));
+	  *sp = ATOM_garbage_collected;
+	}
+      }
+#endif
     }
 
     PC = fr->programPointer;
@@ -1691,7 +1742,7 @@ compact_global(void)
 		 current, *current, gTop, *v);
 #endif
       dest -= offset + 1;
-      DEBUG(3, Sdprintf("Marked cell at %p (size = %ld; dest = %p)\n",
+      DEBUG(4, Sdprintf("Marked cell at %p (size = %ld; dest = %p)\n",
 			current, offset+1, dest));
       if ( is_first(current) )
 	update_relocation_chain(current, dest PASS_LD);
@@ -1821,8 +1872,7 @@ void
 considerGarbageCollect(Stack s)
 { GET_LD
 
-  if ( trueFeature(GC_FEATURE) &&
-       !gc_status.requested )
+  if ( trueFeature(GC_FEATURE) && !PL_pending(SIG_GC) )
   { if ( s == NULL )
     { considerGarbageCollect((Stack)&LD->stacks.global);
       considerGarbageCollect((Stack)&LD->stacks.trail);
@@ -1840,13 +1890,13 @@ considerGarbageCollect(Stack s)
 	if ( used > s->factor*s->gced_size + s->small )
 	{ DEBUG(2, Sdprintf("GC: request on %s, factor=%d, last=%ld, small=%ld\n",
 			    s->name, s->factor, s->gced_size, s->small));
-	  gc_status.requested = TRUE;
+	  PL_raise(SIG_GC);
 	} else if ( free < limit/8 && used > s->gced_size + limit/32 ) 
 	{ DEBUG(2, Sdprintf("GC: request on low free\n"));
-	  gc_status.requested = TRUE;
+	  PL_raise(SIG_GC);
 	}
     
-	DEBUG(1, if ( gc_status.requested)
+	DEBUG(1, if ( PL_pending(SIG_GC) )
 		 { Sdprintf("%s overflow: Posted garbage collect request\n",
 			    s->name);
 		 });
@@ -1947,9 +1997,12 @@ check_environments(LocalFrame fr, Code PC, Word key)
 
     assert(onStack(local, fr));
 
-    DEBUG(3, Sdprintf("Check [%ld] %s:",
+    DEBUG(3, Sdprintf("Check [%ld] %s (PC=%d):",
 		      levelFrame(fr),
-		      predicateName(fr->predicate)));
+		      predicateName(fr->predicate),
+		      (false(fr->predicate, FOREIGN) && PC)
+		        ? (PC-fr->clause->clause->codes)
+		      	: 0));
 
     slots = slotsInFrame(fr, PC);
     sp = argFrameP(fr, 0);
@@ -2170,9 +2223,11 @@ garbageCollect(LocalFrame fr, Choice ch)
     ch = LD->choicepoints;
 
   enterGC();
+#ifndef UNBLOCKED_GC
   blockSignals(&mask);
+#endif
   blockGC(PASS_LD1);			/* avoid recursion due to */
-  gc_status.requested = FALSE;		/* printMessage() */
+  PL_clearsig(SIG_GC);
 
   gc_status.active = TRUE;
   if ( verbose )
@@ -2216,8 +2271,8 @@ garbageCollect(LocalFrame fr, Choice ch)
   mark_phase(fr, ch);
   tgar = trailcells_deleted * sizeof(struct trail_entry);
   ggar = (gTop - gBase - total_marked) * sizeof(word);
-  gc_status.trail_gained  += tgar;
   gc_status.global_gained += ggar;
+  gc_status.trail_gained  += tgar;
   gc_status.collections++;
 
   DEBUG(2, Sdprintf("Compacting trail ... "));
@@ -2238,12 +2293,18 @@ garbageCollect(LocalFrame fr, Choice ch)
   trimStacks(PASS_LD1);
   LD->stacks.global.gced_size = usedStack(global);
   LD->stacks.trail.gced_size  = usedStack(trail);
+  gc_status.global_left      += usedStack(global);
+  gc_status.trail_left       += usedStack(trail);
   gc_status.active = FALSE;
 
+#if LIFE_GC == 1
+  SECURE(checkStacks(fr, ch));
+#else
   SECURE(if ( checkStacks(fr, ch) != key )
 	 { sysError("ERROR: Stack checksum failure\n");
 	   trap_gdb();
 	 });
+#endif
 
   if ( verbose )
     printMessage(ATOM_informational,
@@ -2263,7 +2324,9 @@ garbageCollect(LocalFrame fr, Choice ch)
 #endif
 
   unblockGC(PASS_LD1);
+#ifndef UNBLOCKED_GC
   unblockSignals(&mask);
+#endif
   LD->gc.inferences = LD->statistics.inferences;
   leaveGC();
 }
@@ -2742,7 +2805,7 @@ growStacks(LocalFrame fr, Choice ch, Code PC,
   enterGC();				/* atom-gc synchronisation */
   blockSignals(&mask);
   blockGC(PASS_LD1);			/* avoid recursion due to */
-  gc_status.requested = FALSE;		/* printMessage() */
+  PL_clearsig(SIG_GC);
 
   if ( !fr )
     fr = environment_frame;
@@ -3188,4 +3251,7 @@ BeginPredDefs(gc)
 #if O_SECURE || O_DEBUG || defined(O_MAINTENANCE)
   PRED_DEF("$check_stacks", 1, check_stacks, 0)
 #endif
+#ifdef GC_COUNTING
+  PRED_DEF("gc_statistics", 1, gc_statistics, 0)
+#endif
 EndPredDefs
