@@ -5,7 +5,7 @@
     Author:        Jan Wielemaker
     E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2007, University of Amsterdam
+    Copyright (C): 1985-2008, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -241,6 +241,9 @@ typedef struct
     Procedure	dwakeup1;		/* system:$wakeup/1 */
     Procedure	portray_attvar1;	/* $attvar:portray_attvar/1 */
 #endif
+#ifdef O_CALL_RESIDUE
+    Procedure	call_residue_vars2;	/* $attvar:call_residue_vars/2 */
+#endif
 
     SourceFile  reloading;		/* source file we are re-loading */
     int		active_marked;		/* #prodedures marked active */
@@ -305,9 +308,10 @@ typedef struct PL_local_data
 #ifdef O_PLMT
   int		cancel_counter;		/* check cancellation */
 #endif
-  uintptr_t	pending_signals;	/* PL_raise() pending signals */
+  int64_t	pending_signals;	/* PL_raise() pending signals */
   record_t	pending_exception;	/* Pending exception from signal */
   int		current_signal;		/* Currently handled signal */
+  int		sync_signal;		/* Current signal is synchronous */
   int		critical;		/* heap is being modified */
   int		aborted;		/* thread asked for abort */
   Stack		outofstack;		/* thread is out of stack */
@@ -402,10 +406,6 @@ typedef struct PL_local_data
   { intptr_t 	generator;		/* See PL_atom_generator() */
   } atoms;
 
-  struct
-  { struct assoc* bags;			/* findall/setof bags storage */
-  } bags;
-
   struct
   { VarDef *	vardefs;		/* compiler variable analysis */
     int		nvardefs;
@@ -446,6 +446,12 @@ typedef struct PL_local_data
   { FindData	find;			/* /<ports> <goal> in tracer */
   } trace;
 
+#ifdef O_ATOMGC
+  struct
+  { struct findall_bag *bags;		/* Known bags */
+  } bags;
+#endif
+
   struct
   { AbortHandle	_abort_head;		/* PL_abort_hook() */
     AbortHandle _abort_tail;
@@ -489,8 +495,12 @@ typedef struct PL_local_data
     Word *_mark_top;			/* Top of this array */
     Table _check_table;			/* relocation address table */
 #endif
+    int64_t inferences;			/* #inferences at last GC */
 
     pl_gc_status_t	status;		/* Garbage collection status */
+#ifdef O_CALL_RESIDUE
+    int			marked_attvars;	/* do not GC attvars */
+#endif
   } gc;
 
 #ifdef O_SHIFT_STACKS
