@@ -148,6 +148,7 @@ handy for it someone wants to add a data type to the system.
 #define O_ATOMGC		1
 #define O_CLAUSEGC		1
 #define O_ATTVAR		1
+#define O_CALL_RESIDUE		1
 #define O_GVAR			1
 #define O_CYCLIC		1
 #ifdef HAVE_GMP_H
@@ -461,6 +462,8 @@ them.  Descriptions:
 				/* Prolog's integer range */
 #define PLMINTAGGEDINT		(-(intptr_t)((word)1<<(WORDBITSIZE-LMASK_BITS-1)))
 #define PLMAXTAGGEDINT		(-PLMINTAGGEDINT - 1)
+#define PLMINTAGGEDINT32	(-(intptr_t)((word)1<<(32-LMASK_BITS-1)))
+#define PLMAXTAGGEDINT32	(-PLMINTAGGEDINT32 - 1)
 #define inTaggedNumRange(n)	(((n)&~PLMAXTAGGEDINT) == 0 || \
 				 ((n)&~PLMAXTAGGEDINT) == ~PLMAXTAGGEDINT)
 #define PLMININT		(((int64_t)-1<<(INT64BITSIZE-1)))
@@ -1016,6 +1019,7 @@ with one operation, it turns out to be faster as well.
 #define P_THREAD_LOCAL		(0x01000000L) /* predicate */
 #define P_FOREIGN_CREF		(0x02000000L) /* predicate */
 #define P_DIRTYREG		(0x04000000L) /* predicate */
+#define P_ISO			(0x08000000L) /* predicate */
 
 #define ERASED			(0x0001) /* clause, record */
 					 /* clause flags */
@@ -1040,6 +1044,7 @@ with one operation, it turns out to be faster as well.
 #define R_DIRTY			(0x0001) /* recordlist */
 #define R_EXTERNAL		(0x0002) /* record: inline atoms */
 #define R_DUPLICATE		(0x0004) /* record: include references */
+#define R_NOLOCK		(0x0008) /* record: do not lock atoms */
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Handling environment (or local stack) frames.
@@ -1198,7 +1203,7 @@ struct atom
 { Atom		next;		/* next in chain */
   word		atom;		/* as appearing on the global stack */
 #ifdef O_HASHTERM
-  int		hash_value;	/* hash-key value */
+  unsigned int  hash_value;	/* hash-key value */
 #endif
 #ifdef O_ATOMGC
   unsigned int	references;	/* reference-count */
@@ -1512,7 +1517,9 @@ struct queryFrame
   Word	       *aSave;			/* saved argument-stack */
   int		solutions;		/* # of solutions produced */
   Choice	saved_bfr;		/* Saved choice-point */
+#ifndef LIFE_GC
   QueryFrame	parent;			/* Only valid inside GC */
+#endif
   struct choice	choice;			/* First (dummy) choice-point */
   LocalFrame	saved_environment;	/* Parent local-frame */
 					/* Do not put anything between */
@@ -1803,7 +1810,8 @@ Defining built-in predicates using the new interface
 		 *******************************/
 
 #if HAVE_SIGNAL
-#define MAXSIGNAL		32	/* highest system signal number */
+#define MAXSIGNAL		64	/* highest system signal number */
+#define SIG_PROLOG_OFFSET	32	/* Start of Prolog signals */
 
 typedef RETSIGTYPE (*handler_t)(int);
 typedef void *SignalContext;		/* struct sigcontext on sun */
@@ -1816,9 +1824,15 @@ typedef struct
 } sig_handler, *SigHandler;
 #endif /* HAVE_SIGNAL */
 
-#define SIG_EXCEPTION	   9		/* cannot be catched anyway */
-#define SIG_ATOM_GC	  30		/* `safe' and reserved signals */
-#define SIG_THREAD_SIGNAL 31
+#define SIG_EXCEPTION	  (SIG_PROLOG_OFFSET+0)
+#ifdef O_ATOMGC
+#define SIG_ATOM_GC	  (SIG_PROLOG_OFFSET+1)
+#endif
+#define SIG_GC		  (SIG_PROLOG_OFFSET+2)
+#ifdef O_PLMT
+#define SIG_THREAD_SIGNAL (SIG_PROLOG_OFFSET+3)
+#endif
+
 
 		 /*******************************
 		 *	   OPTION LISTS		*
@@ -2026,12 +2040,13 @@ typedef struct redir_context
 		*********************************/
 
 typedef struct
-{ bool		requested;		/* GC is requested by stack expander */
-  int		blocked;		/* GC is blocked now */
+{ int		blocked;		/* GC is blocked now */
   bool		active;			/* Currently running? */
-  intptr_t		collections;		/* # garbage collections */
+  long		collections;		/* # garbage collections */
   int64_t	global_gained;		/* global stack bytes collected */
   int64_t	trail_gained;		/* trail stack bytes collected */
+  int64_t	global_left;		/* global stack bytes left after GC */
+  int64_t	trail_left;		/* trail stack bytes left after GC */
   real		time;			/* time spent in collections */
 } pl_gc_status_t;
 
@@ -2071,6 +2086,7 @@ typedef struct
 #define PROCEDURE_setup_and_call_cleanup4 \
 				(GD->procedures.setup_and_call_cleanup4)
 #define PROCEDURE_dwakeup1		(GD->procedures.dwakeup1)
+#define PROCEDURE_call_residue_vars2	(GD->procedures.call_residue_vars2)
 #define PROCEDURE_dthread_init0		(GD->procedures.dthread_init0)
 #define PROCEDURE_exception_hook4	(GD->procedures.exception_hook4)
 
@@ -2243,6 +2259,7 @@ decrease).
 #include "pl-global.h"			/* global data */
 #include "pl-funcs.h"			/* global functions */
 #include "pl-text.h"			/* text manipulation */
+#include "pl-hash.h"			/* Murmurhash function */
 
 #ifdef __DECC				/* Dec C-compiler: avoid conflicts */
 #undef leave
