@@ -1,4 +1,4 @@
-/*  $Id: pl-itf.c,v 1.42 2005/09/05 19:41:05 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -130,7 +130,7 @@ _PL_term(PL_atomic_t a)
 
 
 PL_atomic_t
-_PL_new_integer(long i)
+_PL_new_integer(intptr_t i)
 { return consInt(i);
 }
 
