@@ -63,7 +63,7 @@ before loading this file.  See end of this file.
 /* PLVERSION: 10000 * <Major> + 100 * <Minor> + <Patch> */
 
 #ifndef PLVERSION
-#define PLVERSION 50655
+#define PLVERSION 50658
 #endif
 
 		 /*******************************
@@ -878,6 +878,8 @@ typedef struct
 
 
 PL_EXPORT(int)	PL_thread_self(void);	/* Prolog thread id (-1 if none) */
+PL_EXPORT(int)  PL_unify_thread_id(term_t t, int i);
+PL_EXPORT(int)	PL_get_thread_id_ex(term_t t, int *idp);
 PL_EXPORT(int)	PL_thread_attach_engine(PL_thread_attr_t *attr);
 PL_EXPORT(int)	PL_thread_destroy_engine(void);
 PL_EXPORT(int)	PL_thread_at_exit(void (*function)(void *),
