@@ -69,7 +69,7 @@ contributions.
 #include <dlfcn.h>
 #endif
 
-#endif /*HAVE_DLOPEN*/
+#else /*HAVE_DLOPEN*/
 
 #ifdef HAVE_SHL_LOAD			/* HPUX */
 
@@ -79,10 +79,11 @@ contributions.
 #define dlerror()	    OsError()
 
 void *
-dlsym(shl_t handle, const char *name)
+dlsym(void *handle, const char *name)
 { void *value;
+  shl_t h = handle;
 
-  if ( shl_findsym(&handle, name, TYPE_PROCEDURE, &value) < 0 )
+  if ( shl_findsym(&h, name, TYPE_PROCEDURE, &value) < 0 )
     return NULL;
 
   return value;
@@ -93,7 +94,8 @@ dlsym(shl_t handle, const char *name)
 #define RTLD_NOW	BIND_IMMEDIATE
 #endif
 
-#endif
+#endif /*HAVE_SHL_LOAD*/
+#endif /*HAVE_DLOPEN*/
 
 #if defined(HAVE_DLOPEN) || defined(HAVE_SHL_LOAD) || defined(EMULATE_DLOPEN)
 
@@ -124,6 +126,30 @@ DlEntry dl_tail;			/* end of this chain */
 #define DL_NOW	  0x1
 #define DL_GLOBAL 0x2
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+under_valgrind()
+
+True if "$VALGRIND" = "yes". It  ensures   dlclose  is  never called. It
+appears that calling dlclose looses source information for loaded shared
+objects.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+static int
+under_valgrind()
+{ const char *v;
+  static int vg = -1;
+
+  if ( vg == -1 )
+  { if ( (v=getenv("VALGRIND")) && streq(v, "yes") )
+      vg = TRUE;
+    else
+      vg = FALSE;
+  }
+
+  return vg;
+}
+
+
 word
 pl_open_shared_object(term_t file, term_t plhandle,
 		      term_t flags)
@@ -149,12 +175,13 @@ pl_open_shared_object(term_t file, term_t plhandle,
 		    ATOM_open, dlerror());
 
   e = allocHeap(sizeof(struct dl_entry));
+
+  LOCK();
   e->id       = ++dl_plid;
   e->dlhandle = dlhandle;
   e->file     = afile;
   e->next     = NULL;
 
-  LOCK();
   if ( !dl_tail )
   { dl_tail = e;
     dl_head = e;
@@ -193,7 +220,8 @@ pl_close_shared_object(term_t plhandle)
 { DlEntry e = find_dl_entry(plhandle);
 
   if ( e && e->dlhandle) 
-  { dlclose(e->dlhandle);
+  { if ( !under_valgrind() )
+      dlclose(e->dlhandle);
     e->dlhandle = NULL;
 
     succeed;
@@ -243,6 +271,32 @@ pl_call_shared_object_function(term_t plhandle, term_t name)
     fail;
 }
 
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Unload all foreign libraries.  As we are doing this at the very end of
+the cleanup, it should be safe now.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+void
+cleanupForeign(void)
+{ DlEntry e, next;
+
+  for(e = dl_head; e; e = next)
+  { next = e->next;
+
+    if ( e->dlhandle )
+    { if ( !under_valgrind() )
+	dlclose(e->dlhandle);
+    }
+
+    freeHeap(e, sizeof(*e));
+  }
+
+  dl_plid = 0;
+  dl_head = dl_tail = NULL;
+}
+
+
 #else /*HAVE_DLOPEN*/
 
 word
