@@ -1,4 +1,4 @@
-/*  $Id: pl-main.h,v 1.27 2006/02/22 12:07:06 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -42,11 +42,11 @@ typedef struct
 } pl_defaults_t; 
 
 typedef struct
-{ long		localSize;		/* size of local stack */
-  long		globalSize;		/* size of global stack */
-  long		trailSize;		/* size of trail stack */
-  long		argumentSize;		/* size of argument stack */
-  long		heapSize;		/* size of the heap */
+{ intptr_t		localSize;		/* size of local stack */
+  intptr_t		globalSize;		/* size of global stack */
+  intptr_t		trailSize;		/* size of trail stack */
+  intptr_t		argumentSize;		/* size of argument stack */
+  intptr_t		heapSize;		/* size of the heap */
   char *	goal;			/* initial goal */
   char *	topLevel;		/* toplevel goal */
   char *	initFile;		/* -f initialisation file */
@@ -79,19 +79,11 @@ typedef struct
 #define OS "unknown"
 #endif
 
-#if O_DYNAMIC_STACKS || O_SHIFT_STACKS
-#define DEF_DEFLOCAL	(500*SIZEOF_LONG)
-#define DEF_DEFGLOBAL	(1000*SIZEOF_LONG)
-#define DEF_DEFTRAIL	(1000*SIZEOF_LONG)
-#define DEF_DEFARGUMENT (250*SIZEOF_LONG)
-#define DEF_DEFHEAP        0		/* unlimited */
-#else					/* static stack areas */
-#define DEF_DEFLOCAL	(50*SIZEOF_LONG)
-#define DEF_DEFGLOBAL	(80*SIZEOF_LONG)
-#define DEF_DEFTRAIL	(80*SIZEOF_LONG)
-#define DEF_DEFARGUMENT	(20*SIZEOF_LONG)
-#define DEF_DEFHEAP       0		/* unlimited */
-#endif
+#define DEF_DEFLOCAL	(4096*SIZEOF_VOIDP)
+#define DEF_DEFGLOBAL	(8192*SIZEOF_VOIDP) /* 32MB on 32-bit hardware */
+#define DEF_DEFTRAIL	(8192*SIZEOF_VOIDP)
+#define DEF_DEFARGUMENT (250*SIZEOF_VOIDP)
+#define DEF_DEFHEAP     0		/* unlimited */
 
 #ifndef DEFLOCAL
 #define DEFLOCAL    DEF_DEFLOCAL
