@@ -700,7 +700,7 @@ pl_export_list(term_t modulename, term_t public)
     LOCKMODULE(module);
     for_table(module->public, s,
 	      { if ( !PL_unify_list(list, head, list) ||
-		     !PL_unify_functor(head, (functor_t)s->name) )
+		     !unify_functor(head, (functor_t)s->name, GP_NAMEARITY) )
 		{ rval = FALSE;
 		  break;
 		}
@@ -720,17 +720,29 @@ head from the context module.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static int
-export_pi(term_t pi)
+export_pi(term_t pi, Module module ARG_LD)
 { functor_t fd;
   Procedure proc;
-  Module module = NULL;
+
+  PL_strip_module(pi, &module, pi);
+
+  if ( PL_is_functor(pi, FUNCTOR_comma2) )
+  { term_t a1 = PL_new_term_ref();
+    term_t a2 = PL_new_term_ref();
+
+    _PL_get_arg(1, pi, a1);
+    _PL_get_arg(2, pi, a2);
+
+    TRY(export_pi(a1, module PASS_LD));
+    return export_pi(a2, module PASS_LD);
+  }
+
 
   if ( !get_functor(pi, &fd, &module, 0, GF_PROCEDURE) )
     fail;
 
-  if ( (proc = isStaticSystemProcedure(fd)) )
-    return PL_error(NULL, 0, NULL, ERR_PERMISSION_PROC,
-		    ATOM_export, ATOM_built_in_procedure, proc->definition);
+  if ( (proc = isStaticSystemProcedure(fd)) && true(proc->definition, P_ISO) )
+    succeed;
   proc = lookupProcedure(fd, module);
 
   LOCKMODULE(module);
@@ -747,18 +759,9 @@ export_pi(term_t pi)
 static
 PRED_IMPL("export", 1, export, PL_FA_TRANSPARENT)
 { PRED_LD
-  term_t decl = PL_copy_term_ref(A1);
-  term_t pi = PL_new_term_ref();
+  Module module = NULL;
 
-  while ( PL_is_functor(decl, FUNCTOR_comma2) )
-  { _PL_get_arg(1, decl, pi);
-    _PL_get_arg(2, decl, decl);
-    
-    if ( !export_pi(decl) )
-      fail;
-  }
-  
-  return export_pi(decl);
+  return export_pi(A1, module PASS_LD);
 }
 
 
@@ -770,15 +773,17 @@ pl_check_export()
   for_table(module->public, s,
 	    { Procedure proc = (Procedure) s->value;
 	      Definition def = proc->definition;
+	      FunctorDef fd = def->functor;
 
-	      if ( !isDefinedProcedure(proc) && /* not defined */
-		   proc->definition->module == module ) /* not imported */
+	      if ( !isDefinedProcedure(proc) &&			/* not defined */
+		   def->module == module &&			/* not imported */
+		   !autoImport(fd->functor, module) )
 	      { printMessage(ATOM_error,
 			     PL_FUNCTOR_CHARS, "undefined_export", 2,
 			       PL_ATOM, module->name,
 			       PL_FUNCTOR, FUNCTOR_divide2,
-			         PL_ATOM, def->functor->name,
-			         PL_INT, (int)def->functor->arity);
+			         PL_ATOM, fd->name,
+			         PL_INT, fd->arity);
 	      }
 	    })
 
@@ -838,13 +843,12 @@ pl_import(term_t pred)
 { GET_LD
   Module source = NULL;
   Module destination = contextModule(environment_frame);
-  term_t head = PL_new_term_ref();
   functor_t fd;
   Procedure proc, old;
 
-  PL_strip_module(pred, &source, head);
-  if ( !PL_get_functor(head, &fd) )
-    return warning("import/1: instantiation fault");
+  if ( !get_functor(pred, &fd, &source, 0, GF_PROCEDURE) )
+    fail;
+
   proc = lookupProcedure(fd, source);
 
   if ( !isDefinedProcedure(proc) )
