@@ -2593,24 +2593,12 @@ System(char *cmd)
   int rval;
   void (*old_int)();
   void (*old_stop)();
-  unsigned char fds[256];
-  int nfds = openFileDescriptors(fds, sizeof(fds));
 
-  Setenv("PROLOGCHILD", "yes");
-
-  if ( (pid = vfork()) == -1 )
+  if ( (pid = fork()) == -1 )
   { return PL_error("shell", 2, OsError(), ERR_SYSCALL, "fork");
   } else if ( pid == 0 )		/* The child */
-  { int i;
-
-    for(i = 0; i < nfds; i++)
-    { int fd = fds[i];
-
-      if ( fd >= 3 )
-	close(fd);
-    }
-    stopItimer();
-
+  { Setenv("PROLOGCHILD", "yes");
+    PL_cleanup_fork();
     execl(shell, BaseName(shell), "-c", cmd, (char *)0);
     fatalError("Failed to execute %s: %s", shell, OsError());
     fail;
@@ -2816,6 +2804,23 @@ char *command;
 
 #endif
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+[candidate]
+
+exec(+Cmd, [+In, +Out, +Error], -Pid)
+
+The streams may be one of standard   stream,  std, null stream, null, or
+pipe(S), where S is a pipe stream
+
+Detach if none is std!
+
+TBD: Sort out status. The above is SICStus 3. YAP uses `Status' for last
+argument (strange). SICStus 4 appears to drop this altogether.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     char *Symbols(char *buf)
 
