@@ -249,7 +249,6 @@ pl_abort(abort_type type)
   resetSignals();
   resetForeign();
   resetAtoms();
-  resetBags();
 
   longjmp(abort_context, 1);
   /*NOTREACHED*/
@@ -275,7 +274,6 @@ resetProlog()
 #endif
 
   gc_status.blocked    = 0;
-  gc_status.requested  = FALSE;
 #if O_SHIFT_STACKS
   LD->shift_status.blocked = 0;
 #endif
