@@ -243,7 +243,8 @@ static int
 checkModifySystemProc(functor_t fd)
 { Procedure proc;
 
-  if ( (proc = isStaticSystemProcedure(fd)) )
+  if ( (proc = isStaticSystemProcedure(fd)) &&
+       true(proc->definition, P_ISO) )
     return PL_error(NULL, 0, NULL, ERR_MODIFY_STATIC_PROC, proc);
 
   succeed;
@@ -622,17 +623,19 @@ pl_current_predicate1(term_t spec, control_t ctx)
       if ( mt )
       { atom_t mname;
 
-	if ( PL_get_atom(mt, &mname) )
-	{ e->module = isCurrentModule(mname);
-	  if ( !e->module )
-	    fail;
-	} else if ( PL_is_variable(mt) )
+	if ( PL_is_variable(mt) )
 	{ e->emod = newTableEnum(GD->tables.modules);
 
 	  if ( (sm = advanceTableEnum(e->emod)) )
 	    e->module = sm->value;
 	  else
 	    fail;			/* no modules!? */
+	} else if ( PL_get_atom_ex(mt, &mname) )
+	{ e->module = isCurrentModule(mname);
+	  if ( !e->module )
+	    fail;
+	} else
+	{ fail;
 	}
       } else
       { if ( environment_frame )
@@ -1419,7 +1422,7 @@ Definition
 autoImport(functor_t f, Module m)
 { GET_LD
   Procedure proc;
-  Definition def;
+  Definition def, odef;
   ListCell c;
 					/* Defined: no problem */
   if ( (proc = isCurrentProcedure(f, m)) && isDefinedProcedure(proc) )
@@ -1436,18 +1439,20 @@ found:
     proc = lookupProcedure(f, m);
 					/* Safe? See above */
 					/* TBD: find something better! */
+  odef = proc->definition;
+  proc->definition = def;
+
 #ifdef O_PLMT
   PL_LOCK(L_THREAD);
   if ( (GD->statistics.threads_created -
 	GD->statistics.threads_finished) == 1 )
   { assert(false(proc->definition, P_DIRTYREG));
-    freeHeap(proc->definition, sizeof(struct definition));
+    freeHeap(odef, sizeof(struct definition));
   }
   PL_UNLOCK(L_THREAD);
 #else
-  freeHeap(proc->definition, sizeof(struct definition));
+  freeHeap(odef, sizeof(struct definition));
 #endif
-  proc->definition = def;
 
   return def;
 }
@@ -1892,6 +1897,7 @@ attribute_mask(atom_t key)
   if (key == ATOM_volatile)	 return VOLATILE;
   if (key == ATOM_thread_local)  return P_THREAD_LOCAL;
   if (key == ATOM_noprofile)     return P_NOPROFILE;
+  if (key == ATOM_iso)      	 return P_ISO;
 
   return 0;
 }
@@ -2482,7 +2488,7 @@ addProcedureSourceFile(SourceFile sf, Procedure proc)
 
 
 void
-redefineProcedure(Procedure proc, SourceFile sf)
+redefineProcedure(Procedure proc, SourceFile sf, unsigned int suppress)
 { GET_LD
   Definition def = proc->definition;
 
@@ -2499,7 +2505,7 @@ redefineProcedure(Procedure proc, SourceFile sf)
   { ClauseRef first = hasClausesDefinition(def);
 
     if ( first && first->clause->source_no == sf->index )
-    { if ( (debugstatus.styleCheck & DISCONTIGUOUS_STYLE) &&
+    { if ( ((debugstatus.styleCheck & ~suppress) & DISCONTIGUOUS_STYLE) &&
 	   false(def, DISCONTIGUOUS) )
 	printMessage(ATOM_warning,
 		     PL_FUNCTOR_CHARS, "discontiguous", 1,
