@@ -44,7 +44,7 @@ static void	addUTF8Buffer(Buffer b, int c);
 		 *******************************/
 
 #define CharTypeW(c, t, w) \
-	((unsigned)(c) <= 0xff ? (_PL_char_types[(unsigned char)(c)] t) \
+	((unsigned)(c) <= 0xff ? (_PL_char_types[(unsigned)(c)] t) \
 			       : (uflagsW(c) & w))
 
 #define PlBlankW(c)	CharTypeW(c, <= SP, U_SEPARATOR)
@@ -56,6 +56,11 @@ static void	addUTF8Buffer(Buffer b, int c);
 #define PlPunctW(c)	CharTypeW(c, == PU, 0)
 #define PlSoloW(c)	CharTypeW(c, == SO, 0)
 
+int
+unicode_separator(pl_wchar_t c)
+{ return PlBlankW(c);
+}
+
 
 		 /*******************************
 		 *	   CHAR-CONVERSION	*
@@ -694,6 +699,17 @@ add_comment(Buffer b, IOPOS *pos, ReadData _PL_rd ARG_LD)
 }
 
 
+static void
+setErrorLocation(IOPOS *pos, ReadData _PL_rd)
+{ GET_LD
+
+  source_char_no = pos->charno;
+  source_line_pos = pos->linepos;
+  source_line_no = pos->lineno;
+  rb.here = rb.base+1;			/* see rawSyntaxError() */
+}
+
+
 static unsigned char *
 raw_read2(ReadData _PL_rd ARG_LD)
 { int c;
@@ -740,7 +756,7 @@ raw_read2(ReadData _PL_rd ARG_LD)
 		strcpy((char *)rb.base, "end_of_file. ");
 		rb.here = rb.base + 14;
 		return rb.base;
-      case '/': if ( _PL_rd->comments && rb.stream->position )
+      case '/': if ( rb.stream->position )
 		{ pbuf = *rb.stream->position;
 		  pbuf.charno--;
 		  pbuf.linepos--;
@@ -767,6 +783,7 @@ raw_read2(ReadData _PL_rd ARG_LD)
 		  if ((last = getchr()) == EOF)
 		  { if ( cbuf )
 		      discardBuffer(cbuf);
+		    setErrorLocation(pos, _PL_rd);
 		    rawSyntaxError("end_of_file_in_block_comment");
 		  }
 		  if ( cbuf )
@@ -788,6 +805,7 @@ raw_read2(ReadData _PL_rd ARG_LD)
 		    { case EOF:
 			if ( cbuf )
 			  discardBuffer(cbuf);
+		        setErrorLocation(pos, _PL_rd);
 			rawSyntaxError("end_of_file_in_block_comment");
 		      case '*':
 			if ( last == '/' )
@@ -1104,8 +1122,13 @@ warn_singleton(const char *name)	/* Name in UTF-8 */
     return TRUE;
   if ( name[1] == '_' )			/* __*: never warn */
     return FALSE;
-  if ( name[1] && !PlUpperW(name[1]) )	/* _a: warn */
-    return TRUE;
+  if ( name[1] )			/* _a: warn */
+  { int c;
+
+    utf8_get_char(&name[1], &c);
+    if ( !PlUpperW(c) )
+      return TRUE;
+  }
   return FALSE;
 }
 
@@ -2824,6 +2847,9 @@ pl_raw_read2(term_t from, term_t term)
   t2 = backSkipUTF8(s, top, &chr);
   if ( chr == '.' )
     top = backSkipBlanks(s, t2);
+					/* watch for "0' ." */
+  if ( top < e && top-2 >= s && top[-1] == '\'' && top[-2] == '0' )
+    top++;
   *top = EOS;
   s = skipSpaces(s);
   
@@ -3099,7 +3125,7 @@ atom_to_term(term_t atom, term_t term, term_t bindings)
     stream = Sopen_text(&txt, "r");
 
     init_read_data(&rd, stream PASS_LD);
-    if ( PL_is_variable(bindings) || PL_is_list(bindings) )
+    if ( bindings && (PL_is_variable(bindings) || PL_is_list(bindings)) )
       rd.varnames = bindings;
     else if ( bindings )
       return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_list, bindings);
