@@ -3,9 +3,9 @@
     Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        J.Wielemaker@uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2008, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,9 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#define bool pl_bool			/* avoid conflict with curses */
 #include "pl-incl.h"
+#undef bool
 
 #define LOCK()   PL_LOCK(L_TERM)
 #define UNLOCK() PL_UNLOCK(L_TERM)
@@ -42,12 +44,9 @@ windowing!
 #include <sys/ioctl.h>
 #endif
 
-extern int  tgetent();
-extern int  tgetnum();
-extern int  tgetflag();
-extern char *tgetstr();
-extern char *tgoto();
-extern int  tputs();
+#undef clear				/* conflicts */
+#include <curses.h>
+#include <term.h>
 
 #define MAX_TERMBUF	1024		/* Confirming manual */
 #define STAT_START	0
@@ -90,7 +89,7 @@ resetTerm()
 /* MT: Locked by calling lookupEntry()
 */
 
-static bool
+static int
 initTerm(void)
 { if ( term_initialised == STAT_START )
   { char term[100];
@@ -148,7 +147,7 @@ lookupEntry(atom_t name, atom_t type)
 
   LOCK();
   if ( (s = lookupHTable(capabilities, (void*)name)) == NULL )
-  { if ( initTerm() == FALSE )
+  { if ( !initTerm() )
     { e = NULL;
       goto out;
     }
@@ -164,7 +163,7 @@ lookupEntry(atom_t name, atom_t type)
       if ( (n = tgetnum(stringAtom(name))) != -1 )
         e->value  = consInt(n);
     } else if ( type == ATOM_bool )
-    { bool b;
+    { int b;
     
       if ( (b = tgetflag(stringAtom(name))) != -1 )
         e->value = (b ? ATOM_on : ATOM_off);
