@@ -54,7 +54,6 @@ typedef struct
 	  (txt)->canonical = FALSE; \
 	}
 
-int	PL_get_text(term_t l, PL_chars_t *text, int flags);
 int	PL_unify_text(term_t term, term_t tail, PL_chars_t *text, int type);
 int	PL_unify_text_range(term_t term, PL_chars_t *text,
 			    size_t from, size_t len, int type);
@@ -71,6 +70,7 @@ int	PL_concat_text(int n, PL_chars_t **text, PL_chars_t *result);
 void	PL_free_text(PL_chars_t *text);
 void	PL_save_text(PL_chars_t *text, int flags);
 
+COMMON(int)		PL_get_text__LD(term_t l, PL_chars_t *text, int flags ARG_LD);
 COMMON(atom_t)		textToAtom(PL_chars_t *text);
 
 COMMON(IOSTREAM *)	Sopen_text(PL_chars_t *text, const char *mode);
