@@ -1,4 +1,4 @@
-/*  $Id: util.c,v 1.7 2004/10/15 08:48:05 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -69,7 +69,7 @@ strerror(int e)
 const char *
 rc_strerror(int e)
 {
-#ifdef WIN32
+#ifdef __WINDOWS__
   if ( e == RCE_WINERRNO )
   {					/* TBD */
   }
