@@ -881,6 +881,8 @@ cleanup(findall-2) :-
 cleanup(findall-3) :-
 	catch(findall(X, genex3(X), _), E, true),
 	ex(Ex), E == Ex.
+cleanup(gc) :-
+	garbage_collect.
 
 
 		 /*******************************
@@ -1542,6 +1544,13 @@ catchme :-
 undef :-
 	'this is not defined'.
 
+tcatch :-
+	catch(ierror, E, throw(ok(E))).
+
+ierror :-
+	garbage_collect,
+	atom_codes(_,_).
+
 exception(call-1) :-
 	catch(do_exception_1, E, true),
 	error(E, instantiation_error).
@@ -1562,6 +1571,9 @@ exception(context-1) :-
 exception(context-2) :-
 	catch(undef, E, true),
 	error_pred(E, undef/0).
+exception(catch-gc) :-
+	catch(tcatch, E, true),
+	subsumes_chk(ok(error(_,_)), E).
 
 
 		 /*******************************
@@ -1578,9 +1590,13 @@ local_overflow :-
 global_overflow(X) :-			% Causes gracefully signalled overflow
 	global_overflow(s(X)).
 
-string_overflow([H|T]) :-		% Causes PL_throw() overflow
+string_overflow(StringList) :-
+	string_overflow2(StringList),
+	is_list(StringList).		% avoid GC of list
+
+string_overflow2([H|T]) :-		% Causes PL_throw() overflow
 	format(string(H), '~txx~1000000|', []),
-	string_overflow(T).
+	string_overflow2(T).
 		
 
 resource(stack-1) :-
