@@ -1,4 +1,4 @@
-/*  $Id: history.c,v 1.6 2004/11/30 14:23:33 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -81,7 +81,7 @@ rlc_add_history(rlc_console c, const TCHAR *line)
 
   if ( b->history.size )
   { int i = next(b, b->history.head);
-    int len = _tcslen(line);
+    size_t len = _tcslen(line);
 
     while(*line && *line <= ' ')	/* strip leading white-space */
       line++;
